#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#
# Copyright 2012-2014 Frédéric Magniette, Miguel Rubio-Roy, LLR
#

import os,time,random,penv

env=penv.penv()

#**************** ACQPC **********************

def init_acqpc(acqpc_id,datadir,convert_script,stream_suffix,media,media_path,media_speed,replay):
    if not (os.path.isfile(convert_script) and os.access(convert_script,os.X_OK)):
        return 0,"invalid convert_script: file not exists or not executable"
    env["name"]=acqpc_id
    env["datadir"]=datadir
    env["stream_suffix"]=stream_suffix
    env["media"]=media
    env["media_path"]=media_path
    env["media_speed"]=media_speed
    env["convert_script"]=convert_script
    env["replay"]=replay
    return 1,"ok"

def init_fin_acqpc(acqpc_id):

    #get varmod ip
    retcode,res=submod_execcmd("get_ip@cmod","storage")
    if retcode==0:
        return 0,"cant get storage ip <- %s" % (res)
    env["storage_ip"]=res

    #get list of GDCCs
    retcode,res=submod_execcmd("get_name_sublist@cmod","gdcc",acqpc_id)
    if retcode==0:
        return 0,"error while getting GDCCs of ACQPC %s <- %s"%(env["name"],res)
    if res=="":
        gdcc_ids=[]
    else:
        gdcc_ids=res.split(",")

    #check unicity of GDCC network interface
    acq_inters=[]
    for gdcc_id in gdcc_ids:
        retcode,res=submod_execcmd("get_param@cmod",gdcc_id,"pc_dev")
        if retcode==0:
            return 0,"error while getting pc_dev of gdcc_id %s <- %s"%(gdcc_id,res)
        if res not in acq_inters:
            acq_inters.append(res)
    if len(acq_inters)>1:
        return 0,"too many NICs"

    #get list of difs
    retcode,res=submod_execcmd("get_name_sublist@cmod","dif",acqpc_id)
    if retcode==0:
        return 0,"error while getting DIFs of acqpc_id %s <- %s"%(acqpc_id,res)
    if res=="":
        dif_list=[]
    else:
        dif_list=res.split(",")
    dif_list.sort()
    total_nb_difs=len(dif_list)
    print("total nb of difs=%d"%(total_nb_difs))

    #add the dif acqunit
    if env["media"]=="ethernet":
        retcode,res=submod_execcmd("init@acq","ecal_dif",str(total_nb_difs),"/opt/pyrame/eth_acq.so","/opt/calicoes/uncap_dif4_1.so",acq_inters[0],"null","null","100000",env["replay"],env["datadir"],env["convert_script"])
        if retcode==0:
            return 0,"cant add acq_unit <- %s" % (res)
    elif env["media"]=="wagasci":
        retcode,res=submod_execcmd("init@acq","ecal_dif",str(total_nb_difs),"/opt/pyrame/eth_acq.so","/opt/calicoes/uncap_wagasci.so",acq_inters[0],"null","null","100000",env["replay"],env["datadir"],env["convert_script"])
        if retcode==0:
            return 0,"cant add acq_unit <- %s" % (res)
    elif env["media"]=="udp":
        retcode,res=submod_execcmd("init@acq","ecal_dif",str(total_nb_difs),"/opt/pyrame/udp_multiport_acq.so","/opt/calicoes/uncap_dif_udp.so","2064:2065","null","null","100000",env["replay"],env["datadir"],env["convert_script"])
        if retcode==0:
            return 0,"cant add acq_unit <- %s" % (res)
    elif env["media"]=="run_ref":
        retcode,res=submod_execcmd("init@acq","ecal_dif",str(total_nb_difs),"/opt/pyrame/file_acq.so","/opt/calicoes/uncap_dif4_1.so",env["media_path"],env["media_speed"],"null","100000","0",env["datadir"],env["convert_script"])
        if retcode==0:
            return 0,"cant add acq_unit <- %s" % (res)
    elif env["media"]=="wagasci_replay":
        retcode,res=submod_execcmd("init@acq","ecal_dif",str(total_nb_difs),"/opt/pyrame/file_acq.so","/opt/calicoes/uncap_wagasci.so",env["media_path"],env["media_speed"],"null","100000","0",env["datadir"],env["convert_script"])
        if retcode==0:
            return 0,"cant add acq_unit <- %s" % (res)
    elif env["media"]=="inject":
        retcode,res=submod_execcmd("init@acq","ecal_dif",str(total_nb_difs),"/opt/pyrame/tcps_acq.so","/opt/calicoes/uncap_dif4_1.so","8003","null","null","100000","0",env["datadir"],env["convert_script"])
        if retcode==0:
            return 0,"cant add acq_unit <- %s" % (res)
    elif env["media"]=="test":
        retcode,res=submod_execcmd("init@acq","ecal_dif","3","/opt/pyrame/file_acq.so","/opt/pyrame/uncap_dummy.so","/opt/calicoes/test.raweth","3000000","null","100000","0",env["datadir"],env["convert_script"])
        if retcode==0:
            return 0,"cant add acq_unit <- %s" % (res)
    else:
        return 0,"bad acqpc_media parameter: %s"%(env["media"])
    time.sleep(0.5)

    #launch CD
    for dif in range(total_nb_difs):
        retcode,res=submod_execcmd("start_converter@dif",dif_list[dif])
        if retcode==0:
            return 0,"cant start converter <- %s" % (res)
    
    #start acquisition
    if env["media"] not in ["run_ref","test"]:
        retcode,res=submod_execcmd("start@acq","ecal_dif")
        if retcode==0:
            return 0,"cant start acq <- %s" % (res)
    return 1,"ok"

def deinit_acqpc(acqpc_id):
    retcode,res=submod_execcmd("deinit@acq","ecal_dif")
    if retcode==0:
        return 0,"cant deinit acqunit<- %s" % (res)
    return 1,"ok"

def start_acq_acqpc(acqpc_id,acq_id,acq_name,acq_mp,acq_path):
    env["acq_id"]=acq_id
    env["acq_name"]=acq_name
    env["acq_mp"]=acq_mp
    env["acq_path"]=acq_path

    #flush acquisition chain that contains old control packets
    if env["media"] not in ["run_ref","test"]:
        #in fact the acquisition is already started for control packets
        retcode,res=submod_execcmd("stop@acq","ecal_dif")
        if retcode==0:
            return 0,"cant stop acq <- %s"%(res)
    retcode,res=submod_execcmd("flush_files@acq","ecal_dif","crap","file:///","/tmp/","crap","undef","undef","undef")
    if retcode==0:
        return 0,"cant flush acq files <- %s"%(res)

    #reinit the stats
    retcode,res=submod_execcmd("zero_stats@acq")
    if retcode==0:
        return 0,"cant zero acq stats <- %s"%(res)

    #start acquisition to get the data
    retcode,res=submod_execcmd("start@acq","ecal_dif")
    if retcode==0:
        return 0,"cant start acq <- %s"%(res)
    return 1,"ok"

def stop_acq_acqpc(acqpc_id):
    #stop data acquisition
    retcode,res=submod_execcmd("stop@acq","ecal_dif")
    if retcode==0:
        return 0,"cant stop acq <- %s"%(res)
  
    #flush files with convert script that moves from
    #acqpc_datadir/flush_prefix to its second parameter complete_prefix
    flush_prefix="%x"%(random.getrandbits(16))
    retcode,res=submod_execcmd("flush_files@acq","ecal_dif",flush_prefix,env["acq_mp"],env["acq_path"],env["acq_name"],env["acq_id"],env["storage_ip"],"storage")
    if retcode==0:
        return 0,"cant flush acq files <- %s"%(res)

     #start acq for control packets if necessary
    if env["media"] not in ["run_ref","test"]:
        retcode,res=submod_execcmd("start@acq","ecal_dif")
        if retcode==0:
            return 0,"cant start acq <- %s"%(res)
    return 1,"ok"
