#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#
# Copyright 2012-2015 Frédéric Magniette, Miguel Rubio-Roy, LLR
#

import pools

asu_pool = pools.pool("asu")

def init_asu(asu_id,version,missing_rocs,id):
    "Initialize an active sensor unit (ASU)"
    #parse missing_rocs
    if missing_rocs not in ["","undef"]:
        try:
            missing_rocs_int = map(int,missing_rocs.split(","))
        except Exception as e:
            return 0,"invalid format of asu_missing_rocs: %s" % (str(e))

    else:
        missing_rocs_int = []
        missing_rocs=""
    #determine roctype
    version=version.lower()
    minrocchipid=0
    maxrocchipid=0
    if version=="fev8":
        roctype="skiroc"
        maxrocchipid=3
    elif version=="fev8_cob":
        roctype="skiroc"
        maxrochipid=7
    elif version in ["fev9","fev10","fev11"]:
        roctype="skiroc"
        maxrocchipid=15
    elif version=="eval_skiroc":
        roctype="skiroc"
    elif version=="eval_skiroc2cms":
        roctype="skiroc2cms"
    elif version=="eval_easiroc":
        roctype="easiroc"
    elif version=="eval_spiroc2b":
        roctype="spiroc2b"
    elif version=="eval_spiroc2d":
        roctype="spiroc2d"
    elif version=="eval_maroc3":
        roctype="maroc3"
    else:
        return 0,"invalid asu version %s for %s"%(version,asu_id)
    if not all(rocchipid<=maxrocchipid and rocchipid>=minrocchipid for rocchipid in missing_rocs_int):
        return 0,"invalid chipid in asu_missing_rocs"
    #chipid will be composed of 4 bits for ASU (asuchipid) and 4 bits for ROC (rocchipid)
    asu_pool.new(asu_id,{"name":asu_id,"version":version,"roctype":roctype,"asuchipid":int(id),"missing_rocs":missing_rocs,"nb_missing":len(missing_rocs_int)})
    return 1,"ok"

def init_fin_asu(asu_id):
    "Finish initialization of ASU"
    try:
        asu = asu_pool.get(asu_id)
    except Exception as e:
        return 0,str(e)
    set_param = True
    if asu["version"]=="fev8":
        asu_nb_rocs = 4
    elif asu["version"]=="fev8_cob":
        asu_nb_rocs = 8
    elif asu["version"] in ["fev9","fev10","fev11"]:
        asu_nb_rocs = 16
    elif asu["version"] in ["eval_skiroc","eval_skiroc2cms"]:
        asu_nb_rocs = 1
    elif asu["version"]=="eval_easiroc":
        asu_nb_rocs = 1
        set_param = False
    elif asu["version"] in ["eval_spiroc2b","eval_spiroc2d"]:
        asu_nb_rocs = 1
    elif asu["version"]=="eval_maroc3":
        asu_nb_rocs = 1
        set_param = False
    else:
        return 0,"invalid asu version %s for %s"%(asu["version"],asu["name"])
    retcode,res=submod_execcmd("get_name_sublist@cmod",asu["roctype"],asu["name"])
    if retcode==0 or res=="":
        return 0,"error while getting rocs of asu %s <- %s"%(asu["name"],res)
    rocs=res.split(",")
    asu["rocs"]=res
    asu["nb_rocs"]=len(rocs)
    if len(rocs)!=asu_nb_rocs:
        return 0,"ASU %s (version %s) must have %d rocs of type %s"%(asu["name"],asu["version"],asu_nb_rocs,asu["roctype"])
    rocchipid=0
    for roc in rocs:
        if set_param:
            chipid=asu["asuchipid"]*asu_nb_rocs+rocchipid
            retcode,res=submod_execcmd("set_chipid_param@"+asu["roctype"],roc,str(chipid))
            if retcode==0:
                return 0,"error setting chipid parameter of %s %s <- %s"%(asu["roctype"],roc,res)
        asu["rocchipid_%d"%(rocchipid)]=roc
        rocchipid+=1
    if asu["missing_rocs"]!="":
        for missing_rocchipid in asu["missing_rocs"].split(","):
            roc=asu["rocchipid_%s"%(missing_rocchipid)]
            retcode,res=submod_execcmd("set_missing@"+asu["roctype"],roc,"1")
            if retcode==0:
                return 0,"can't set roc with rocchipid %s missing <- %s"%(missing_rocchipid,res)
    return 1,"ok"

def deinit_asu(asu_id):
    "Deinitialize an ASU"
    try:
        asu = asu_pool.get(asu_id)
    except Exception as e:
        return 1,str(e)
    asu_pool.remove(asu_id)
    return 1,"ok"

def dump_sc_asu(asu_id):
    "Get the slow control for this ASU"
    try:
        asu = asu_pool.get(asu_id)
    except Exception as e:
        return 0,str(e)
    sc = ""
    if asu["version"] in ["eval_spiroc2b","eval_spiroc2d","eval_maroc3","eval_easiroc","eval_skiroc","eval_skiroc2cms"]:
        chipid_list = [0]
    elif asu["version"] in ["fev9","fev10","fev11"]:
        chipid_list = [9,11,13,15,-1,14,12,10,8,-1,1,3,5,7,-1,6,4,2,0]
    elif asu["version"]=="fev8_cob":
        chipid_list = [1,3,5,7,-1,6,4,2,0]
    elif asu["version"]=="fev8":
        chipid_list = [1,3,-1,2,0]
    else:
        return 0,"asu : unknown asu type %s"%(asu["version"])
    for rocchipid in chipid_list:
        if rocchipid==-1:
            sc += "%"
        else:
            roc=asu["rocchipid_%d"%(rocchipid)]
            retcode,res=submod_execcmd("dump_sc@"+asu["roctype"],roc)
            if retcode==0:
                return 0,"can't get sc from roc %s <- %s"%(roc,res)
            sc += res
    return 1,sc

def get_roc_type_asu(asu_id):
    try:
        asu = asu_pool.get(asu_id)
    except Exception as e:
        return 0,str(e)
    return 1,asu["roctype"]

def get_nb_active_rocs_asu(asu_id):
    try:
        asu = asu_pool.get(asu_id)
    except Exception as e:
        return 0,str(e)
    nb_active_rocs=asu["nb_rocs"]-asu["nb_missing"]
    return 1,str(nb_active_rocs)

def get_nb_rocs_asu(asu_id):
    try:
        asu = asu_pool.get(asu_id)
    except Exception as e:
        return 0,str(e)
    return 1,str(asu["nb_rocs"])
