#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#
# Copyright 2012-2014 Frédéric Magniette, Miguel Rubio-Roy, LLR
#

import pools

ccc_pool=pools.pool("ccc")

def init_ccc(ccc_id,mac_addr,pc_dev):
    "Initialize a ccc"
    ccc_pool.new({"name":ccc_id,"mac_addr":mac_addr,"pc_dev":pc_dev})
    return 1,"ok"

def deinit_ccc(ccc_id):
    "Deinitialize ccc *ccc_id*"
    try:
        ccc=ccc_pool.get(ccc_id)
    except Exception as e:
        return 1,str(e)
    # Remove ccc from the pool
    ccc_pool.remove(ccc_id)
    return 1,"ok"

def send_char(ccc_id,char):
    try:
        ccc=ccc_pool.get(ccc_id)
    except Exception as e:
        return 0,str(e)
    if len(char)!=1:
        return 0,"FC code must be one character"
    try:
        int(char,16)
    except:
        return 0,"invalid FC code"
    retcode,res=submod_execcmd("send_packet@raweth",ccc["pc_dev"],ccc["mac_addr"],"0x809","0x"+char+",0x00"*49)
    if retcode==0:
        return 0,"<- %s"%(res)
    return 1,"ok"

def send_char_ccc(ccc_id,char):
    "Send an arbitrary Fast Command to the CCC"
    retcode,res=send_char(ccc_id,char)
    if retcode==0:
        return 0,"Error sending FC %s to CCC %s"%(char,res)
    return 1,"ok"

def trigger_pulse_ccc(ccc_id):
    "Send trigger pulse Fast Command to the CCC"
    retcode,res=send_char(ccc_id,"A")
    if retcode==0:
        return 0,"Error sending trigger pulse (A) FC to CCC %s"%(res)
    return 1,"ok"

def reset_ccc(ccc_id):
    "Send reset Fast Command to the CCC"
    retcode,res=send_char(ccc_id,"21")
    if retcode==0:
        return 0,"Error sending reset (0x21) FC to CCC %s"%(res)
    return 1,"ok"

def start_acq_ccc(ccc_id):
    "Send start acquisition Fast Command to the CCC"
    retcode,res=send_char(ccc_id,"42")
    if retcode==0:
        return 0,"Error sending start acquisition (0x42) FC to CCC %s"%(res)
    return 1,"ok"

def stop_acq_ccc(ccc_id):
    "Send stop acquisition Fast Command to the CCC"
    retcode,res=send_char(ccc_id,"62")
    if retcode==0:
        return 0,"Error sending stop acquisition (0x62) FC to CCC %s"%(res)
    return 1,"ok"

def pause_trig_ccc(ccc_id):
    "Send stop acquisition Fast Command to the CCC"
    retcode,res=send_char(ccc_id,"3")
    if retcode==0:
        return 0,"Error sending pause / toggle trigger veto (3) FC to CCC %s"%(res)
    return 1,"ok"

def resume_trig_ccc(ccc_id):
    "Send stop acquisition Fast Command to the CCC"
    retcode,res=send_char(ccc_id,"4")
    if retcode==0:
        return 0,"Error sending resume (4) FC to CCC %s"%(res)
    return 1,"ok"
