#!/usr/bin/env python2
# -*- coding: utf-8 -*-
#
# Copyright 2012-2015 Frédéric Magniette, Miguel Rubio-Roy, LLR
#

import sys,time,os,base64,datetime,penv
sys.path.append("/opt/calicoes")
sys.path.append("/opt/pyrame")
env=penv.penv()

#constants
stats_list=["nb_data_pkts","nb_lost_pkts","bytes_on_dsk","nb_spills","cur_spill","nb_evts","nb_hits"]

def init():
    env["state"]="UNDEFINED"

#**************************** INTERNAL FUNCTIONS ***********************

def find_n_act_signal(signame,action):
    """start or stop a signal
    signame : name of the signal
    action : start or stop"""
    retcode,res=submod_execcmd("get_type@cmod",signame)
    if retcode==0:
        return 0,"unable to find %s signal type <- %s"%(signame,res)
    stype=res
    if action=="start":
        func="power_on@"+stype
    elif action=="stop":
        func="power_off@"+stype
    retcode,res=submod_execcmd(func,signame)
    if retcode==0:
        return 0,"cant %s signal %s <- %s"%(action,signame,res)
    return 1,"ok"

#***************************** STATE MACHINE FUNCTIONS **********************

def get_state_ecal(ecal_id):
    "returns the current state of the detector"
    return 1,env["state"]

def update_state(newstate):
    env["state"]=newstate

def goto_ecal(ecal_id,dst_state,goto):
    # Already there
    state=env["state"]
    if state==dst_state:
        return 1,""

    seq="unable"
    # UNDEFINED
    if state=="UNDEFINED"    and dst_state=="READY":
        seq="init"
    if state=="UNDEFINED"    and dst_state=="CONFIGURED" and goto:
        seq="init,config"
    if state=="UNDEFINED"    and dst_state=="FCONFIGURED" and goto:
        seq="init,fconfig"
    if state=="UNDEFINED"    and dst_state=="ACQUIRING" and goto:
        seq="init,config,start_acq"

    # READY
    if state=="READY"        and dst_state=="UNDEFINED":
        seq="deinit"
    if state=="READY"        and dst_state=="FCONFIGURED":
        seq="fconfig"
    if state=="READY"        and dst_state=="CONFIGURED":
        seq="config"
    if state=="READY"        and dst_state=="ACQUIRING" and goto:
        seq="config,start_acq"

    # CONFIGURED
    if state=="CONFIGURED"   and dst_state=="UNDEFINED" and goto:
        seq="inval,deinit"
    if state=="CONFIGURED"   and dst_state=="READY":
        seq="inval"
    if state=="CONFIGURED"   and dst_state=="ACQUIRING":
        seq="start_acq"

    # RECONFIGURED
    if state=="RECONFIGURED" and dst_state=="UNDEFINED" and goto:
        seq="inval,deinit"
    if state=="RECONFIGURED" and dst_state=="READY":
        seq="inval"
    if state=="RECONFIGURED" and dst_state=="CONFIGURED":
        seq="validate"
    if state=="RECONFIGURED" and dst_state=="ACQUIRING":
        seq="validate,start_acq"

    # ACQUIRING
    if state=="ACQUIRING"    and dst_state=="UNDEFINED" and goto:
        seq="stop_acq,inval,deinit"
    if state=="ACQUIRING"    and dst_state=="READY" and goto:
        seq="stop_acq,inval"
    if state=="ACQUIRING"    and dst_state=="CONFIGURED":
        seq="stop_acq"

    if seq=="unable":
        return 0,"invalid transition"

    #apply sequence

    return 1,"ok"
    

def get_seq_transition_ecal(ecal_id,transition_name,goto):
    goto=sbool(goto)
    transitions_fstate={
            "init":      "READY",
            "config":    "CONFIGURED",
            "fconfig":   "FCONFIGURED",
            "validate":  "CONFIGURED",
            "start_acq": "ACQUIRING",
            "stop_acq":  "CONFIGURED",
            "inval":     "READY",
            "deinit":    "UNDEFINED"
            }
    # Get future state
    fstate=transitions_fstate.get(transition_name,"")

    # Already there
    state=env["state"]
    if state==fstate:
        if goto:
            return 1,""
        return 0,"already %s"%(state)

    seq="unable"
    # UNDEFINED
    if state=="UNDEFINED"    and fstate=="READY":
        seq="init"
    if state=="UNDEFINED"    and fstate=="CONFIGURED" and goto:
        seq="init,config"
    if state=="UNDEFINED"    and fstate=="FCONFIGURED" and goto:
        seq="init,fconfig"
    if state=="UNDEFINED"    and fstate=="ACQUIRING" and goto:
        seq="init,config,start_acq"

    # READY
    if state=="READY"        and fstate=="UNDEFINED":
        seq="deinit"
    if state=="READY"        and fstate=="FCONFIGURED":
        seq="fconfig"
    if state=="READY"        and fstate=="CONFIGURED":
        seq="config"
    if state=="READY"        and fstate=="ACQUIRING" and goto:
        seq="config,start_acq"

    # CONFIGURED
    if state=="CONFIGURED"   and fstate=="UNDEFINED" and goto:
        seq="inval,deinit"
    if state=="CONFIGURED"   and fstate=="READY":
        seq="inval"
    if state=="CONFIGURED"   and fstate=="ACQUIRING":
        seq="start_acq"

    # RECONFIGURED
    if state=="RECONFIGURED" and fstate=="UNDEFINED" and goto:
        seq="inval,deinit"
    if state=="RECONFIGURED" and fstate=="READY":
        seq="inval"
    if state=="RECONFIGURED" and fstate=="CONFIGURED":
        seq="validate"
    if state=="RECONFIGURED" and fstate=="ACQUIRING":
        seq="validate,start_acq"

    # ACQUIRING
    if state=="ACQUIRING"    and fstate=="UNDEFINED" and goto:
        seq="stop_acq,inval,deinit"
    if state=="ACQUIRING"    and fstate=="READY" and goto:
        seq="stop_acq,inval"
    if state=="ACQUIRING"    and fstate=="CONFIGURED":
        seq="stop_acq"

    if seq=="unable":
        return 0,"invalid transition"
    return 1,seq

#######################################################
# INIT
#######################################################

def init_ecal(ecal_id):
    retcode,res=submod_execcmd("checkns@varmod","varmod","ecal")
    if retcode==1:
        submod_execcmd("delns@varmod","varmod","ecal")
    retcode,res=submod_execcmd("newns@varmod","varmod","ecal")
    if retcode==0:
        return 0,"error creating ns ecal in varmod <- %s"%(res)
    return 1,"ok"

def init_fin_ecal(ecal_id):
    update_state("READY")
    return 1,"ok"

#######################################################
# DEINIT
#######################################################

def deinit_fin_ecal(ecal_id):
    "deinitialize the detector"
    update_state("UNDEFINED")
    retcode,res=submod_execcmd("delns@varmod","varmod","ecal")
    if retcode==0:
        return 0,"error deleting ns ecal in varmod <- %s"%(res)
    return 1,"ok"

#######################################################
# CONFIGURE
#######################################################

def config_ecal(ecal_id,config_mode):
    "configure the detector"
    retcode,res=submod_execcmd("setvar@varmod","varmod","ecal","config_mode",config_mode)
    if retcode==0:
        return 0,"error setting config_mode in varmod <- %s"%(res)
    return 1,"ok"

def config_fin_ecal(ecal_id):
    #stopping the injector signal just in case
    find_n_act_signal("injector","stop")
    #start beam clock in case we have it
    find_n_act_signal("beam_clock","start")
    #check if a signal exists
    retcode,res=submod_execcmd("get_type@cmod","trigger")
    if retcode!=0:
        #send a trigger for synchronization
        retcode1,res1=find_n_act_signal("trigger","start")
        retcode2,res2=find_n_act_signal("trigger","stop")
        if retcode1==0:
            return 0,res1
        if retcode2==0:
            return 0,res2
    update_state("CONFIGURED")
    return 1,"ok"

def fconfig_ecal(ecal_id):
    update_state("CONFIGURED")
    return 1,"ok"

#######################################################
# INVALIDATE
#######################################################

def inval_fin_ecal(ecal_id):
    update_state("READY")
    return 1,"ok"

#######################################################
# VALIDATE
#######################################################

def validate_fin_ecal(ecal_id):
    """start the configuration after a bunch of reconfiguration
    mandatory to go to ACQUIRING"""
    update_state("CONFIGURED")
    return 1,"ok"

#######################################################
# START ACQUISITION
#######################################################

def start_acq_ecal(ecal_id,acq_id,acq_path):
    "start the acquisition"

    env["current_acq_id"]=acq_id
    env["current_acq_path"]=acq_path

    retcode,res=submod_execcmd("setvar@varmod","varmod","ecal","current_acq_id",acq_id)

    # get list of acqpc
    retcode,res=submod_execcmd("get_name_list@cmod","acqpc")
    if retcode==0:
        return 0,"error getting list of acqpc's <- %s"%(res)
    acqpcs=res.split(",")
   
    return 1,"ok"

def start_acq_fin_ecal(ecal_id):
    find_n_act_signal("injector","start")
    find_n_act_signal("spill","start")
    update_state("ACQUIRING")
    return 1,"ok"

#######################################################
# STOP ACQUISITION
#######################################################


def stop_acq_ecal(ecal_id):
    "stop the acquisition"
    find_n_act_signal("spill","stop")
    find_n_act_signal("injector","stop")
    return 1,"ok"

def stop_acq_fin_ecal(ecal_id):
    update_state("CONFIGURED")
    return 1,"ok"

#######################################################
# RECONFIGURE
#######################################################

def reconfigure_ecal(ecal_id,device_name,func_name,*params):
    "reconfigure a device"
    state=env["state"]
    if state not in ["CONFIGURED","RECONFIGURED"]:
        return 0,"reconfiguration is only available in CONFIGURED or RECONFIGURED state (actual state is %s)"%(state)
    if device_name==ecal_id:
        return 0,"ecal cant reconfigure itself"

    retcode,res=submod_execcmd("command@cmod",device_name,"false",func_name,*params)
    if retcode==0:
        return 0,"device cannot be reconfigured <- %s"%(res)
    update_state("RECONFIGURED")
    return 1,"ok"

