#!/bin/sh
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Calicoes.
. /usr/local/bin/unit_test_lib.sh
. /opt/pyrame/ports.sh

get12bytes() {
  echo "getting 12 bytes of data..."
  RETSTR=0
  while test ${RETSTR%.*} -lt 12; do
    sleep 0.5s
    exec_n_test chkpyr2.py localhost $VARMOD_PORT getvar_varmod varmod default bytes_on_dsk > /dev/null 2>&1
    echo -n "."
  done
  echo ""
}

clean_all
echo 
echo 
echo "ECAL TESTS"
echo
autolauncher.py -n > autolauncher.trace 2>&1 &
wait_until_awake localhost $ECAL_PORT
wait_until_awake localhost $CMOD_PORT
wait_until_awake localhost $RC_PORT
wait_until_awake localhost $ACQPC_PORT
wait_until_awake localhost $ACQ_PORT
wait_until_awake localhost $VARMOD_PORT
wait_until_awake localhost $RUNDB_PORT

# test rc script
exec_n_test ./unit_tests/unit_test_run.py


# boot
exec_n_test load_config_file.sh /opt/calicoes/config/test_ecal.xml
exec_n_test initialize.sh
exec_n_test configure.sh

DEST_DATADIR=/home/data

#acquisition with default run
ACQ="`date -u +%Y%m%d-%H%M%S`-UTC/unit_test_acq"
exec_n_test startacq.sh $ACQ
get12bytes
exec_n_test stopacq.sh
sleep 2s
check_file_size 12 $DEST_DATADIR/default/${ACQ}_ecal_dif_0.raw
check_file_size 0  $DEST_DATADIR/default/${ACQ}_ecal_dif_1.raw
check_file_size 0  $DEST_DATADIR/default/${ACQ}_ecal_dif_2.raw
check_equal_files ref.xml /home/data/default/${ACQ}.xml

# create new run (test remove mode, as well)
RUN=unit_test_run
ACQ="`date -u +%Y%m%d-%H%M%S`-UTC/unit_test_acq"
exec_n_test startacq.sh $ACQ $RUN
get12bytes
exec_n_test stopacq.sh
sleep 2s
check_equal_files ref0.raw $DEST_DATADIR/$RUN/${ACQ}_ecal_dif_0.raw
check_empty_file $DEST_DATADIR/$RUN/${ACQ}_ecal_dif_1.raw
check_empty_file $DEST_DATADIR/$RUN/${ACQ}_ecal_dif_2.raw
check_equal_files ref.xml $DEST_DATADIR/$RUN/${ACQ}.xml
check_exist_file $DEST_DATADIR/$RUN/${ACQ}_acq_rundb_dump.json
check_exist_file $DEST_DATADIR/$RUN/${ACQ}.log

# test quick stop/start
ACQ1="unit_test_acq1"
ACQ2="unit_test_acq2"
exec_n_test startacq.sh $ACQ1 $RUN
exec_n_test stopacq.sh
exec_n_test startacq.sh $ACQ2 $RUN
exec_n_test stopacq.sh
exec_n_test invalidate.sh
exec_n_test deinitialize.sh

clean_all
echo "all tests passed"
