#!/bin/sh
# -*- coding: utf-8 -*-
# 
# Copyright 2012-2015 Frédéric Magniette, Miguel Rubio-Roy
# This file is part of Calicoes.
. /usr/local/bin/unit_test_lib.sh
. /opt/pyrame/ports.sh

echo 
echo 
echo "ECAL TESTS"
echo
wait_until_awake localhost $CMOD_PORT
wait_until_awake localhost $RC_PORT
wait_until_awake localhost $ACQPC_PORT
wait_until_awake localhost $ACQ_PORT
wait_until_awake localhost $VARMOD_PORT
wait_until_awake localhost $RUNDB_PORT
wait_until_awake localhost $DOMAIN_PORT

# boot
exec_n_test set_use_rundb.sh 1
exec_n_test load_config_file.sh `pwd`/../config/ecal.xml
exec_n_test initialize.sh
exec_n_test configure.sh
exec_n_test get_dest_datadir.sh
DEST_DATADIR=`cat ent.trace | awk -F= '{ print $3 }'`

echo "test with RunDB"
PREFIX="unit_test_run/`date -u +%Y%m%d-%H%M%S`-UTC/unit_test_acq"
exec_n_test startacq.sh $PREFIX
echo "getting data for 10s..."
sleep 10s
exec_n_test stopacq.sh
sleep 3s
check_not_empty_file $DEST_DATADIR/${PREFIX}_by_dif0.raw
check_equal_files ref_online0.xml $DEST_DATADIR/${PREFIX}.xml
check_exist_file $DEST_DATADIR/${PREFIX}_acq_rundb_dump.json
echo "test remove mode"
PREFIX="unit_test_acq"
exec_n_test startacq.sh $PREFIX remove
echo "getting data for 10s..."
sleep 10s
exec_n_test stopacq.sh
sleep 3s
check_not_empty_file $DEST_DATADIR/${PREFIX}_by_dif0.raw
check_equal_files ref_online0.xml $DEST_DATADIR/${PREFIX}.xml
check_exist_file $DEST_DATADIR/${PREFIX}_acq_rundb_dump.json
echo "test quick stop/start"
PREFIX1="unit_test_acq1"
PREFIX2="unit_test_acq2"
exec_n_test startacq.sh $PREFIX1 remove
exec_n_test stopacq.sh
exec_n_test startacq.sh $PREFIX2 remove
exec_n_test stopacq.sh
echo "check test keyword"
PREFIX="test"
exec_n_test startacq.sh $PREFIX
echo "getting data for 10s..."
sleep 10s
exec_n_test stopacq.sh
sleep 3s
check_not_empty_file $DEST_DATADIR/${PREFIX}_by_dif0.raw
check_equal_files ref_online0.xml $DEST_DATADIR/${PREFIX}.xml
echo "reload exported config"
exec_n_test invalidate.sh
exec_n_test deinitialize.sh
exec_n_test load_config_file.sh $DEST_DATADIR/${PREFIX}.xml
exec_n_test initialize.sh
exec_n_test configure.sh
exec_n_test startacq.sh $PREFIX
echo "getting data for 10s..."
sleep 10s
exec_n_test stopacq.sh
echo "reconfiguring all chips"
exec_n_test ./get_name_sublist.sh skiroc root
CHIPS=${RETSTR//,/ }
i=300
for chip in $CHIPS; do
  reconfigure.sh $chip set_gtrigger_skiroc $i
  i=$(($i+1))
done
PREFIX="unit_test_run/`date -u +%Y%m%d-%H%M%S`-UTC/unit_test_acq"
exec_n_test startacq.sh $PREFIX
echo "getting data for 10s..."
sleep 10s
exec_n_test stopacq.sh
sleep 3s
check_not_empty_file $DEST_DATADIR/${PREFIX}_by_dif0.raw
check_equal_files ref_online1.xml $DEST_DATADIR/${PREFIX}.xml
check_exist_file $DEST_DATADIR/${PREFIX}_acq_rundb_dump.json
echo "reload exported reconfigured config"
exec_n_test invalidate.sh
exec_n_test deinitialize.sh
exec_n_test load_config_file.sh $DEST_DATADIR/${PREFIX}.xml
exec_n_test initialize.sh
exec_n_test configure.sh
exec_n_test startacq.sh $PREFIX remove
echo "getting data for 10s..."
sleep 10s
exec_n_test stopacq.sh

echo "test without RunDB"
exec_n_test_fail load_config_file.sh `pwd`/../config/ecal.xml
exec_n_test invalidate.sh
exec_n_test load_config_file.sh `pwd`/../config/ecal.xml
exec_n_test deinitialize.sh
exec_n_test load_config_file.sh `pwd`/../config/ecal.xml
exec_n_test initialize.sh
exec_n_test configure.sh
exec_n_test set_use_rundb.sh 0
exec_n_test get_dest_datadir.sh
DEST_DATADIR=`cat ent.trace | awk -F= '{ print $3 }'`
PREFIX="`date -u +%Y%m%d-%H%M%S`/unit_test_acq"
exec_n_test startacq.sh $PREFIX
echo "getting data for 10s..."
sleep 10s
exec_n_test stopacq.sh
sleep 3s
check_not_empty_file $DEST_DATADIR/${PREFIX}_by_dif0.raw
check_equal_files ref_online0.xml $DEST_DATADIR/${PREFIX}.xml
PREFIX="unit_test_acq"
exec_n_test startacq.sh $PREFIX remove
echo "getting data for 10s..."
sleep 10s
exec_n_test stopacq.sh
sleep 3s
check_not_empty_file $DEST_DATADIR/${PREFIX}_by_dif0.raw
check_equal_files ref_online0.xml $DEST_DATADIR/${PREFIX}.xml
PREFIX="test"
exec_n_test startacq.sh $PREFIX
echo "getting data for 10s..."
sleep 10s
exec_n_test stopacq.sh
sleep 3s
check_not_empty_file $DEST_DATADIR/${PREFIX}_by_dif0.raw
check_equal_files ref_online0.xml $DEST_DATADIR/${PREFIX}.xml

exec_n_test invalidate.sh
exec_n_test deinitialize.sh

echo "all tests passed"
