//
// Copyright 2012-2014 Frédéric Magniette, Miguel Rubio-Roy
//

#include <arpa/inet.h>
#include "uncap_dif4_1.h"

//allocate memory structure
void *init(int nb_difs,int pfid,char *pname) {
  struct uncdif_workspace *ws=malloc(sizeof(struct uncdif_workspace));
  int i;
  printf("init uncap for %d difs\n",nb_difs);
  ws->next_id=malloc(sizeof(unsigned int)*nb_difs);
  for(i=0;i<nb_difs;i++)
    ws->next_id[i]=-1;
  ws->maxdifnb=nb_difs;
  ws->fid=pfid;
  ws->name=strdup(pname);
  return (void *)ws;
}

//free memory structure
void deinit(void *workspace) {
  struct uncdif_workspace *ws=(struct uncdif_workspace *)workspace;
  printf("deinit dif uncap %s\n",ws->name);
  free(ws->name);
  free(ws->next_id);
  free(ws);
}

//used by the acquisition to know if the packet has to be kept or not
//be careful this function is called in the critical path so be parcimonic
int prefilter(void *workspace,char * packet,int size) {
  //printf("prefilter called wih size : %d\n",size);
  int protocol=ntohs( *(unsigned short*)(packet+12));
  if ((protocol==0x809) || (protocol==0x810))
    return PREF_CTRL;
  if ((protocol==0x811))
    return PREF_DATA;
  return PREF_JUNK;
}


//uncap a packet
//packet is the buffer and packet_size its size
//result is the resulting buffer and result_size its size
//loss should be set to 1 if a loss of packet has been detected
//data should be set to 1 if a data packet is detected (0 for a control packet)
//corrupted should be set to 1 if the packet is corrupted
//stream should contain the number of the stream, i.e. the dif localid
int uncap(void *workspace,unsigned char *packet,int packet_size,unsigned char ** result,int *result_size,unsigned char *loss,unsigned char *data,unsigned char *corrupted,int *stream,char *refclock) {

  struct uncdif_workspace *ws=(struct uncdif_workspace *)workspace;

  int newid;
  int dif;
  int difraw;

  //initialization
  *corrupted=0;
  *loss=0;
  
  //control packet
  if ((ntohs( *(unsigned short*)(packet+12))==0x809) || (ntohs( *(unsigned short*)(packet+12))==0x810)) {
    *data=0;
    *stream=0;
    //uncap 14 bytes for ethernet header + 16 bytes for gdcc header
    *result=packet+14+16;
    //size = orig size - 14 bytes of eth header - 16 bytes of gdcc header - 2 bytes of gdcc trailer
    *result_size=packet_size-32;
    return 1;
  }

    //data packet
  if ((ntohs( *(unsigned short*)(packet+12))==0x811)) {
    *data=1;

    //get the id of the packet
    newid=*(unsigned short*)(packet+24);
    //printf("newid=%d\n",newid);
    
    //determine the stream number (localid)
    difraw=*(unsigned short*)(packet+26);
    dif=difraw-1;//+ws->fid;
    //printf("found dif %d\n",dif);
    *stream=dif+ws->fid;
    
    //uncap
    *result=packet+30;
    //printf("result=%x\n",(int)result);
    *result_size=packet_size-32;
    //printf("newid=%d dif=%d difraw=%d fid=%d result_size=%d\n",newid,dif,difraw,ws->fid,*result_size);

    //if (dif<ws->fid) {
       //printf("difid %d is before the fid (%d)\n",dif,ws->fid);
    if (dif<0) {
       printf("difid %d is negative\n",dif);
       *stream=-1;
       *corrupted=1;
       return 1;
    }

    if (difraw-1>ws->maxdifnb) {
      printf("difid %d outside the range (%d)\n",dif,ws->maxdifnb);
      *stream=-1;
      *corrupted=1;
      return 1;
    }
    
    //check for replay packets
    if (newid<ws->next_id[dif])
      if (newid>5) 
        if (ws->next_id[dif]!=-1) {
          printf("corr packet %d for dif %d waiting for %d\n",newid,dif,ws->next_id[dif]);
          *corrupted=1;
        }

    //check for lost packets
    if ((newid>ws->next_id[dif]) && (ws->next_id[dif]!=-1) && (newid!=0)) {
      printf("lost packet %d for dif %d (received %d)\n",ws->next_id[dif],dif,newid);
      //inject an error tag in the packet
      *((unsigned short *)(*result))=0xfffa;
      *loss=1;
    }

    ws->next_id[dif]=newid+1;
    return 1;
  }
  printf("unknown packet (should never happen)\n");
  return 0;
}

//extract the packet id from the packet buffer
int get_pktid(unsigned char *packet) {
  return ntohs(*(unsigned short *)(packet+18));
}

//extract the modifier from the packet buffer
int get_modifier(unsigned char *packet) {
  return ntohs(*(unsigned short *)(packet+26));
}


//this funtion decide if a packet is coherent with a set of ids 
int select_packet(void *workspace,unsigned char *packet,int id1,int id2,int id3,int id4) {
  int pktid;
  int pktmod;
  pktid=get_pktid(packet);
  pktmod=htons(get_modifier(packet));
  //printf("id=%x mod=%x : ",pktid,pktmod);
  
  if (id1==0xd00d) {
    if (pktmod==id2) {
      return 1;
    } else {
      return 0;
    }
  } else {
    if (pktid==id1) {
      return 1;
    } else {
      return 0;
    }
  }
  printf("should not happen : uncap_dif\n");
  return 0;
}
