#include "CalicoesProducer.hpp"

void newblock(void *prod,struct block *b) {
  CalicoesProducer *producer=(CalicoesProducer *)prod;
  producer->NewBlock(b);
}

void newevt(void *prod,struct event *e) {
  CalicoesProducer *producer=(CalicoesProducer *)prod;
  producer->NewEvt(e);
}

void CalicoesProducer::NewBlock(struct block *b) {
  cout<<"new spill "<<get_block_field(b,"spill")<<endl;
}

void CalicoesProducer::NewEvt(struct event *e) {
  int new_spill=atoi(get_event_field(e,"spill"));

  //if the spill has changed : send the data to eudaq
  if (new_spill!=current_spill) {
    printf("new spill %d is different from old spill %d\n",new_spill,current_spill);
    if (current_event!=NULL) {
      printf("sending and detroying event\n");
      SendEvent(*current_event);
      delete(current_event);
    }
    current_event=NULL;
  }

  //if the data has been sent or it is the first event : allocate event and set current_spill
  if (current_event==NULL) {
    current_spill=new_spill;
    printf("creating new event for spill %d\n",current_spill); 
    current_event=new eudaq::RawDataEvent(EVENT_TYPE,run_nb,current_spill);
    event_counter=0;
  }

  //extract fields from data
  float spill=atof(get_event_field(e,"spill"));
  float bcid=atof(get_event_field(e,"bcid"));
  float time=atof(get_event_field(e,"time"));
  float x=atof(get_event_field(e,"x"));
  float y=atof(get_event_field(e,"y"));
  float z=atof(get_event_field(e,"z"));
  float en1=atof(get_event_field(e,"en1"));
  float en2=atof(get_event_field(e,"en2"));
  printf("from data : spill=%f bcid=%f time=%f x=%f y=%f z=%f en1=%f en2=%f\n",spill,bcid,time,x,y,z,en1,en2);
  
  //add the fields to the event
  vector<float> data;
  data.push_back(bcid);
  data.push_back(time);
  data.push_back(x);
  data.push_back(y);
  data.push_back(z);
  data.push_back(en1);
  data.push_back(en2);
  current_event->AddBlock(event_counter,data);
  event_counter++;
}

CalicoesProducer::CalicoesProducer(const string & name,const string & runcontrol) : eudaq::Producer(name,runcontrol) {
  run_nb=0;
  done=0;
  rc_port=get_port("RC_PORT");
  el=NULL;
  system("/usr/local/bin/autolauncher.py -n 2>&1 > /dev/null");
  printf("CalicoesProducer launched (current_spill=%d\n",current_spill);
}
  

void CalicoesProducer::apply_sequence(vector<string> trans) {
  struct cmd_result *cres;
  for (vector<string>::iterator it=trans.begin();it!=trans.end();++it) {
    cout<<"firing transition "<<it->c_str()<<endl;
    cres=sendcmd("localhost",rc_port,"transition_rc",it->c_str(),"undef","end");
    if (cres->retcode==0) {
      retcode=0;
      retstr=cres->str;
      return;
    } else {
      free_cmd_result(cres);
      cres=NULL;
    }
  }
  retcode=1;
  retstr="ok";
} 


  // This gets called whenever the DAQ is configured
void CalicoesProducer::OnConfigure(const eudaq::Configuration & config) {
  struct cmd_result *cres;

  //go to busy status
  //SetStatus(eudaq::Status::LVL_BUSY,"configuring Calicoes");

  //getting configuration params
  cout<<"Configuring: "<<config.Name()<<endl;
  config_file=config.Get("CalicoesConfigFilename","unknown");
  if (config_file=="unknown") {
    SetStatus(eudaq::Status::LVL_ERROR,"No config file available for calicoes Producer");
    return;
  }
  cout<<"Calicoes config filename = "<<config_file<<endl;


  //getting the state machine to the right state
  string state="unknown";
  retcode=1;
  cres=sendcmd("localhost",rc_port,"get_state_rc","end");
  if (cres->retcode!=0) {
    state=cres->str;
    if (state=="ACQUIRING") {
      vector<string> trans {"stopacq","inval","deinit"};
      apply_sequence(trans);
    }
    if (state=="CONFIGURED" || state=="RECONFIGURED") {
      vector<string> trans {"inval","deinit"};
      apply_sequence(trans);
    }
    if (state=="READY") {
      vector<string> trans {"deinit"};
      apply_sequence(trans);
    }
    if (retcode==0) {
      SetStatus(eudaq::Status::LVL_ERROR,retstr);
      return;
    }
  }
  free_cmd_result(cres);
  cres=NULL;

  //load config file
  cres=sendcmd("localhost",rc_port,"load_config_file_rc",config_file.c_str(),"end");
  if (cres->retcode==0) {
    SetStatus(eudaq::Status::LVL_ERROR,cres->str);
    free_cmd_result(cres);
    return;
  } else {
    free_cmd_result(cres);
    cres=NULL;
  }

  //apply configuration
  vector<string> trans {"init","config"};
  apply_sequence(trans);
  if (retcode==0) {
    SetStatus(eudaq::Status::LVL_ERROR,retstr);
    return;
  }

  //set internal variables
  event_counter=0;
  current_spill=0;
  current_event=NULL;

  // At the end, set the status that will be displayed in the Run Control.
  SetStatus(eudaq::Status::LVL_OK,"Configured ("+config.Name()+")");
}

  
  // This gets called whenever a new run is started
  // It receives the new run number as a parameter

void CalicoesProducer::OnStartRun(unsigned param) {
  struct cmd_result *cres;
  run_nb=param;
  event_counter=0;
  cout<<"Start Run: "<<run_nb<<endl;

  //go to busy status
  //SetStatus(eudaq::Status::LVL_BUSY,"starting Calicoes");


  // It must send a BORE to the Data Collector
  eudaq::RawDataEvent bore(eudaq::RawDataEvent::BORE(EVENT_TYPE,run_nb));
  SendEvent(bore);
  
  stringstream ss;
  ss<<"Eudaq_run_"<<run_nb;
  cres=sendcmd("localhost",rc_port,"transition_rc","start_acq","undef",ss.str().c_str(),"end");
  if (cres->retcode==0) {
    SetStatus(eudaq::Status::LVL_ERROR,cres->str);
    free_cmd_result(cres);
    return;
  } else {
    free_cmd_result(cres);
  }

  //start the event loop
  if (el!=NULL) {
    free_event_loop(el);
  }
  el=new_event_loop("localhost",0,(void *)this,newevt,newblock,FIRST);
  clean_loop(el);
  //run_loop(el);
  
  // At the end, set the status that will be displayed in the Run Control.
  SetStatus(eudaq::Status::LVL_OK,"Running");
}


void CalicoesProducer::OnStopRun() {
  struct cmd_result *cres;
  cout<<"Stopping Run"<<endl;

  //go to busy status
  //SetStatus(eudaq::Status::LVL_BUSY,"stopping Calicoes");
  

  //stopping calicoes
  cres=sendcmd("localhost",rc_port,"transition_rc","stop_acq","undef","end");
  if (cres->retcode==0) {
    SetStatus(eudaq::Status::LVL_ERROR,cres->str);
    free_cmd_result(cres);
    return;
  } else {
    free_cmd_result(cres);
  }

  //stop the event loop
  printf("stopping loop\n");
  stop_loop(el);
  printf("loop stopped\n");
  

// Send an EORE after all the real events have been sent
  // You can also set tags on it (as with the BORE) if necessary
  event_counter++;
  SendEvent(eudaq::RawDataEvent::EORE(EVENT_TYPE,run_nb,event_counter));

  //set final ok state
  SetStatus(eudaq::Status::LVL_OK,"Stopped");
}
  
  
void CalicoesProducer::OnTerminate() {
  cout<<"Terminating..."<<endl;
  system("/usr/local/bin/kill_pyrame.sh");
  done=true;
}

int CalicoesProducer::IsDone() {
  return done;
}

int main(int /*argc*/,const char ** argv) {
  struct cp_ws *ws=NULL;

  eudaq::OptionParser op("Eudaq/Calicoes Producer","1.0","for common testbeam with IPNL HGCAL");
  eudaq::Option<string> rctrl(op,"r","runcontrol","tcp://localhost:44000","address","The address of the RunControl.");
  eudaq::Option<string> level(op,"l","log-level","NONE","level","The minimum level for displaying log messages locally");
  eudaq::Option<string> name (op,"n","name","Calicoes","string","The name of this Producer");
  
  try {
    op.Parse(argv);
    // Set the Log level for displaying messages based on command-line
    EUDAQ_LOG_LEVEL(level.Value());
    CalicoesProducer producer(name.Value(),rctrl.Value());
    while (!producer.IsDone()) {
      eudaq::mSleep(20);
    }
    cout<<"Quitting"<<endl;
  } catch (...) {
    // This does some basic error handling of common exceptions
    return op.HandleMainException();
  }
  return 0;
}
