#include <stdio.h>
#include <TH2F.h>
#include <TThread.h>
#include <TCanvas.h>
#include <TString.h>
#include <TApplication.h>
#include <TGFrame.h>
//#include <THttpServer.h>
#include <pyrame.h>

struct or_ws {
  char first_time;
  TCanvas *canvas;
  //THttpServer *serv;
  struct evloop *loop;
  TGMainFrame *fmain;
  TH2F * NHitsRate_buf0;
  TH2F * NHitsRate;
  TH2F * BCIDRate_buf0;
  TH2F * BCIDRate;
  int last_sca;
  int last_roc;
  int hit_counter;
};


void newblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;
  if (ws->first_time) {
    printf("waiting for app to start\n");
    sleep(1);
    ws->first_time=0;
  }
  printf("new spill : %s\n",get_block_field(b,"spill"));
  //print_block(b);
}

void endblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;
  for (int i=1;i<=4;i++)
    ws->canvas->cd(i)->Modified();
  ws->canvas->Update();
}

void reinit(void *workspace) {
}



void newevt(void *workspace,struct event *e) {
  struct or_ws *ws=(struct or_ws *)workspace;
  //Double_t energy=(Double_t)atof(get_event_field(e,"en"));
  int hit=atoi(get_event_field(e,"hit"));
  int roc=atoi(get_event_field(e,"roc"));
  int sca=atoi(get_event_field(e,"sca"));
  int bcid=atoi(get_event_field(e,"bcid"));
  int error=atoi(get_event_field(e,"error"));
  int tag=atoi(get_event_field(e,"tag"));


  printf("hit=%d roc=%d sca=%d bcid=%d tag=%d error=%d\n",hit,roc,sca,bcid,tag,error);

  if (sca!=ws->last_sca || roc!=ws->last_roc) {
    if (ws->last_sca==0) {
      ws->NHitsRate_buf0->Fill(ws->hit_counter,ws->last_roc);
    } else {
      printf("filling NHitsRate\n");
      ws->NHitsRate->Fill(ws->hit_counter,ws->last_roc);
    }

    //initialize for new roc/sca couple
    ws->hit_counter=0;
    ws->last_sca=sca;
    ws->last_roc=roc;
  }
  if (hit) {
    ws->hit_counter++;
    if (sca==0)
      ws->BCIDRate_buf0->Fill(bcid,roc);
    else
      ws->BCIDRate->Fill(bcid,roc);
  }
}


int main(int argc, char **argv) {

  if (argc!=2) {
    printf("usage : %s data_source_name\n",argv[0]);
    exit(1);
  }
  
  //initialize
  struct or_ws *ws=(struct or_ws *)malloc(sizeof(struct or_ws));
  TApplication ecalApp("App",&argc,argv);

  //ws->serv=new THttpServer("http:8080");
  ws->canvas=new TCanvas("ecal","Ecal Dynamic Histogram",600,600);
  ws->canvas->Divide(2,2);
 
  ws->NHitsRate_buf0 = new TH2F("NHitsRate_buf0","NHitsRate_buf0",10,-0.5,9.5,16,-0.5,15.5);
  ws->NHitsRate = new TH2F("NHitsRate","NHitsRate",10,-0.5,9.5,16,-0.5,15.5);
  ws->BCIDRate_buf0 = new TH2F("BCIDRate_buf0","BCIDRate_buf0",500,0,5000,16,-0.5,15.5);
  ws->BCIDRate = new TH2F("BCIDRate","BCIDRate",500,0,5000,16,-0.5,15.5);

  ws->canvas->cd(1);
  ws->NHitsRate_buf0->SetTitle("NHits rates, SCA=0");
  ws->NHitsRate_buf0->GetXaxis()->SetTitle("NHits");
  ws->NHitsRate_buf0->GetYaxis()->SetTitle("chip");
  ws->NHitsRate_buf0->Draw("colz");

  ws->canvas->cd(2);
  ws->NHitsRate->SetTitle("NHits rates, SCA>0");
  ws->NHitsRate->GetXaxis()->SetTitle("NHits");
  ws->NHitsRate->GetYaxis()->SetTitle("chip");
  ws->NHitsRate->Draw("colz");

  ws->canvas->cd(3);
  ws->BCIDRate_buf0->SetTitle("BCIDRate, SCA=0");
  ws->BCIDRate_buf0->GetXaxis()->SetTitle("BCIDRate");
  ws->BCIDRate_buf0->GetYaxis()->SetTitle("chip");
  ws->BCIDRate_buf0->Draw("colz");

  ws->canvas->cd(4);
  ws->BCIDRate->SetTitle("BCIDRate, SCA>0");
  ws->BCIDRate->GetXaxis()->SetTitle("BCIDRate");
  ws->BCIDRate->GetYaxis()->SetTitle("chip");
  ws->BCIDRate->Draw("colz");

  ws->canvas->Update();
  ws->canvas->cd(0);

  ws->loop=new_event_loop(argv[1],ws,newevt,newblock,endblock,reinit,'f');
  ws->first_time=1;
  ws->last_sca=-1;
  ws->last_roc=-1;
  run_loop(ws->loop);
  
  //run the TApp
  ecalApp.Run();

  //never reached
  return 0;
}

