#include <stdio.h>
#include <TH2F.h>
#include <TThread.h>
#include <TCanvas.h>
#include <TString.h>
#include <TApplication.h>
//#include <THttpServer.h>
#include <pyrame.h>
#include <pthread.h>
#include <TThread.h>

struct or_ws {
  char first_time;
  TCanvas *canvas;
  //THttpServer *serv;
  struct evloop *loop;

  //-----------------------------------------------
  //SimpleSCA Monitoring
  //good events: no retrig, no plane event, no negative data
  TH2F * NHitsRate_buf0;// rate of nhits per event for SCA =0
  TH2F * NHitsRate;// rate of nhits per event for SCA >0
  TH2F * BCID_buf0; //bcid distribution for SCA =0
  TH2F * BCID;//bcid distribution for SCA >0
  //same but for events tagged as retriggered
  TH2F * RetrigNHitsRate_buf0;
  TH2F * RetrigNHitsRate;
  TH2F * RetrigBCID_buf0;
  TH2F * RetrigBCID;
  //same but for events tagged as negative
  TH2F * UnderFlowNHitsRate_buf0;
  TH2F * UnderFlowNHitsRate;
  TH2F * UnderFlowBCID_buf0;
  TH2F * UnderFlowBCID;
  //same but for events tagged as plane events
  TH2F * PlaneNHitsRate_buf0;
  TH2F * PlaneNHitsRate;
  TH2F * PlaneBCID_buf0;
  TH2F * PlaneBCID;

  TThread *updater_t;

  int last_sca;
  int last_roc;
  int hit_counter;
  int hit_counter_retrig;
  int hit_counter_plane;
  int hit_counter_under;

  pthread_mutex_t * updater_mutex;
};


void *updater(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  while(1) {
    pthread_mutex_lock(ws->updater_mutex);
    for (int i=1;i<=16;i++) {
      ws->canvas->cd(i)->Modified();
    }
    ws->canvas->Update();
    pthread_mutex_unlock(ws->updater_mutex);
    usleep(50000);
  }
  return NULL;
}

void newblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;
  if (ws->first_time) {
    printf("waiting for app to start\n");
    sleep(1);
    ws->first_time=0;
  }
  printf("new spill : %s\n",get_block_field(b,"spill"));
  //print_block(b);
}

void endblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;
}

void reinit(void *workspace) {
}



void newevt(void *workspace,struct event *e) {
  struct or_ws *ws=(struct or_ws *)workspace;

  pthread_mutex_lock(ws->updater_mutex);

  //Double_t energy=(Double_t)atof(get_event_field(e,"en"));
  int hit=atoi(get_event_field(e,"hit"));
  int roc=atoi(get_event_field(e,"roc"));
  int sca=atoi(get_event_field(e,"sca"));
  int bcid=atoi(get_event_field(e,"bcid"));
  int error=atoi(get_event_field(e,"error"));
  int tag=atoi(get_event_field(e,"tag"));

  //  printf("hit=%d roc=%d sca=%d bcid=%d tag=%d error=%d\n",hit,roc,sca,bcid,tag,error);

  /*
    tag list
    #define bxpn_tag 1
    #define plane_tag 2
    #define underflow_tag 4
    #define overflow_tag 8
  */

  if (sca!=ws->last_sca || roc!=ws->last_roc) {
    if (ws->last_sca==0) {
      if(tag==0) ws->NHitsRate_buf0->Fill(ws->hit_counter,ws->last_roc);
      if(tag==1) ws->RetrigNHitsRate_buf0->Fill(ws->hit_counter_retrig,ws->last_roc);
      if(tag==2) ws->PlaneNHitsRate_buf0->Fill(ws->hit_counter_plane,ws->last_roc);
      if(tag==4 || tag==8) ws->UnderFlowNHitsRate_buf0->Fill(ws->hit_counter_under,ws->last_roc);
    } else {
      //printf("filling NHitsRate\n");
      if(tag==0) ws->NHitsRate->Fill(ws->hit_counter,ws->last_roc);
      if(tag==1) ws->RetrigNHitsRate->Fill(ws->hit_counter_retrig,ws->last_roc);
      if(tag==2) ws->PlaneNHitsRate->Fill(ws->hit_counter_plane,ws->last_roc);
      if(tag==4 || tag==8) ws->UnderFlowNHitsRate->Fill(ws->hit_counter_under,ws->last_roc);
    }

    //initialize for new roc/sca couple
    ws->hit_counter=0;
    ws->hit_counter_retrig=0;
    ws->hit_counter_plane=0;
    ws->hit_counter_under=0;
    ws->last_sca=sca;
    ws->last_roc=roc;
  }


  if (hit) {

    if(tag==0) ws->hit_counter++;
    if(tag==1) ws->hit_counter_retrig++; 
    if(tag==2) ws->hit_counter_plane++;
    if(tag==4 || tag==8) ws->hit_counter_under++;

    if (sca==0) {
      if(tag==0) ws->BCID_buf0->Fill(bcid,roc);
      if(tag==1) ws->RetrigBCID_buf0->Fill(bcid,roc);
      if(tag==2) ws->PlaneBCID_buf0->Fill(bcid,roc);
      if(tag==4 || tag==8) ws->UnderFlowBCID_buf0->Fill(bcid,roc);
    } else {
      if(tag==0) ws->BCID->Fill(bcid,roc);
      if(tag==1) ws->RetrigBCID->Fill(bcid,roc);
      if(tag==2) ws->PlaneBCID->Fill(bcid,roc);
      if(tag==4 || tag==8) ws->UnderFlowBCID->Fill(bcid,roc);
    }
  }


  pthread_mutex_unlock(ws->updater_mutex);

}


int main(int argc, char **argv) {

  if (argc!=2) {
    printf("usage : %s data_source_name\n",argv[0]);
    exit(1);
  }
  
  //initialize
  struct or_ws *ws=(struct or_ws *)malloc(sizeof(struct or_ws));
  TApplication ecalApp("App",0,NULL);//&argc,argv);

  ws->updater_mutex=(pthread_mutex_t*)malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(ws->updater_mutex,NULL);

  //ws->serv=new THttpServer("http:8080");
  ws->canvas = new TCanvas(TString::Format("SimpleScaMonitoring_%s",argv[0]),TString::Format("SimpleScaMonitoring_%s",argv[0]),1000,30,1800,800);
  ws->canvas->Divide(4,4);
 
  ws->NHitsRate_buf0 = new TH2F("NHitsRate_buf0","NHitsRate_buf0",65,0.5,64.5,16,-0.5,15.5);
  ws->NHitsRate = new TH2F("NHitsRate","NHitsRate",65,0.5,64.5,16,-0.5,15.5);
  ws->BCID_buf0 = new TH2F("BCID_buf0","BCID_buf0",42,-50,4150,16,-0.5,15.5);
  ws->BCID = new TH2F("BCID","BCID",42,-50,4150,16,-0.5,15.5);

  ws->RetrigNHitsRate_buf0 = new TH2F("RetrigNHitsRate_buf0","RetrigNHitsRate_buf0",65,0.5,64.5,16,-0.5,15.5);
  ws->RetrigNHitsRate = new TH2F("RetrigNHitsRate","RetrigNHitsRate",65,0.5,64.5,16,-0.5,15.5);
  ws->RetrigBCID_buf0 = new TH2F("RetrigBCID_buf0","RetrigBCID_buf0",42,-50,4150,16,-0.5,15.5);
  ws->RetrigBCID = new TH2F("RetrigBCID","RetrigBCID",42,-50,4150,16,-0.5,15.5);
  
  ws->UnderFlowNHitsRate_buf0 = new TH2F("UnderFlowNHitsRate_buf0","UnderFlowNHitsRate_buf0",65,0.5,64.5,16,-0.5,15.5);
  ws->UnderFlowNHitsRate = new TH2F("UnderFlowNHitsRate","UnderFlowNHitsRate",65,0.5,64.5,16,-0.5,15.5);
  ws->UnderFlowBCID_buf0 = new TH2F("UnderFlowBCID_buf0","UnderFlowBCID_buf0",42,-50,4150,16,-0.5,15.5);
  ws->UnderFlowBCID = new TH2F("UnderFlowBCID","UnderFlowBCID",42,-50,4150,16,-0.5,15.5);

  ws->PlaneNHitsRate_buf0 = new TH2F("PlaneNHitsRate_buf0","PlaneNHitsRate_buf0",65,0.5,64.5,16,-0.5,15.5);
  ws->PlaneNHitsRate = new TH2F("PlaneNHitsRate","PlaneNHitsRate",65,0.5,64.5,16,-0.5,15.5);
  ws->PlaneBCID_buf0 = new TH2F("PlaneBCID_buf0","PlaneBCID_buf0",42,-50,4150,16,-0.5,15.5);
  ws->PlaneBCID = new TH2F("PlaneBCID","PlaneBCID",42,-50,4150,16,-0.5,15.5);

  ws->NHitsRate_buf0->SetStats(kFALSE);
  ws->NHitsRate->SetStats(kFALSE);

  ws->RetrigNHitsRate_buf0->SetStats(kFALSE);
  ws->RetrigNHitsRate->SetStats(kFALSE);

  ws->UnderFlowNHitsRate_buf0->SetStats(kFALSE);
  ws->UnderFlowNHitsRate->SetStats(kFALSE);
  
  ws->canvas->cd(1);
  ws->NHitsRate_buf0->SetTitle("SCA=0");
  ws->NHitsRate_buf0->GetXaxis()->SetTitle("NHits rate");
  ws->NHitsRate_buf0->GetYaxis()->SetTitle("sca");
  ws->NHitsRate_buf0->Draw("colz");
  ws->NHitsRate_buf0->SetStats(kFALSE);
  ws->canvas->cd(1)->Modified();

  ws->canvas->cd(2);
  ws->NHitsRate->SetTitle("SCA>0");
  ws->NHitsRate->GetXaxis()->SetTitle("NHits rate");
  ws->NHitsRate->GetYaxis()->SetTitle("sca");
  ws->NHitsRate->Draw("colz");
  ws->canvas->cd(2)->Modified();

  ws->canvas->cd(3);
  ws->BCID_buf0->SetTitle("SCA=0");
  ws->BCID_buf0->GetXaxis()->SetTitle("BCID");
  ws->BCID_buf0->GetYaxis()->SetTitle("sca");
  ws->BCID_buf0->Draw("colz");
  ws->canvas->cd(3)->Modified();

  ws->canvas->cd(4);
  ws->BCID->SetTitle("SCA>0");
  ws->BCID->GetXaxis()->SetTitle("BCID");
  ws->BCID->GetYaxis()->SetTitle("sca");
  ws->BCID->Draw("colz");
  ws->canvas->cd(4)->Modified();

  ws->canvas->cd(5);
  ws->RetrigNHitsRate_buf0->SetTitle("SCA=0, retriggered");
  ws->RetrigNHitsRate_buf0->GetXaxis()->SetTitle("NHits rate");
  ws->RetrigNHitsRate_buf0->GetYaxis()->SetTitle("sca");
  ws->RetrigNHitsRate_buf0->Draw("colz");
  ws->canvas->cd(5)->Modified();

  ws->canvas->cd(6);
  ws->RetrigNHitsRate->SetTitle("SCA>0, retriggered");
  ws->RetrigNHitsRate->GetXaxis()->SetTitle("NHits rate");
  ws->RetrigNHitsRate->GetYaxis()->SetTitle("sca");
  ws->RetrigNHitsRate->Draw("colz");
  ws->canvas->cd(6)->Modified();


  ws->canvas->cd(7);
  ws->RetrigBCID_buf0->SetTitle("SCA=0, retriggered");
  ws->RetrigBCID_buf0->GetXaxis()->SetTitle("BCID");
  ws->RetrigBCID_buf0->GetYaxis()->SetTitle("sca");
  ws->RetrigBCID_buf0->Draw("colz");
  ws->canvas->cd(7)->Modified();

  ws->canvas->cd(8);
  ws->RetrigBCID->SetTitle("SCA>0, retriggered");
  ws->RetrigBCID->GetXaxis()->SetTitle("BCID");
  ws->RetrigBCID->GetYaxis()->SetTitle("sca");
  ws->RetrigBCID->Draw("colz");
  ws->canvas->cd(8)->Modified();


  ws->canvas->cd(9);
  ws->UnderFlowNHitsRate_buf0->SetTitle("SCA=0, under/overflow");
  ws->UnderFlowNHitsRate_buf0->GetXaxis()->SetTitle("NHits rate");
  ws->UnderFlowNHitsRate_buf0->GetYaxis()->SetTitle("sca");
  ws->UnderFlowNHitsRate_buf0->Draw("colz");
  ws->canvas->cd(9)->Modified();

  ws->canvas->cd(10);
  ws->UnderFlowNHitsRate->SetTitle("SCA>0, under/overflow");
  ws->UnderFlowNHitsRate->GetXaxis()->SetTitle("NHits rate");
  ws->UnderFlowNHitsRate->GetYaxis()->SetTitle("sca");
  ws->UnderFlowNHitsRate->Draw("colz");
  ws->canvas->cd(10)->Modified();


  ws->canvas->cd(11);
  ws->UnderFlowBCID_buf0->SetTitle("SCA=0, under/overflow");
  ws->UnderFlowBCID_buf0->GetXaxis()->SetTitle("BCID");
  ws->UnderFlowBCID_buf0->GetYaxis()->SetTitle("sca");
  ws->UnderFlowBCID_buf0->Draw("colz");
  ws->canvas->cd(11)->Modified();


  ws->canvas->cd(12);
  ws->UnderFlowBCID->SetTitle("SCA>0, under/overflow");
  ws->UnderFlowBCID->GetXaxis()->SetTitle("BCID");
  ws->UnderFlowBCID->GetYaxis()->SetTitle("sca");
  ws->UnderFlowBCID->Draw("colz");
  ws->canvas->cd(12)->Modified();

  ws->canvas->cd(13);
  ws->PlaneNHitsRate_buf0->SetTitle("SCA=0, plane events");
  ws->PlaneNHitsRate_buf0->GetXaxis()->SetTitle("NHits rate");
  ws->PlaneNHitsRate_buf0->GetYaxis()->SetTitle("sca");
  ws->PlaneNHitsRate_buf0->Draw("colz");
  ws->canvas->cd(13)->Modified();

  ws->canvas->cd(14);
  ws->PlaneNHitsRate->SetTitle("SCA>0, plane events");
  ws->PlaneNHitsRate->GetXaxis()->SetTitle("NHits rate");
  ws->PlaneNHitsRate->GetYaxis()->SetTitle("sca");
  ws->PlaneNHitsRate->Draw("colz");
  ws->canvas->cd(14)->Modified();


  ws->canvas->cd(15);
  ws->PlaneBCID_buf0->SetTitle("SCA=0, plane events");
  ws->PlaneBCID_buf0->GetXaxis()->SetTitle("BCID");
  ws->PlaneBCID_buf0->GetYaxis()->SetTitle("sca");
  ws->PlaneBCID_buf0->Draw("colz");
  ws->canvas->cd(15)->Modified();


  ws->canvas->cd(16);
  ws->PlaneBCID->SetTitle("SCA>0, plane events");
  ws->PlaneBCID->GetXaxis()->SetTitle("BCID");
  ws->PlaneBCID->GetYaxis()->SetTitle("sca");
  ws->PlaneBCID->Draw("colz");
  ws->canvas->cd(16)->Modified();

  ws->canvas->Update();
  //ws->canvas->cd(0);

  ws->loop=new_event_loop(argv[1],ws,newevt,newblock,endblock,reinit,'f');
  ws->first_time=1;
  ws->last_sca=-1;
  ws->last_roc=-1;
  run_loop(ws->loop);
  
  ws->updater_t=new TThread("Updater thread",updater,(void *)ws);
  ws->updater_t->Run();

  //run the TApp
  ecalApp.Run(kTRUE);

  //never reached
  return 0;
}

