#include <stdio.h>
#include <TH2F.h>
#include <TThread.h>
#include <TCanvas.h>
#include <TString.h>
#include <TApplication.h>
//#include <THttpServer.h>
#include <pyrame.h>
#include <pthread.h>
#include <TThread.h>

struct or_ws {
  TCanvas *canvas;
  TCanvas *canvas_profile;

  //THttpServer *serv;
  struct evloop **loops;

  //-----------------------------------------------

  TThread *updater_t;
  int dif_number;
  TH1F *h_energy_profile;
  Float_t energy_sum;

  pthread_mutex_t * updater_mutex;
};

struct loop_ws {

  char first_time;
  int dif_number;

  Float_t energy_sum;

  struct or_ws* ows;

};

void *updater(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  while(1) {
    pthread_mutex_lock(ws->updater_mutex);
    //    ws->h_energy_profile->GetYaxis()->SetRangeUser( 0, ws->h_energy_profile->GetMaximum()) );

    ws->canvas_profile->cd()->Modified();
    ws->canvas_profile->Update();

    pthread_mutex_unlock(ws->updater_mutex);
    usleep(50000);
  }
  return NULL;
}

void newblock(void *workspace,struct block *b) {
  struct loop_ws *ws=(struct loop_ws *)workspace;
  //  struct or_ws *global_ws=(struct or_ws *)workspace;

  if (ws->first_time) {
    printf("waiting for app to start\n");
    sleep(1);
    ws->first_time=0;
  }

  //   ws->ows->h_energy_profile->Fill((Float_t)ws->dif_number,(Float_t)ws->energy_sum);  
  //printf("new spill : %s  dif=%d  energy=%f\n",get_block_field(b,"spill"),ws->dif_number,ws->energy_sum);
  //ws->energy_sum=0;


  //print_block(b);
}

void endblock(void *workspace,struct block *b) {
  //  struct loop_ws *ws=(struct loop_ws *)workspace;

}

void reinit(void *workspace) {
  struct loop_ws *ws=(struct loop_ws *)workspace;
  ws->ows->h_energy_profile->Reset();
}


void newevt(void *workspace,struct event *e) {
  struct loop_ws *ws=(struct loop_ws *)workspace;

  pthread_mutex_lock(ws->ows->updater_mutex);

  Double_t energy=(Double_t)atof(get_event_field(e,"en"));
  int hit=atoi(get_event_field(e,"hit"));
  int tag=atoi(get_event_field(e,"tag"));
  int roc=atoi(get_event_field(e,"roc"));
  int bcid=atoi(get_event_field(e,"bcid"));
  int sca=atoi(get_event_field(e,"sca"));


  Float_t x=-((Float_t)atof(get_event_field(e,"x"))-193.2);
  Float_t y=-((Float_t)atof(get_event_field(e,"y"))-105.8);

  /*
    tag list
    #define bxpn_tag 1
    #define plane_tag 2
    #define underflow_tag 4
    #define overflow_tag 8
  */


  //  if(tag==0  && hit==1 && energy>500 && (roc ==12 || roc==13 || roc==15 || roc==14) && bcid >2000 && bcid < 2800) {
  if(tag==0  && hit==1 && energy>500 && bcid < 2800) {

    // ws->energy_sum+=(energy-250.);
    ws->ows->h_energy_profile->Fill(ws->dif_number,energy-300.);  
    //    printf("dif=%d  energy=%f\n",ws->dif_number,ws->energy_sum);
  }

  pthread_mutex_unlock(ws->ows->updater_mutex);

}


int main(int argc, char **argv) {


  //initialize
  struct or_ws *ws=(struct or_ws *)malloc(sizeof(struct or_ws));

  struct cmd_result *result;
  int port;

  port=get_port("cmod");
  if (port == -1) {
    printf("Error getting port number\n");
    return -1;
  }

  // SENDCMD
  result=sendcmd("localhost",port,"get_name_sublist_cmod","dif","root","end");
  if (result->retcode==0) {
    printf("impossible to get the dif list names\n");
    exit(1);
  }

  char** dif_names = split(result->str,',',&(ws->dif_number));

  /// ------------
  TApplication ecalApp("App",0,NULL);

  ws->updater_mutex=(pthread_mutex_t*)malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(ws->updater_mutex,NULL);


  // shower profile plots
  ws->canvas_profile = new TCanvas("Monitoring_profile","Monitoring_profile",600,600);
  ws->h_energy_profile = new TH1F("energy_profile","energy_profile",ws->dif_number+1,-0.5,ws->dif_number+0.5);

  ws->canvas_profile->cd(1);
  ws->h_energy_profile->SetTitle("Shower Profile");
  ws->h_energy_profile->GetXaxis()->SetTitle("layer");
  ws->h_energy_profile->GetYaxis()->SetTitle("ADC (no calib.)");
  ws->h_energy_profile->Draw("h");
  ws->canvas_profile->cd(1)->Modified();


  ws->loops=(struct evloop**)malloc(ws->dif_number*sizeof(struct evloop*));


  for(int i=0;i<ws->dif_number; i++) {

    char tmp[1024];
    sprintf(tmp,"converter_%s",dif_names[i]);
    
    struct loop_ws* newlws;
    newlws=(struct loop_ws*)malloc(sizeof(struct loop_ws));
    newlws->first_time=1;
    newlws->dif_number=i;
    ws->energy_sum=0.;

    newlws->ows=ws;
   

    ws->loops[i]=new_event_loop(tmp,newlws,newevt,newblock,endblock,reinit,'f');  

  }

  ws->canvas_profile->Update();


  for(int i=0;i<ws->dif_number; i++) {
    run_loop(ws->loops[i]);
  }

  ws->updater_t=new TThread("Updater thread",updater,(void *)ws);
  ws->updater_t->Run();

  //run the TApp
  ecalApp.Run(kTRUE);

  //never reached
  return 0;
}

