#include <stdio.h>
#include <TH2F.h>
#include <TThread.h>
#include <TCanvas.h>
#include <TString.h>
#include <TApplication.h>
#include <pyrame.h>
#include <pthread.h>
#include <TThread.h>

struct or_ws {
  char first_time;
  TCanvas *canvas;
  struct evloop *loop;
  TH1F * histo_nb_hits;
  TH1F * histo_tbh;
  TH1F * histo_bcid;
  int hit_counter;
  int last_spill;
  int last_bcid;
  TThread *updater_t;
  TString filename;
  pthread_mutex_t * updater_mutex;
};


void *updater(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  while(1) {
    pthread_mutex_lock(ws->updater_mutex);
    for (int i=1;i<=3;i++) {
      ws->canvas->cd(i)->Modified();
    }
    ws->canvas->Update();
    pthread_mutex_unlock(ws->updater_mutex);
    usleep(100);
  }
  return NULL;
}

void newblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;
  if (ws->first_time) {
    printf("waiting for app to start\n");
    usleep(1000000);
    ws->first_time=0;
  }
  pthread_mutex_lock(ws->updater_mutex);
  ws->histo_nb_hits->Fill(ws->hit_counter);
  ws->hit_counter=0;
  pthread_mutex_unlock(ws->updater_mutex);
}

void endblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;

  pthread_mutex_lock(ws->updater_mutex);

  pthread_mutex_unlock(ws->updater_mutex);
  usleep(1000);

}

void reinit(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;

  ws->histo_nb_hits->Reset();
  ws->histo_tbh->Reset();

}



void newevt(void *workspace,struct event *e) {
  struct or_ws *ws=(struct or_ws *)workspace;

  pthread_mutex_lock(ws->updater_mutex);

  int diff;
  int hit=atoi(get_event_field(e,"hit"));
  int tag=atoi(get_event_field(e,"tag"));
  int spill=atoi(get_event_field(e,"spill"));
  int bcid=atoi(get_event_field(e,"bcid"));

  if(tag==0 && hit==1) {

    ws->hit_counter++;
    if (ws->last_spill==spill) {
      diff=bcid-ws->last_bcid;
      ws->histo_tbh->Fill(diff);
    }
    ws->histo_bcid->Fill(bcid);
    ws->last_spill=spill;
    ws->last_bcid=bcid;
  }

  pthread_mutex_unlock(ws->updater_mutex);

}


int main(int argc, char **argv) {

  if (argc!=2) {
    printf("usage : %s data_source_name\n",argv[0]);
    exit(1);
  }
  
  //initialize
  struct or_ws *ws=(struct or_ws *)malloc(sizeof(struct or_ws));
  TApplication ecal_app("App",0,NULL);//&argc,argv);

  ws->updater_mutex=(pthread_mutex_t*)malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(ws->updater_mutex,NULL);

  ws->filename=TString::Format("Hitmap_%s",argv[1]);
  ws->canvas = new TCanvas(ws->filename,ws->filename,1000,30,700,200);
  ws->canvas->Divide(3,1);

  ws->histo_nb_hits=new TH1F("histo_nb_hits","histo_nb_hits",80,(Double_t)0,(Double_t)25);
  ws->histo_tbh=new TH1F("histo_tbh","histo_tbh",256,(Double_t)1,(Double_t)8500);
  ws->histo_bcid=new TH1F("histo_bcid","histo_bcid",256,(Double_t)1,(Double_t)12300);

  ws->canvas->cd(1);
  ws->histo_nb_hits->SetTitle("Hits per spill");
  ws->histo_nb_hits->GetXaxis()->SetTitle("#hits/spill");
  ws->histo_nb_hits->GetYaxis()->SetTitle("# spill");
  ws->histo_nb_hits->Draw("h");
  ws->histo_nb_hits->SetLineColor(kMagenta);
  ws->canvas->cd(1)->Modified();

 ws->canvas->cd(2);
 ws->histo_tbh->SetTitle("Time between hits");
 ws->histo_tbh->GetXaxis()->SetTitle("duration");
 ws->histo_tbh->GetYaxis()->SetTitle("# hits");
 ws->canvas->cd(2)->SetLogy();
 ws->histo_tbh->SetLineColor(kGreen);
 ws->histo_tbh->Draw("h");
 ws->canvas->cd(2)->Modified();

 ws->canvas->cd(3);
 ws->histo_bcid->SetTitle("Bcid repartition");
 ws->histo_bcid->GetXaxis()->SetTitle("bcid");
 ws->histo_bcid->GetYaxis()->SetTitle("# hits");
 ws->histo_bcid->SetLineColor(kAzure);
 ws->histo_bcid->Draw("h");
 ws->canvas->cd(3)->Modified();
 
 ws->canvas->Update();
 
 ws->loop=new_event_loop(argv[1],ws,newevt,newblock,endblock,reinit,'f');
 ws->first_time=1;
 ws->hit_counter=0;
 ws->last_spill=-1;
 ws->last_bcid=0;
 
 run_loop(ws->loop);
 
 ws->updater_t=new TThread("Updater thread",updater,(void *)ws);
 ws->updater_t->Run();
 
 //run the TApp
 ecal_app.Run(kFALSE);
 
 //never reached
 return 0;
}

