#include <stdio.h>
#include <TH2F.h>
#include <TThread.h>
#include <TCanvas.h>
#include <TString.h>
#include <TApplication.h>
#include <pyrame.h>
#include <pthread.h>
#include <TThread.h>

struct or_ws {
  char first_time;
  TCanvas *canvas;
  //THttpServer *serv;
  struct evloop *loop;

  // maps, SCA, triggers
  TH2F * map_hits;
  TH2F * map_energy;

  TH1F * histo_hits;
  TH1F * histo_hits_tagged;

  //-----------------------------------------------
  Float_t n_spills_reset;

  TThread *updater_t;
  TString filename;

  pthread_mutex_t * updater_mutex;
};


void *updater(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  while(1) {
    pthread_mutex_lock(ws->updater_mutex);
    ws->histo_hits->GetYaxis()->SetRangeUser( 0, TMath::Max(ws->histo_hits->GetMaximum(),ws->histo_hits_tagged->GetMaximum()) );
    for (int i=1;i<=3;i++) {
      ws->canvas->cd(i)->Modified();
    }
    ws->canvas->Update();
    //    ws->canvas->Print(ws->filename+".png");
    pthread_mutex_unlock(ws->updater_mutex);
    usleep(100);
  }
  return NULL;
}

void newblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;
  if (ws->first_time) {
    printf("waiting for app to start\n");
    usleep(100);
    ws->first_time=0;
  }
  ws->n_spills_reset++;

}

void endblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;

  pthread_mutex_lock(ws->updater_mutex);

  if(ws->n_spills_reset>600) {
      ws->n_spills_reset=0;
      ws->map_hits->Reset();
      ws->map_energy->Reset();
  }
  pthread_mutex_unlock(ws->updater_mutex);
  usleep(1000);

}

void reinit(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;

  ws->histo_hits->Reset();
  ws->histo_hits_tagged->Reset();

}



void newevt(void *workspace,struct event *e) {
  struct or_ws *ws=(struct or_ws *)workspace;

  pthread_mutex_lock(ws->updater_mutex);

  Double_t energy=(Double_t)atof(get_event_field(e,"en"));
  int hit=atoi(get_event_field(e,"hit"));
  int tag=atoi(get_event_field(e,"tag"));
  int roc=atoi(get_event_field(e,"roc"));
  Float_t x=-((Float_t)atof(get_event_field(e,"x"))-193.2);
  Float_t y=-((Float_t)atof(get_event_field(e,"y"))-105.8);

  /*
    tag list
    #define bxpn_tag 1
    #define plane_tag 2
    #define underflow_tag 4
    #define overflow_tag 8
  */

  //if(ws->n_spills_reset==1) printf("hit=%d roc=%d sca=%d chn=%d tag=%d\n",hit,roc,sca,chn,tag);

  if(tag==0 && hit==1) {
    Float_t energy_per_spill=0.;
    Float_t hits_per_spill=0.;
    TAxis *xaxis = ws->map_hits->GetXaxis();
    TAxis *yaxis = ws->map_hits->GetYaxis();
    Int_t binx =xaxis->FindBin(x);
    Int_t biny =yaxis->FindBin(y);
    if(ws->n_spills_reset>0) {
      if(ws->map_energy->GetBinContent(binx,biny)>0) 
	energy_per_spill=(Float_t)(ws->map_energy->GetBinContent(binx,biny))*(ws->n_spills_reset-1.);
      energy_per_spill=(Float_t)(energy_per_spill + (Float_t)energy) / ws->n_spills_reset;

      hits_per_spill=ws->map_hits->GetBinContent(binx,biny)*(ws->n_spills_reset-1);
      hits_per_spill=(hits_per_spill + 1.) / ws->n_spills_reset;
    }

    ws->map_hits->SetBinContent(binx,biny,hits_per_spill);
    ws->map_energy->SetBinContent(binx,biny,(Float_t)energy_per_spill);
    // printf("energy=%f\n",energy_per_spill);

  }

  if(tag==0 && hit==1)
    ws->histo_hits->Fill(roc);


  if(tag!=0 && hit==1)
    ws->histo_hits_tagged->Fill(roc);

  pthread_mutex_unlock(ws->updater_mutex);

}


int main(int argc, char **argv) {

  if (argc!=2) {
    printf("usage : %s data_source_name\n",argv[0]);
    exit(1);
  }
  
  //initialize
  struct or_ws *ws=(struct or_ws *)malloc(sizeof(struct or_ws));
  TApplication ecal_app("App",0,NULL);//&argc,argv);

  ws->updater_mutex=(pthread_mutex_t*)malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(ws->updater_mutex,NULL);

  ws->filename=TString::Format("Hitmap_%s",argv[1]);

  //ws->serv=new THttpServer("http:8080");
  ws->canvas = new TCanvas(ws->filename,ws->filename,1000,30,700,200);
  ws->canvas->Divide(3,1);
 
  ws->map_hits = new TH2F("map_hits","map_hits",32,-86.5,86.5,32,-86.5,86.5);
  ws->map_energy = new TH2F("map_energy","map_energy",32,-86.5,86.5,32,-86.5,86.5);

  ws->histo_hits = new TH1F("histo_hits","histo_hits",17,-0.5,16.5);
  ws->histo_hits_tagged = new TH1F("histo_hits_tagged","histo_hits_tagged",16,-0.5,15.5);

  ws->canvas->cd(1);
  ws->histo_hits->SetTitle("Hit counter");
  ws->histo_hits->GetXaxis()->SetTitle("chip");
  ws->histo_hits->GetYaxis()->SetTitle("# hits");
  ws->histo_hits->Draw("h");
  ws->histo_hits_tagged->SetLineColor(2);
  ws->histo_hits_tagged->Draw("hsame");
  ws->histo_hits->SetStats(kFALSE);
  ws->canvas->cd(1)->Modified();

 ws->canvas->cd(2);
  ws->map_hits->SetTitle("HitMap - Events per SCA");
  ws->map_hits->GetXaxis()->SetTitle("x");
  ws->map_hits->GetYaxis()->SetTitle("y");
  ws->map_hits->Draw("colz");
  ws->map_hits->SetStats(kFALSE);
  ws->canvas->cd(2)->Modified();

  ws->canvas->cd(3);
  ws->map_energy->SetTitle("EnergyMap - Events per SCA");
  ws->map_energy->GetXaxis()->SetTitle("x");
  ws->map_energy->GetYaxis()->SetTitle("y");
  ws->map_energy->Draw("colz");
  ws->map_energy->SetStats(kFALSE);
  ws->canvas->cd(3)->Modified();


  ws->canvas->Update();
  //ws->canvas->cd(0);

  ws->loop=new_event_loop(argv[1],ws,newevt,newblock,endblock,reinit,'f');
  ws->first_time=1;
  ws->n_spills_reset=0;

  run_loop(ws->loop);
  
  ws->updater_t=new TThread("Updater thread",updater,(void *)ws);
  ws->updater_t->Run();

  //run the TApp
  ecal_app.Run(kFALSE);

  //never reached
  return 0;
}

