#include <stdio.h>
#include <TH2F.h>
#include <TThread.h>
#include <TCanvas.h>
#include <TString.h>
#include <TApplication.h>
#include <pyrame.h>
#include <pthread.h>
#include <TThread.h>

struct or_ws {
  char first_time;
  TCanvas *canvas;
  //THttpServer *serv;
  struct evloop *loop;

  // maps, SCA, triggers
  TH2F * map_energy;
  //-----------------------------------------------
  Float_t n_spills_reset;

  TThread *updater_t;
  TString filename;

  pthread_mutex_t * updater_mutex;
};


void *updater(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  while(1) {
    pthread_mutex_lock(ws->updater_mutex);
    ws->canvas->cd()->Modified();

    ws->canvas->Update();
    pthread_mutex_unlock(ws->updater_mutex);
    usleep(500000);
  }
  return NULL;
}

void newblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;
  if (ws->first_time) {
    printf("waiting for app to start\n");
    usleep(5000);
    ws->first_time=0;
  }
  ws->n_spills_reset++;

}

void endblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;

  pthread_mutex_lock(ws->updater_mutex);

  if(ws->n_spills_reset>600) {
      ws->map_energy->Reset();
  }
  pthread_mutex_unlock(ws->updater_mutex);
  usleep(50000);

}

void reinit(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  ws->map_energy->Reset();
}



void newevt(void *workspace,struct event *e) {
  struct or_ws *ws=(struct or_ws *)workspace;

  pthread_mutex_lock(ws->updater_mutex);

  Double_t energy=(Double_t)(atof(get_event_field(e,"en"))-280.);
  int hit=atoi(get_event_field(e,"hit"));
  int tag=atoi(get_event_field(e,"tag"));
  Float_t x=-((Float_t)atof(get_event_field(e,"x"))-193.2);
  Float_t y=-((Float_t)atof(get_event_field(e,"y"))-105.8);

  /*
    tag list
    #define bxpn_tag 1
    #define plane_tag 2
    #define underflow_tag 4
    #define overflow_tag 8
  */


  if(tag==0 && hit==1) {
    Float_t energy_per_spill=0.;
    TAxis *xaxis = ws->map_energy->GetXaxis();
    TAxis *yaxis = ws->map_energy->GetYaxis();
    Int_t binx =xaxis->FindBin(x);
    Int_t biny =yaxis->FindBin(y);
    if(ws->n_spills_reset>0) {
      if(ws->map_energy->GetBinContent(binx,biny)>0) 
	energy_per_spill=(Float_t)(ws->map_energy->GetBinContent(binx,biny))*(ws->n_spills_reset-1.);
      energy_per_spill=(Float_t)(energy_per_spill + (Float_t)energy) / ws->n_spills_reset;
    }

    ws->map_energy->SetBinContent(binx,biny,(Float_t)energy_per_spill);

  }

  pthread_mutex_unlock(ws->updater_mutex);

}


int main(int argc, char **argv) {

  if (argc!=2) {
    printf("usage : %s data_source_name\n",argv[0]);
    exit(1);
  }
  
  //initialize
  struct or_ws *ws=(struct or_ws *)malloc(sizeof(struct or_ws));
  TApplication ecal_app("App",0,NULL);//&argc,argv);

  
  ws->updater_mutex=(pthread_mutex_t*)malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(ws->updater_mutex,NULL);

  ws->filename=TString::Format("energy_map_%s",argv[1]);

  //ws->serv=new THttpServer("http:8080");
  ws->canvas = new TCanvas(ws->filename,ws->filename,1000,30,300,250);
 
  ws->map_energy = new TH2F("map_energy","map_energy",32,-86.5,86.5,32,-86.5,86.5);

  ws->canvas->cd(1);
  ws->map_energy->SetTitle(TString::Format("Energy map weighted per SCA, %s",argv[1]));
  ws->map_energy->GetXaxis()->SetTitle("x");
  ws->map_energy->GetYaxis()->SetTitle("y");
  ws->map_energy->Draw("colz");
  ws->map_energy->SetStats(kFALSE);
  ws->canvas->cd(3)->Modified();


  ws->canvas->Update();
  //ws->canvas->cd(0);

  ws->loop=new_event_loop(argv[1],ws,newevt,newblock,endblock,reinit,'f');
  ws->first_time=1;
  ws->n_spills_reset=0;

  run_loop(ws->loop);
  
  ws->updater_t=new TThread("Updater thread",updater,(void *)ws);
  ws->updater_t->Run();

  //run the TApp
  ecal_app.Run(kFALSE);

  //never reached
  return 0;
}

