#include <stdio.h>
#include <TH2F.h>
#include <TThread.h>
#include <TCanvas.h>
#include <TString.h>
#include <TApplication.h>
//#include <THttpServer.h>
#include <pyrame.h>
#include <pthread.h>
#include <TThread.h>

struct or_ws {
  char first_time;
  TCanvas *canvas;
  //THttpServer *serv;
  struct evloop *loop;

  // maps, SCA, triggers
  TH2F * map_energy;
  TH2F * map_energy_retrig;
  TH2F * map_energy_plane;
  TH2F * map_energy_underflow;

  //bcid stuff
  TH2F * map_bcid;//bcid distribution for SCA >0
  TH2F * map_bcid_retrig;
  TH2F * map_bcid_plane;
  TH2F * map_bcid_underflow;

 //lastsca stuff
  TH2F * map_lastsca;//lastsca distribution for SCA >0
  TH2F * map_lastsca_retrig;
  TH2F * map_lastsca_plane;
  TH2F * map_lastsca_underflow;

  //-----------------------------------------------

  TThread *updater_t;

  int last_sca;
  int last_roc;
  int hit_counter;
  int hit_counter_retrig;
  int hit_counter_plane;
  int hit_counter_under;

  pthread_mutex_t * updater_mutex;
};


void *updater(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  while(1) {
    pthread_mutex_lock(ws->updater_mutex);
    for (int i=1;i<=12;i++) {
      ws->canvas->cd(i)->Modified();
    }
    ws->canvas->Update();
    pthread_mutex_unlock(ws->updater_mutex);
    usleep(50000);
  }
  return NULL;
}

void newblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;
  if (ws->first_time) {
    printf("waiting for app to start\n");
    sleep(1);
    ws->first_time=0;
  }

}

void endblock(void *workspace,struct block *b) {
}

void reinit(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;

  ws->map_energy_underflow->Reset();
  ws->map_energy->Reset();
  ws->map_energy_retrig->Reset();
  ws->map_energy_plane->Reset();

  ws->map_bcid->Reset();
  ws->map_bcid_retrig->Reset();
  ws->map_bcid_plane->Reset();
  ws->map_bcid_underflow->Reset();

  ws->map_lastsca->Reset();
  ws->map_lastsca_retrig->Reset();
  ws->map_lastsca_plane->Reset();
  ws->map_lastsca_underflow->Reset();

}



void newevt(void *workspace,struct event *e) {
  struct or_ws *ws=(struct or_ws *)workspace;

  pthread_mutex_lock(ws->updater_mutex);

  Double_t energy=(Double_t)atof(get_event_field(e,"en"));
  int hit=atoi(get_event_field(e,"hit"));
  int roc=atoi(get_event_field(e,"roc"));
  int sca=atoi(get_event_field(e,"sca"));
  int bcid=atoi(get_event_field(e,"bcid"));
  int tag=atoi(get_event_field(e,"tag"));
  Float_t x=-((Float_t)atof(get_event_field(e,"x"))-193.2);
  Float_t y=-((Float_t)atof(get_event_field(e,"y"))-105.8);
  /*
    tag list
    #define bxpn_tag 1
    #define plane_tag 2
    #define underflow_tag 4
    #define overflow_tag 8
  */


  if(tag==0 && hit==1) 
    ws->map_energy->Fill(x,y,energy);

  if(tag==1 && hit==1)
    ws->map_energy_retrig->Fill(x,y,energy);


  if(tag==2 && hit==1) 
    ws->map_energy_plane->Fill(x,y,energy);

  if( (tag==4 || tag ==8 ) && hit==1) 
    ws->map_energy_underflow->Fill(x,y,energy);


  if (sca!=ws->last_sca || roc!=ws->last_roc) {
  
    if(tag==0) ws->map_lastsca->Fill(sca,roc);
    if(tag==1) ws->map_lastsca_retrig->Fill(sca,roc);
    if(tag==2) ws->map_lastsca_plane->Fill(sca,roc);
    if(tag==4 || tag==8) ws->map_lastsca_underflow->Fill(sca,roc);
  
    ws->hit_counter_retrig=0;
    ws->hit_counter_plane=0;
    ws->hit_counter_under=0;
    ws->last_sca=sca;
    ws->last_roc=roc;
  }

  if (hit) {

    if(tag==0) ws->hit_counter++;
    if(tag==1) ws->hit_counter_retrig++; 
    if(tag==2) ws->hit_counter_plane++;
    if(tag==4 || tag==8) ws->hit_counter_under++;

    if(tag==0) ws->map_bcid->Fill(bcid,roc);
    if(tag==1) ws->map_bcid_retrig->Fill(bcid,roc);
    if(tag==2) ws->map_bcid_plane->Fill(bcid,roc);
    if(tag==4 || tag==8) ws->map_bcid_underflow->Fill(bcid,roc);
    
  }

  pthread_mutex_unlock(ws->updater_mutex);

}


int main(int argc, char **argv) {

  if (argc!=2) {
    printf("usage : %s data_source_name\n",argv[0]);
    exit(1);
  }
  
  //initialize
  struct or_ws *ws=(struct or_ws *)malloc(sizeof(struct or_ws));
  TApplication ecal_app("App",0,NULL);//&argc,argv);

  ws->updater_mutex=(pthread_mutex_t*)malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(ws->updater_mutex,NULL);

  //ws->serv=new THttpServer("http:8080");
  ws->canvas = new TCanvas(TString::Format("Monitoring_%s",argv[1]),TString::Format("Monitoring_%s",argv[1]),1000,30,1800,800);
  ws->canvas->Divide(4,3);
 
  ws->map_energy_underflow = new TH2F("map_SCA","map_SCA",32,-86.5,86.5,32,-86.5,86.5);
  ws->map_energy_retrig = new TH2F("map_SCA_bad","map_SCA_bad",32,-86.5,86.5,32,-86.5,86.5);//16,-0.5,15.5,64,-0.5,63.5);//
  ws->map_energy = new TH2F("map_energy","map_energy",32,-86.5,86.5,32,-86.5,86.5);
  ws->map_energy_plane = new TH2F("map_energy_bad","map_energy_bad",32,-86.5,86.5,32,-86.5,86.5);

  ws->map_bcid = new TH2F("map_bcid","map_bcid",42,-50,4150,16,-0.5,15.5);
  ws->map_bcid_retrig = new TH2F("map_bcid_retrig","map_bcid_retrig",42,-50,4150,16,-0.5,15.5);
  ws->map_bcid_plane = new TH2F("map_bcid_plane","map_bcid_plane",42,-50,4150,16,-0.5,15.5);
  ws->map_bcid_underflow = new TH2F("map_bcid_underflow","map_bcid_underflow",42,-50,4150,16,-0.5,15.5);

  ws->map_lastsca = new TH2F("map_lastsca","map_lastsca",15,-0.5,14.5,16,-0.5,15.5);
  ws->map_lastsca_retrig = new TH2F("map_lastsca_retrig","map_lastsca_retrig",15,-0.5,14.5,16,-0.5,15.5);
  ws->map_lastsca_plane = new TH2F("map_lastsca_plane","map_lastsca_plane",15,-0.5,14.5,16,-0.5,15.5);
  ws->map_lastsca_underflow = new TH2F("map_lastsca_underflow","map_lastsca_underflow",15,-0.5,14.5,16,-0.5,15.5);

  ws->canvas->cd(1);
  ws->map_energy_retrig->SetTitle("HitMap Regtriggers");
  ws->map_energy_retrig->GetXaxis()->SetTitle("x");
  ws->map_energy_retrig->GetYaxis()->SetTitle("y");
  ws->map_energy_retrig->Draw("colz");
  ws->map_energy_retrig->SetStats(kFALSE);
  ws->canvas->cd(1)->Modified();

  ws->canvas->cd(2);
  ws->map_energy_plane->SetTitle("HitMap PlaneEvents");
  ws->map_energy_plane->GetXaxis()->SetTitle("x");
  ws->map_energy_plane->GetYaxis()->SetTitle("y");
  ws->map_energy_plane->Draw("colz");
  ws->map_energy_plane->SetStats(kFALSE);
  ws->canvas->cd(2)->Modified();


  ws->canvas->cd(3);
  ws->map_energy_underflow->SetTitle("HitMap over/under flow");
  ws->map_energy_underflow->GetXaxis()->SetTitle("x");
  ws->map_energy_underflow->GetYaxis()->SetTitle("y");
  ws->map_energy_underflow->Draw("colz");
  ws->map_energy_underflow->SetStats(kFALSE);
  ws->canvas->cd(3)->Modified();

  ws->canvas->cd(4);
  ws->map_energy->SetTitle("HitMap good");
  ws->map_energy->GetXaxis()->SetTitle("x");
  ws->map_energy->GetYaxis()->SetTitle("y");
  ws->map_energy->Draw("colz");
  ws->map_energy->SetStats(kFALSE);
  ws->canvas->cd(4)->Modified();

  ws->canvas->cd(5);
  ws->map_bcid_retrig->SetTitle("BCID Regtriggers");
  ws->map_bcid_retrig->GetXaxis()->SetTitle("bcid");
  ws->map_bcid_retrig->GetYaxis()->SetTitle("chip");
  ws->map_bcid_retrig->Draw("colz");
  ws->map_bcid_retrig->SetStats(kFALSE);
  ws->canvas->cd(5)->Modified();

  ws->canvas->cd(6);
  ws->map_bcid_plane->SetTitle("BCID PlaneEvents");
  ws->map_bcid_plane->GetXaxis()->SetTitle("bcid");
  ws->map_bcid_plane->GetYaxis()->SetTitle("chip");
  ws->map_bcid_plane->Draw("colz");
  ws->map_bcid_plane->SetStats(kFALSE);
  ws->canvas->cd(6)->Modified();

  ws->canvas->cd(7);
  ws->map_bcid_underflow->SetTitle("BCID over/under flow");
  ws->map_bcid_underflow->GetXaxis()->SetTitle("bcid");
  ws->map_bcid_underflow->GetYaxis()->SetTitle("chip");
  ws->map_bcid_underflow->Draw("colz");
  ws->map_bcid_underflow->SetStats(kFALSE);
  ws->canvas->cd(7)->Modified();

  ws->canvas->cd(8);
  ws->map_bcid->SetTitle("BCID good");
  ws->map_bcid->GetXaxis()->SetTitle("bcid");
  ws->map_bcid->GetYaxis()->SetTitle("chip");
  ws->map_bcid->Draw("colz");
  ws->map_bcid->SetStats(kFALSE);
  ws->canvas->cd(8)->Modified();


  ws->canvas->cd(9);
  ws->map_lastsca_retrig->SetTitle("LASTSCA Regtriggers");
  ws->map_lastsca_retrig->GetXaxis()->SetTitle("lastsca");
  ws->map_lastsca_retrig->GetYaxis()->SetTitle("chip");
  ws->map_lastsca_retrig->Draw("colz");
  ws->map_lastsca_retrig->SetStats(kFALSE);
  ws->canvas->cd(5)->Modified();

  ws->canvas->cd(10);
  ws->map_lastsca_plane->SetTitle("LASTSCA PlaneEvents");
  ws->map_lastsca_plane->GetXaxis()->SetTitle("lastsca");
  ws->map_lastsca_plane->GetYaxis()->SetTitle("chip");
  ws->map_lastsca_plane->Draw("colz");
  ws->map_lastsca_plane->SetStats(kFALSE);
  ws->canvas->cd(6)->Modified();

  ws->canvas->cd(11);
  ws->map_lastsca_underflow->SetTitle("LASTSCA over/under flow");
  ws->map_lastsca_underflow->GetXaxis()->SetTitle("lastsca");
  ws->map_lastsca_underflow->GetYaxis()->SetTitle("chip");
  ws->map_lastsca_underflow->Draw("colz");
  ws->map_lastsca_underflow->SetStats(kFALSE);
  ws->canvas->cd(7)->Modified();

  ws->canvas->cd(12);
  ws->map_lastsca->SetTitle("LASTSCA good");
  ws->map_lastsca->GetXaxis()->SetTitle("lastsca");
  ws->map_lastsca->GetYaxis()->SetTitle("chip");
  ws->map_lastsca->Draw("colz");
  ws->map_lastsca->SetStats(kFALSE);
  ws->canvas->cd(8)->Modified();






  ws->canvas->Update();
  //ws->canvas->cd(0);

  ws->loop=new_event_loop(argv[1],ws,newevt,newblock,endblock,reinit,'f');
  ws->first_time=1;
  ws->last_sca=-1;
  ws->last_roc=-1;
  run_loop(ws->loop);
  
  ws->updater_t=new TThread("Updater thread",updater,(void *)ws);
  ws->updater_t->Run();

  //run the TApp
  ecal_app.Run(kFALSE);

  //never reached
  return 0;
}

