#include <stdio.h>


#include <TMath.h>
#include <TGraph2D.h>
#include <TRandom2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TF2.h>
#include <TH1.h>
#include <TVirtualFitter.h>
#include <TPolyLine3D.h>
#include <Math/Vector3D.h>
#include <TApplication.h>
#include <TThread.h>
#include <bindpyrame.h>
#include <datapyrame.h>
#include "../../ecal/ecal.h"

using namespace ROOT::Math;

struct or_ws {
  TCanvas *canvas;
  TGraph2D *graph;
  TPolyLine3D *l;
  int mute;
  int nbp;
  char init;
  struct evloop *loop;
};

// define the parameteric line equation 
void line(double t, double *p, double &x, double &y, double &z) { 
   // a parameteric line is define from 6 parameters but 4 are independent
   // x0,y0,z0,z1,y1,z1 which are the coordinates of two points on the line
   // can choose z0 = 0 if line not parallel to x-y plane and z1 = 1; 
   x = p[0] + p[1]*t; 
   y = p[2] + p[3]*t;
   z = t; 
}

// calculate distance line-point 
double distance2(double x,double y,double z, double *p) { 
   // distance line point is D= | (xp-x0) cross  ux | 
   // where ux is direction of line and x0 is a point in the line (like t = 0) 
   XYZVector xp(x,y,z); 
   XYZVector x0(p[0], p[2], 0. ); 
   XYZVector x1(p[0] + p[1], p[2] + p[3], 1. ); 
   XYZVector u = (x1-x0).Unit(); 
   double d2 = ((xp-x0).Cross(u)) .Mag2(); 
   return d2; 
}
bool first = true;

// function to be minimized 
void SumDistance2(int &, double *, double & sum, double * par, int ) { 
   // the TGraph must be a global variable
   TGraph2D * gr = dynamic_cast<TGraph2D*>( (TVirtualFitter::GetFitter())->GetObjectFit() );
   assert(gr != 0);
   double * x = gr->GetX();
   double * y = gr->GetY();
   double * z = gr->GetZ();
   int npoints = gr->GetN();
   sum = 0;
   for (int i  = 0; i < npoints; ++i) { 
      double d = distance2(x[i],y[i],z[i],par); 
      sum += d;
#ifdef DEBUG
      if (first) std::cout << "point " << i << "\t" 
                           << x[i] << "\t" 
                           << y[i] << "\t" 
                           << z[i] << "\t" 
                           << std::sqrt(d) << std::endl; 
#endif
   }
   if (first) 
      std::cout << "Total sum2 = " << sum << std::endl;
   first = false;
}

void line3Dfit(void *workspace)
{
  struct or_ws *ws=(struct or_ws *)workspace;
  gStyle->SetOptStat(0);
  gStyle->SetOptFit();
   
  TVirtualFitter *min = TVirtualFitter::Fitter(0,4);
  min->SetObjectFit(ws->graph);
  min->SetFCN(SumDistance2);
  
  
  Double_t arglist[10];
  arglist[0] = 3;
  min->ExecuteCommand("SET PRINT",arglist,1);
  
  double pStart[4] = {1,1,1,1};
  min->SetParameter(0,"x0",pStart[0],0.01,0,0);
  min->SetParameter(1,"Ax",pStart[1],0.01,0,0);
  min->SetParameter(2,"y0",pStart[2],0.01,0,0);
  min->SetParameter(3,"Ay",pStart[3],0.01,0,0);
  
  arglist[0] = 1000; // number of function calls 
  arglist[1] = 0.001; // tolerance 
  min->ExecuteCommand("MIGRAD",arglist,2);
  
  // get fit parameters
  double parFit[4];
  for (int i = 0; i <4; ++i) 
    parFit[i] = min->GetParameter(i);

  //compute the angle
  double x1,y1,z1;
  double t1=0;
  line(t1,parFit,x1,y1,z1);
  printf("first coord %g,%g,%g\n",x1,y1,z1);
  double x2,y2,z2;
  double t2=1000;
  line(t2,parFit,x2,y2,z2);
  printf("second coord %g,%g,%g\n",x2,y2,z2);
  double dist=std::sqrt((x1-x2)*(x1-x2)+(y1-y2)*(y1-y2));
  printf("dist=%g\n",dist);
  double angle=TMath::ATan(dist/1000)*180.0/Pi();
  printf("angle=%d\n",(int)angle);
  

  
  // draw the fitted line
  int n=1000;
  double t0=0;
  double dt=1;
  for (int i=0;i<n;i++) {
    double t=t0+dt*i;
    double x,y,z;
    line(t,parFit,x,y,z);
    ws->l->SetPoint(i,x,y,z);
  }
}

#define MAX_POINTS 3


void newblock_func(void *workspace,struct oel *el) {
  struct or_ws *ws=(struct or_ws *)workspace;

  if ((ws->init==0) && (ws->mute==0)) {
    line3Dfit(ws);
    printf("hit a key to go on\n");
    getchar();
    printf("going on\n");
  }

  
  printf("new block with %d points\n",atoi(el->props[1]));
  if (atoi(el->props[0])!=1) {
    ws->mute=1;
  } else {
    if (ws->init==1) {
      ws->init=0;
    }
    ws->mute=0;
    for(int i=0;i<MAX_POINTS;i++)
      ws->graph->SetPoint(i,0.0,0.0,0.0);
    ws->nbp=0;
  }
}


void treatevt_func(void *workspace,struct event *e) {
  struct or_ws *ws=(struct or_ws *)workspace;

  if (ws->mute)
    return;
  
  //fill the histo with the energy
  Double_t x,y,z;
  if (ws->nbp==MAX_POINTS) {
    printf("CAUTION : removing extra points\n");
    return;
  }


  /*if (verif_integrity(e)) {
    printf("bad event ->throwing away\n");
    return;
    }*/
  
  x=(Double_t)atoi(e->space[3]);
  y=(Double_t)atoi(e->space[5]);
  z=(Double_t)atoi(e->space[7]);
  printf("adding point %d : %g %g %g\n",ws->nbp,x,y,z);
  ws->graph->SetPoint(ws->nbp,x,y,z);
  ws->nbp++;
}

int main(int argc, char **argv) {

  //initialize
  struct or_ws *ws=(struct or_ws *)malloc(sizeof(struct or_ws));
  ws->init=1;
  TApplication ecalApp("App",&argc,argv);
  
  ws->canvas=new TCanvas("ecal","Ecal Event Display",600,600);
  ws->canvas->Divide(1,1);
  ws->canvas->cd(1);

  ws->graph=new TGraph2D(3);
  ws->graph->SetNpy(40);
  ws->graph->SetNpx(40);
  //gStyle->SetPalette(1);
  ws->graph->SetMarkerStyle(20);
  for(int i=0;i<3;i++)
    ws->graph->SetPoint(i,500.0*i,500.0*i,500.0*i);
  ws->graph->Draw("PCOL");

  ws->l=new TPolyLine3D(MAX_POINTS);
  line3Dfit(ws);
  ws->l->SetLineColor(kRed);
  ws->l->Draw("same");

  ws->canvas->cd(0);
  ws->canvas->Update();

  ws->loop=new_event_loop((char *)"localhost",9116,ws,treatevt_func,newblock_func,NULL,FIRST);
  run_loop(ws->loop);

  //run the TApp
  ecalApp.Run();

  //never reached
  return 0;
}

