#include <stdio.h>
#include <TGraph2D.h>
#include <TThread.h>
#include <TCanvas.h>
#include <TString.h>
#include <TApplication.h>
#include <TAxis.h>
#include <TGFrame.h>
#include <pthread.h>
#include "../../ecal/ecal.h"
#include "/opt/pyrame/cmdmod_helper.hpp"

struct or_ws {
  TCanvas *canvas;
  TGraph2D *graph;
  int nbp;
  char inter;
  TApplication *ecalApp;
  pthread_t *t_run;
  struct evloop *loop;
  int mute;
};

#define MAX_POINTS 1000

#define MIN_X -100.0
#define MAX_X 100.0
#define MIN_Y -100.0
#define MAX_Y 100.0
#define MIN_Z 0.0
#define MAX_Z 60.0

extern "C" struct cmd_result *stop_ed(struct cmd *,void *);
struct cmd_result *stop_ed(struct cmd *command,void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  stop_loop(ws->loop);
  return build_result(1,"ok");
}

extern "C" struct cmd_result *run_ed(struct cmd *,void *);
struct cmd_result *run_ed(struct cmd *command,void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  run_loop(ws->loop);
  return build_result(1,"ok");
}

extern "C" struct cmd_result *onestep_ed(struct cmd *,void *);
struct cmd_result *onestep_ed(struct cmd *command,void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  oneblock_loop(ws->loop);
  return build_result(1,"ok");
}

extern "C" struct cmd_result *rewind_ed(struct cmd *,void *);
struct cmd_result *rewind_ed(struct cmd *command,void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  rewind_loop(ws->loop);
  return build_result(1,"ok");
}

extern "C" struct cmd_result *inter_ed(struct cmd *,void *);
struct cmd_result *inter_ed(struct cmd *command,void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  ws->inter=1;
  return build_result(1,"ok");
}


void newblock(void *workspace,struct block *b) {
  struct or_ws *ws=(struct or_ws *)workspace;
  printf("new block\n");
  ws->mute=0;

  ws->canvas->Modified();
  ws->canvas->Update();

  //waiting for user
  if (ws->inter) {
    printf("Hit a key for next block\n");
    getchar();
  } else { //let the user see the event
    //usleep(500000);
  }
  
  //reseting the graph
  for(int i=0;i<MAX_POINTS;i++) {
    if (i%2) {
      ws->graph->SetPoint(i,MIN_X,MIN_Y,MIN_Z);
    } else {
      ws->graph->SetPoint(i,MAX_X,MAX_Y,MAX_Z);
    }
  }
  ws->nbp=0;
}

void endblock(void *workspace,struct block *b) {
}

void newevt(void *workspace,struct event *e) {
  struct or_ws *ws=(struct or_ws *)workspace;
  
  //fill the histo with the energy
  Double_t x,y,z;

  if (ws->mute)
    return;
  
  if (ws->nbp==MAX_POINTS) {
    printf("CAUTION : removing extra points\n");
    return;
  }

  x=(Double_t)atof(get_event_field(e,"x"));
  y=(Double_t)atof(get_event_field(e,"y"));
  z=(Double_t)atof(get_event_field(e,"z"));
  print_event(e);
  //printf("adding point %d : %g %g %g\n",ws->nbp,x,y,z);
  ws->graph->SetPoint(ws->nbp,x,y,z);
  ws->nbp++;
  //ws->canvas->Modified();
  //ws->canvas->Update();
}

void reinit(void *workspace) {
  printf("reinit\n");
}

void *run(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  
  //run the TApp
  ws->ecalApp->Run();
  
  //never reached
  pthread_exit(NULL);
}

extern "C" void*init(int,char **);
void *init(int argc, char **argv) {

  printf("initializing the root structure\n");
  
  //initialize
  struct or_ws *ws=(struct or_ws *)malloc(sizeof(struct or_ws));
  
  ws->ecalApp=new TApplication("App",&argc,argv);
  
  ws->canvas=new TCanvas("ecal","Calicoes Event Display",1000,1000);

  ws->graph=new TGraph2D(MAX_POINTS);
  ws->graph->SetNameTitle("Calicoes Event Display","Calicoes Event Display;x;y;z");
  ws->graph->SetNpy(40);
  ws->graph->SetNpx(40);
  ws->graph->SetMarkerStyle(20);
  for(int i=0;i<MAX_POINTS;i++)
    ws->graph->SetPoint(i,1.0*i,1.0*i,1.0*i);
  ws->graph->Draw("PCOL");

  ws->graph->GetXaxis()->SetTitleOffset(0.05);
  ws->graph->GetYaxis()->SetTitleOffset(0.05);
  ws->graph->GetZaxis()->SetTitleOffset(0.05);

  ws->graph->GetXaxis()->SetNdivisions(505);
  ws->graph->GetYaxis()->SetNdivisions(505);
  ws->graph->GetZaxis()->SetNdivisions(505);

  ws->canvas->Update();

  ws->t_run=(pthread_t*)malloc(sizeof(pthread_t));
  if (pthread_create(ws->t_run,NULL,run,ws)<0) {
    perror("pthread_create");
    exit(1);
  }
  pthread_detach(*ws->t_run);

  sleep(1);
  ws->inter=1;
  ws->loop=new_event_loop((char *)"evb",ws,newevt,newblock,endblock,reinit,'f');
  run_loop(ws->loop);

  //return the workspace to cmdmod
  return ws;
}

