//
// Copyright 2012-2014 Frédéric Magniette, Miguel Rubio-Roy, LLR
//

#include "hitcam.h"

void init_CLUT(struct clut * c) {
  int i;
  for(i=0;i<256;i++) {
    c->R[i]=128.0*sin(i/FREQ+3.14)+128.0;
    c->G[i]=128.0*sin(i/FREQ-1.57)+128.0;
    c->B[i]=128.0*sin(i/FREQ)+128.0;
    //printf("%d %f %f %f\n",i,c->R[i],c->G[i],c->B[i]);
  }
}

void leak_cam(struct hitcam_ws *ws) {
int i,j;
  for(i=0;i<ws->x;i++)
    for(j=0;j<ws->y;j++) {
      ws->leak_hit[i][j]=max(0,ws->leak_hit[i][j]-LEAK_HIT_VALUE);
      ws->leak_en[i][j]=max(0,ws->leak_en[i][j]-LEAK_EN_VALUE);
    }
}


void backcolor(SDL_Surface *screen,int R,int G,int B) {
  SDL_Rect rect;
  Uint32 color=SDL_MapRGB(screen->format,R,G,B);
  rect.x=0;
  rect.y=0;
  rect.w=screen->w;
  rect.h=screen->h;
  SDL_FillRect(screen,&rect,color); 
  SDL_Flip(screen);
}


void sdlgetevent() {
  SDL_Event event;
  if (SDL_PollEvent(&event)) {
    switch(event.type) {
    case SDL_QUIT:
      exit(1);
    }
  }
}

void calc_rect(int x,int y,int width,int spacing,SDL_Rect *rect) {
  rect->x=30+x*width+x*spacing;
  rect->y=30+y*width+y*spacing;
  rect->w=width;
  rect->h=width; //square
}

void print_rgb_pixel(struct hitcam_ws *ws,int x,int y,int R,int G,int B) {
  Uint32 color;
  SDL_Rect rect;
  SDL_Rect str_rect;
  SDL_Surface *sText;
  char str_value[1024];
  SDL_Color clrFg = {255,255,255,255};
  color=SDL_MapRGB(ws->screen->format,R,G,B);
  sprintf(str_value,"%d,%d",ws->mapping[x][y].chip,ws->mapping[x][y].chan);
  sText=TTF_RenderText_Solid(ws->font,str_value,clrFg);
  calc_rect(x,y,PIXEL_WIDTH,PIXEL_SPACING,&rect);
  str_rect.x=rect.x+2;
  str_rect.y=rect.y+7;
  str_rect.w=0;
  str_rect.h=0;
  SDL_FillRect(ws->screen,&rect,color);
  SDL_BlitSurface(sText,NULL,ws->screen,&str_rect);
  SDL_FreeSurface(sText);
}

void print_nbe(struct hitcam_ws *ws) {
  char str_value[1024];
  SDL_Surface *sText;
  SDL_Rect str_rect;
  SDL_Color clrFg = {0,0,0,0};
  Uint32 color;
  
  color=SDL_MapRGB(ws->screen->format,255,255,255);
  str_rect.x=5;
  str_rect.y=3;
  str_rect.w=130;
  str_rect.h=20;
  SDL_FillRect(ws->screen,&str_rect,color);

  sprintf(str_value,"nb_events=%d",ws->nb_points);
  sText=TTF_RenderText_Solid(ws->font,str_value,clrFg);
  str_rect.x=10;
  str_rect.y=6;
  str_rect.w=0;
  str_rect.h=0;
  SDL_BlitSurface(sText,NULL,ws->screen,&str_rect);
  SDL_FreeSurface(sText);
}


void print_pixel(struct hitcam_ws *ws,int x,int y,int value,struct clut *colors) {
  Uint32 color;
  SDL_Rect rect;
  SDL_Surface *sText;
  SDL_Rect str_rect;
  char str_value[1024];
  SDL_Color clrFg = {255,255,255,255};
  color=SDL_MapRGB(ws->screen->format,colors->R[value],colors->G[value],colors->B[value]);
  //sprintf(str_value,"%d",value);
  sprintf(str_value,"%d,%d",ws->mapping[x][y].chip,ws->mapping[x][y].chan);
  sText=TTF_RenderText_Solid(ws->font,str_value,clrFg);
  calc_rect(x,y,PIXEL_WIDTH,PIXEL_SPACING,&rect);
  str_rect.x=rect.x+2;
  str_rect.y=rect.y+7;
  str_rect.w=0;
  str_rect.h=0; 
  SDL_FillRect(ws->screen,&rect,color);
  SDL_BlitSurface(sText,NULL,ws->screen,&str_rect);
  SDL_FreeSurface(sText);
}

void print_map(struct hitcam_ws *ws) {
  int i,j;
  int color;
  for (i=0;i<ws->x;i++)
    for(j=0;j<ws->y;j++) 
      if (ws->first_hit[i][j]) {
        switch(ws->mode) {
        case LEAK_HIT:
          color=ws->leak_hit[i][j];
          break;
        case CUMUL_HIT:
          color=(int)((float)ws->cumul_hit[i][j]/(float)ws->max_cumul_hit*255.0);
          break;
        case LEAK_EN:
          color=ws->leak_en[i][j];
          break;
        case CUMUL_EN:
          color=(int)((float)ws->cumul_en[i][j]/(float)ws->max_cumul_en*255.0);
          break;
        default:
          break;
        }
        print_pixel(ws,i,j,color,&(ws->colors));
      }
  print_nbe(ws);
}

int **allocate_int_tab(int x,int y,int init_value) {
  int i,j;
  int **result=malloc(x*sizeof(int *));
  for (i=0;i<x;i++) {
    result[i]=malloc(y*sizeof(int));
    for (j=0;j<y;j++)
      result[i][j]=init_value;
  }
  return result;
}

void init_tab(int **tab,int x,int y,int init_value) {
  int i,j;
  for (i=0;i<x;i++) {
    for (j=0;j<y;j++)
      tab[i][j]=init_value;
  }
}

struct hitcam_ws * new_hitcam_ws(int x,int y,char *mode) {
  struct hitcam_ws *result;
  int i,j;
  FILE *f;
  int res;
  float rx,ry;
  int rc,rl,rchip,rchan;
  result=malloc(sizeof(struct hitcam_ws));
  memset(result,0,sizeof(struct hitcam_ws));
  result->mode=UNKNOWN;
  result->x=x;
  result->y=y;
  result->k=1;
  if (!strcmp(mode,"cumul_hit"))
    result->mode=CUMUL_HIT;
  if (!strcmp(mode,"leak_hit"))
    result->mode=LEAK_HIT;
  if (!strcmp(mode,"cumul_en"))
    result->mode=CUMUL_EN;
  if (!strcmp(mode,"leak_en"))
    result->mode=LEAK_EN;
  if (result->mode==UNKNOWN) {
    printf("unknown mode %s\n",mode);
    return NULL;
  }

  //allocate rendering structures
  result->cumul_hit=allocate_int_tab(x,y,0);
  result->cumul_en=allocate_int_tab(x,y,0);
  result->leak_hit=allocate_int_tab(x,y,0);
  result->leak_en=allocate_int_tab(x,y,0);
  result->first_hit=allocate_int_tab(x,y,0);
  
  result->screen=SDL_SetVideoMode(1100,1100,32,SDL_DOUBLEBUF|SDL_HWSURFACE);
  result->nb_points=0;
  result->max_cumul_hit=0;
  result->max_cumul_en=0;

  //allocate and fill mapping
  result->mapping=malloc(sizeof(struct cellid*)*x);
  for(i=0;i<y;i++)
    result->mapping[i]=malloc(sizeof(struct cellid)*y);
  f=fopen("/opt/calicoes/mapping_ecal_fev10.txt","r");
  while(!feof(f)) {
    res=fscanf(f,"%d %d %f %f %d %d",&rchip,&rchan,&rx,&ry,&rc,&rl);
    if ((res!=6)&&(res!=-1)) {
      printf("syntax error in file /opt/calicoes/mapping_ecal_fev10.txt obtained %d items instead of 6\n",res);
    } /*else {
      printf("chip=%d chan=%d x=%f y=%f c=%d l=%d\n",rchip,rchan,rx,ry,rc,rl); 
      }*/
    result->mapping[rc][rl].chip=rchip+1;
    result->mapping[rc][rl].chan=rchan;
  }
  fclose(f);


  //initialize color system
  init_CLUT(&(result->colors));
  backcolor(result->screen,255,255,255);

  //initialize font
  result->font=TTF_OpenFont("/root/calicoes/guis/hitcam/LiberationSans-Regular.ttf",10);
  if (result->font==NULL) {
    perror("TTF_OpenFont");
    exit(1);
  }

  //print first map
  for(i=0;i<result->x;i++)
    for(j=0;j<result->y;j++)
      print_rgb_pixel(result,i,j,0,0,0);
  print_nbe(result);

  result->fnb=0;
  result->mo=0;
  return result;
}



void fill(struct hitcam_ws * ws,struct event *e) {

  //feed the graphic system
  int x=atoi(get_event_field(e,"c"));
  int y=atoi(get_event_field(e,"l"));
  int en=atoi(get_event_field(e,"en"));
  switch(ws->mode) {
  case LEAK_HIT:
    ws->leak_hit[x][y]=min(ws->leak_hit[x][y]+60,255);
    break;
  case CUMUL_HIT:
    ws->cumul_hit[x][y]++;
    if (ws->cumul_hit[x][y]>ws->max_cumul_hit)
      ws->max_cumul_hit=ws->cumul_hit[x][y];
    break;
  case LEAK_EN:
    ws->leak_en[x][y]=min(ws->leak_en[x][y]/10,255);
    break;
  case CUMUL_EN:
    ws->cumul_en[x][y]+=en;
    if (en>ws->max_cumul_en)
      ws->max_cumul_en=en;
    break;
  default:
    break;
  }
  ws->nb_points++;
  //printf("cumul[%d][%d]=%d/%d\n",roc,chan,ws->cumul[roc][chan],ws->max_cumul);
  ws->first_hit[x][y]=1;
}


void save_current_pic(struct hitcam_ws * ws) {
  char name[2048];
  if (ws->mo) {
    sprintf(name,"imgs/frame%03d.bmp",ws->fnb);
    ws->fnb++;
    SDL_SaveBMP(ws->screen,name);
  }
}

void waiting_func(void *workspace) {
  usleep(100000);
  sdlgetevent();
}

void newblock(void *workspace,struct block *b) {
  //printf("new block\n");
}

void endblock(void *workspace,struct block *b) {
  //printf("new block\n");
}

void newevt(void *workspace,struct event *e) {
  struct hitcam_ws *ws=(struct hitcam_ws *)workspace;
  char *s_spill=get_event_field(e,"spill");
  char *s_bcid=get_event_field(e,"bcid");
  //char *s_chan=get_event_field(e,"chan");
  //char *s_chip=get_event_field(e,"roc");
  
  //printf("new event on chip %s chan %s\n",s_chip,s_chan);
  //printf("spill=%s bcid=%s\n",s_spill,s_bcid);
  if ((s_spill==NULL) || (s_bcid==NULL))
    return;
  int spill=atoi(s_spill);
  int bcid=atoi(s_bcid);
  //detect spill change
  if (spill!=ws->last_spill) {
    ws->last_spill=spill;
    leak_cam(ws);
  }
  //detect bcid change
  if (bcid!=ws->last_bcid) {
    ws->last_bcid=bcid;
    //leak_cam(ws);
  }
  fill(ws,e);
  print_map(ws);
  SDL_Flip(ws->screen);
  save_current_pic(ws);
  sdlgetevent();
  usleep(1000);
}

void reinit(void *workspace) {
  struct hitcam_ws *ws=(struct hitcam_ws *)workspace;
  int i,j;
  init_tab(ws->cumul_hit,ws->x,ws->y,0);
  init_tab(ws->cumul_en,ws->x,ws->y,0);
  init_tab(ws->leak_hit,ws->x,ws->y,0);
  init_tab(ws->leak_en,ws->x,ws->y,0);
  init_tab(ws->first_hit,ws->x,ws->y,0);
  for(i=0;i<ws->x;i++)
    for(j=0;j<ws->y;j++)
      print_rgb_pixel(ws,i,j,0,0,0);
  ws->nb_points=0;
  print_nbe(ws);
}



int main(int argc,char **argv) {
  struct hitcam_ws *ws;
  int maxx,maxy;

  //check arg number
  if (argc<5) {
    printf("usage for socket reading : hitcam ds_name maxx maxy mode(cumul_hit|leak_hit|cumul_en|leak_en)\n");
    exit(0);
  }
  
  //init graphics
  SDL_Init(SDL_INIT_VIDEO);
  SDL_WM_SetCaption("Calicoes Hitcam", NULL);
  TTF_Init();

  maxx=atoi(argv[2]);
  maxy=atoi(argv[3]);
  
  //init data structure
  ws=new_hitcam_ws(maxx,maxy,argv[4]);

  if (ws==NULL) {
    printf("unable to create workspace...exiting\n");
    exit(1);
  }

  ws->loop=new_event_loop(argv[1],ws,newevt,newblock,endblock,reinit,'f');
  run_loop(ws->loop);
  while(1) {
    usleep(100000);
    sdlgetevent();
  }

  //never reached
  TTF_Quit();
  SDL_Quit();
  return 1;
  
}//main
