//
// Copyright 2012-2014 Frédéric Magniette, Miguel Rubio-Roy, LLR
//

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <fcntl.h>
#include <stdio.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <SDL/SDL.h>
#include <time.h>
#include <SDL/SDL_ttf.h>
#include <pyrame.h>
#define MODE_FILE 1
#define MODE_SOCKET 2
#define MODE_STDIN 3

#ifndef max
#define max(a,b) (((a) > (b)) ? (a) : (b))
#endif
#ifndef min
#define min(a,b) (((a) < (b)) ? (a) : (b))
#endif

#define NB_CHIPS 4
#define NB_CHANS 64

struct dif_stream {
  SDL_Surface *screen;
  struct map_cell **mapping;
  int **asics;
  int **first_hit;
  struct clut colors;  
  //system
  int mode;
  //files reading
  char *filename;
  //socket reading
  int socket;
  char *host;
  int hport;
  //acquisition
  int input_id;
  FILE * input_f;
  //geometry
  int nb_chips;
  int nb_chans;
  //statistics
  unsigned char first_data;
  int k;
  //movie output
  int mo;
  int fnb;
} dif_stream;

struct map_cell {
  int x;
  int y;
  int size;
} mapcell;

#define NB_CHANS 64
#define NB_CHIPS 4

void print_asics(struct dif_stream *ds) {
  int i,j;
  printf("asics %d chips %d chans : \n",ds->nb_chips,ds->nb_chans);
  for(i=0;i<ds->nb_chips;i++) {
    for(j=0;j<ds->nb_chans;j++)
      printf("%d ",ds->asics[i][j]);
    printf("\n");
  }
}

void leak_cam(struct dif_stream *ds) {
int i,j;
  for(i=0;i<ds->nb_chips;i++)
    for(j=0;j<ds->nb_chans;j++) {
//the leak is of value 1
      ds->asics[i][j]=max(0,ds->asics[i][j]-1);
    }
}



struct map_cell **load_mapping(char *filename,int nb_chips,int nb_chans) {
  
  struct map_cell **mapping;
  FILE *f;
  int i;
  int chip,chan,x,y,size;

  //allocate mapping
  mapping=malloc(sizeof(struct map_cell *)*nb_chips);
  for (i=0;i<nb_chips;i++)
    mapping[i]=malloc(sizeof(struct map_cell)*nb_chans);

  //filling mapping
  f=fopen("mapping.txt","r");
  while(!feof(f)) {
    fscanf(f,"%d %d %d %d %d",&chip,&chan,&x,&y,&size);
    printf("chip=%d chan=%d x=%d y=%d size=%d\n",chip,chan,x,y,size); 
    mapping[chip][chan].x=x;
    mapping[chip][chan].y=y;
    mapping[chip][chan].size=size;
  }
  fclose(f);
  return mapping;
}

void backcolor(SDL_Surface *screen,int R,int G,int B) {
  SDL_Rect rect;
  Uint32 color=SDL_MapRGB(screen->format,R,G,B);
  rect.x=0;
  rect.y=0;
  rect.w=screen->w;
  rect.h=screen->h;
  SDL_FillRect(screen,&rect,color); 
  SDL_Flip(screen);
}


void sdlgetevent() {
  SDL_Event event;
  if (SDL_PollEvent(&event)) {
    switch(event.type) {
    case SDL_QUIT:
      exit(1);
    }
  }
}

void calc_rect(int x,int y,int width,int spacing,SDL_Rect *rect) {
  rect->x=30+x*width+x*spacing;
  rect->y=30+y*width+y*spacing;
  rect->w=width;
  rect->h=width; //square
}

void print_rgb_pixel(SDL_Surface *screen,int x,int y,int R,int G,int B) {
  Uint32 color;
  SDL_Rect rect;
  int spacing=2;
  int width=15;
  color=SDL_MapRGB(screen->format,R,G,B);
  calc_rect(x,y,width,spacing,&rect);
  SDL_FillRect(screen,&rect,color);
}


void print_pixel(SDL_Surface *screen,int x,int y,int size,int value,struct clut *colors) {
  Uint32 color;
  SDL_Rect rect;
  int spacing=2;
  int width=15;
  color=SDL_MapRGB(screen->format,colors->R[value],colors->G[value],colors->B[value]);
  switch(size) {
  case 1:
    calc_rect(x,y,width,spacing,&rect);
    SDL_FillRect(screen,&rect,color);
    break;
  case 2:
    calc_rect(x,y,width,spacing,&rect);
    SDL_FillRect(screen,&rect,color);
    calc_rect(x+1,y,width,spacing,&rect);
    SDL_FillRect(screen,&rect,color);
    break;  
   case 4:
     calc_rect(x,y,width,spacing,&rect);
     SDL_FillRect(screen,&rect,color);
     calc_rect(x+1,y,width,spacing,&rect);
     SDL_FillRect(screen,&rect,color);
     calc_rect(x,y+1,width,spacing,&rect);
     SDL_FillRect(screen,&rect,color);
     calc_rect(x+1,y+1,width,spacing,&rect);
     SDL_FillRect(screen,&rect,color);
     break;
  }  
}

void print_map(struct dif_stream *ds) {
  int i,j;
  for (i=0;i<ds->nb_chips;i++)
    for(j=0;j<ds->nb_chans;j++)
      if (ds->first_hit[i][j])
	print_pixel(ds->screen,ds->mapping[i][j].x,ds->mapping[i][j].y,ds->mapping[i][j].size,ds->asics[i][j],&(ds->colors));
}


struct dif_stream * new_dif_stream(int nb_chips,int nb_chans) {
  struct dif_stream *result;
  int i,j;
  result=malloc(sizeof(struct dif_stream));
  memset(result,0,sizeof(struct dif_stream));
  result->nb_chips=nb_chips;
  result->nb_chans=nb_chans;
  result->k=1;
  result->screen=SDL_SetVideoMode(350,350,32,SDL_DOUBLEBUF|SDL_HWSURFACE);
  result->mapping=load_mapping("mapping.txt",nb_chips,nb_chans);
  //allocate and fill asics
  result->asics=malloc(nb_chips*sizeof(int *));
  for (i=0;i<nb_chips;i++) {
    result->asics[i]=malloc(nb_chans*sizeof(int));
    for (j=0;j<nb_chans;j++)
      result->asics[i][j]=0;
  }
  result->first_hit=malloc(nb_chips*sizeof(int *));
  for (i=0;i<nb_chips;i++) {
    result->first_hit[i]=malloc(nb_chans*sizeof(int));
    for (j=0;j<nb_chans;j++)
      result->first_hit[i][j]=0;
  }
  init_CLUT(&(result->colors));
  backcolor(result->screen,255,255,255);
  for(i=0;i<nb_chips;i++)
    for(j=0;j<nb_chans;j++)
      print_rgb_pixel(result->screen,result->mapping[i][j].x,result->mapping[i][j].y,0,0,0);
  result->fnb=0;
  result->mo=0;
  return result;
}
 
 void open_data(struct dif_stream *ds) {
   printf("opening reading resource\n");
   if (ds->mode==MODE_FILE) {
     ds->input_id=file_open(ds->filename);
   }
   if (ds->mode==MODE_SOCKET) {
     ds->input_id=open_socket(ds->host,ds->hport);
   }
   if (ds->mode==MODE_STDIN) {
     ds->input_id=0;
   }
   ds->input_f=fdopen(ds->input_id,"r");
   printf("input_f=%p\n",ds->input_f);
   if (ds->input_f==NULL) {
     perror("fdopen");
     exit(1);
   }
 }

void read_n_fill(struct dif_stream *ds) {
  int scanres;
  int chan;

  //data structure
  int chipid, spill, sca, bcid;
  int *values_g1;
  int *values_g2;
  char *hits_g1;
  char *hits_g2;

  //filter variables
  int totalhits;

  //allocate data structure
  values_g1=(int *)malloc(ds->nb_chans*sizeof(int));
  values_g2=(int *)malloc(ds->nb_chans*sizeof(int));
  hits_g1=(char *)malloc(ds->nb_chans*sizeof(char));
  hits_g2=(char *)malloc(ds->nb_chans*sizeof(char));


  //read the data
  scanres=fscanf(ds->input_f,"%d %d %d %d ",&chipid,&spill,&sca,&bcid);
  if (scanres!=4) {
    printf("cant read headers\n");
  }
  //printf("chipid=%d spill=%d sca=%d bcid=%d\n",chipid,spill,sca,bcid);
  for (chan=0;chan<ds->nb_chans;chan++) {
    //high gain
    scanres=fscanf(ds->input_f,"%c ",&hits_g1[chan]);
    if (scanres==0) {
      printf("cant read anymore (1/%d)\n",chan);
      exit(1);
    }
    scanres=fscanf(ds->input_f,"%d ",&values_g1[chan]);
    if (scanres==0) {
      printf("cant read anymore (2/%d)\n",chan);
      exit(1); 
    }
    //low gain
    scanres=fscanf(ds->input_f,"%c ",&hits_g2[chan]);
    if (scanres==0) {
      printf("cant read anymore (3/%d)\n",chan);
      exit(1);
    }
    scanres=fscanf(ds->input_f,"%d ",&values_g2[chan]);
    if (scanres==0) {
      printf("cant read anymore (4/%d)\n",chan);
      exit(1);
    }
  }

  //filtering : if you find a bad condition just return
  totalhits=0;
  for (chan=0;chan<ds->nb_chans;chan++) {
    if (hits_g1[chan]=='T')
      totalhits++;
  }
  //if (totalhits>10) {
    //printf("plain event\n");
    //return;
  //}

  //feed the graphic system
  for (chan=0;chan<ds->nb_chans;chan++) {
    if (hits_g1[chan]=='T') {
      //gain fixe
      //ds->asics[chipid][chan]=min(ds->asics[chipid][chan]+60,255); 
      //gain pondere par le nb de hits
      ds->asics[chipid][chan]=min(ds->asics[chipid][chan]+(int)(150/totalhits),255); 
      ds->first_hit[chipid][chan]=1;
    }
  }

}


void print_usage() {
  printf("usage for std input reading : hitcam stdin\n");
  printf("usage for file reading : hitcam file filename\n");
  printf("usage for socket reading : hitcam socket host client_port\n");
  exit(0);
}

void save_current_pic(struct dif_stream * ds) {
  char name[2048];
  if (ds->mo) {
    sprintf(name,"imgs/frame%03d.bmp",ds->fnb);
    ds->fnb++;
    SDL_SaveBMP(ds->screen,name);
  }
}


int main(int argc,char **argv) {
  struct dif_stream *mydif;
  int configured=0;

  if (argc<2) {
    print_usage();
  }

  SDL_Init(SDL_INIT_VIDEO);
  SDL_WM_SetCaption("Calice Hitcam", NULL);
  TTF_Init();

  mydif=new_dif_stream(NB_CHIPS,NB_CHANS);

  if (!strcmp(argv[1],"socket")) {
    if (argc<4) {
      print_usage();
    }
    printf("socket mode\n");
    mydif->mode=MODE_SOCKET;
    mydif->host=malloc(sizeof(char)*(strlen(argv[2])+2));
    strcpy(mydif->host,argv[2]);
    mydif->hport=atoi(argv[3]);
    configured=1;
  }
  if (!strcmp(argv[1],"file")) {
    if (argc<3) {
      print_usage();
      exit(1);
    }
    printf("file mode\n");
    mydif->mode=MODE_FILE;
    mydif->filename=malloc(sizeof(char)*(strlen(argv[2])+2));
    strcpy(mydif->filename,argv[2]);
    configured=1;
  }
  if (!strcmp(argv[1],"stdin")) {
    printf("stdin mode\n");
    mydif->mode=MODE_STDIN;
    configured=1;
  }
  
  if (configured==0) {
    printf("unknown mode %s\n",argv[1]);
    print_usage();
  }

  open_data(mydif);
  while (1) {
    leak_cam(mydif);
    read_n_fill(mydif);
    print_map(mydif);
    SDL_Flip(mydif->screen);
    save_current_pic(mydif);
    if ((mydif->mode==MODE_FILE) || (mydif->mode==MODE_STDIN))
      usleep(10000);
    sdlgetevent();
  }

  //never reached
  TTF_Quit();
  SDL_Quit();
  return 1;
  
}//main
