//
// Copyright 2012-2014 Frédéric Magniette, Miguel Rubio-Roy, LLR
//

#include <unistd.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <fcntl.h>
#include <stdio.h>
#include <netdb.h>
#include <netinet/in.h>
#include <sys/socket.h>
#include <SDL/SDL.h>
#include <time.h>
#include <SDL/SDL_ttf.h>


#ifndef max
#define max(a,b) (((a) > (b)) ? (a) : (b))
#endif
#ifndef min
#define min(a,b) (((a) < (b)) ? (a) : (b))
#endif

#define NB_CHIPS 4
#define NB_CHANS 64

//Colors Look-up Table
struct clut {
  float R[256];
  float G[256];
  float B[256];
} clut;
#define FREQ 45.0

struct dif_stream {
  int difid;
  SDL_Surface *screen;
  struct map_cell **mapping;
  int **asics;
  int **first_hit;
  struct clut colors;  
  //geometry
  int nb_chips;
  int nb_chans;
  int maxval;
  int minval;
} dif_stream;

struct map_cell {
  int x;
  int y;
  int size;
} mapcell;

#define NB_CHANS 64
#define NB_CHIPS 4

void init_CLUT(struct clut * c) {
  int i;
  for(i=0;i<256;i++) {
    c->R[i]=128.0*sin(i/FREQ+3.14)+128.0;
    c->G[i]=128.0*sin(i/FREQ-1.57)+128.0;
    c->B[i]=128.0*sin(i/FREQ)+128.0;
    //printf("%d %f %f %f\n",i,c->R[i],c->G[i],c->B[i]);
  }
}

void print_asics(struct dif_stream *ds) {
  int i,j;
  printf("asics %d chips %d chans : \n",ds->nb_chips,ds->nb_chans);
  for(i=0;i<ds->nb_chips;i++) {
    for(j=0;j<ds->nb_chans;j++)
      printf("%d ",ds->asics[i][j]);
    printf("\n");
  }
}


struct map_cell **load_mapping(char *filename,int nb_chips,int nb_chans) {
  
  struct map_cell **mapping;
  FILE *f;
  int i;
  int chip,chan,x,y,size;

  //allocate mapping
  mapping=malloc(sizeof(struct map_cell *)*nb_chips);
  for (i=0;i<nb_chips;i++)
    mapping[i]=malloc(sizeof(struct map_cell)*nb_chans);

  //filling mapping
  f=fopen("mapping.txt","r");
  while(!feof(f)) {
    fscanf(f,"%d %d %d %d %d",&chip,&chan,&x,&y,&size);
    //printf("chip=%d chan=%d x=%d y=%d size=%d\n",chip,chan,x,y,size); 
    mapping[chip][chan].x=x;
    mapping[chip][chan].y=y;
    mapping[chip][chan].size=size;
  }
  fclose(f);
  return mapping;
}

void backcolor(SDL_Surface *screen,int R,int G,int B) {
  SDL_Rect rect;
  Uint32 color=SDL_MapRGB(screen->format,R,G,B);
  rect.x=0;
  rect.y=0;
  rect.w=screen->w;
  rect.h=screen->h;
  SDL_FillRect(screen,&rect,color); 
  SDL_Flip(screen);
}


void sdlgetevent() {
  SDL_Event event;
  if (SDL_PollEvent(&event)) {
    switch(event.type) {
    case SDL_QUIT:
      exit(1);
    }
  }
}

void calc_rect(int x,int y,int width,int spacing,SDL_Rect *rect) {
  rect->x=30+x*width+x*spacing;
  rect->y=30+y*width+y*spacing;
  rect->w=width;
  rect->h=width; //square
}

void print_rgb_pixel(SDL_Surface *screen,int x,int y,int R,int G,int B) {
  Uint32 color;
  SDL_Rect rect;
  int spacing=2;
  int width=15;
  color=SDL_MapRGB(screen->format,R,G,B);
  calc_rect(x,y,width,spacing,&rect);
  SDL_FillRect(screen,&rect,color);
}


void print_pixel(SDL_Surface *screen,int x,int y,int size,int value,struct clut *colors) {
  Uint32 color;
  SDL_Rect rect;
  int spacing=2;
  int width=15;
  color=SDL_MapRGB(screen->format,colors->R[value],colors->G[value],colors->B[value]);
  switch(size) {
  case 1:
    calc_rect(x,y,width,spacing,&rect);
    SDL_FillRect(screen,&rect,color);
    break;
  case 2:
    calc_rect(x,y,width,spacing,&rect);
    SDL_FillRect(screen,&rect,color);
    calc_rect(x+1,y,width,spacing,&rect);
    SDL_FillRect(screen,&rect,color);
    break;  
   case 4:
     calc_rect(x,y,width,spacing,&rect);
     SDL_FillRect(screen,&rect,color);
     calc_rect(x+1,y,width,spacing,&rect);
     SDL_FillRect(screen,&rect,color);
     calc_rect(x,y+1,width,spacing,&rect);
     SDL_FillRect(screen,&rect,color);
     calc_rect(x+1,y+1,width,spacing,&rect);
     SDL_FillRect(screen,&rect,color);
     break;
  }  
}

void print_map(struct dif_stream *ds) {
  int i,j;
  for (i=0;i<ds->nb_chips;i++)
    for(j=0;j<ds->nb_chans;j++)
      if (ds->first_hit[i][j])
	print_pixel(ds->screen,ds->mapping[i][j].x,ds->mapping[i][j].y,ds->mapping[i][j].size,ds->asics[i][j],&(ds->colors));
}


struct dif_stream * new_dif_stream(int difid,int nb_chips,int nb_chans,int minval,int maxval) {
  struct dif_stream *result;
  int i,j;
  result=malloc(sizeof(struct dif_stream));
  memset(result,0,sizeof(struct dif_stream));
  result->difid=difid;
  result->nb_chips=nb_chips;
  result->nb_chans=nb_chans;
  result->minval=minval;
  result->maxval=maxval;
  result->screen=SDL_SetVideoMode(350,350,32,SDL_DOUBLEBUF|SDL_HWSURFACE);
  result->mapping=load_mapping("mapping.txt",nb_chips,nb_chans);
  //allocate and fill asics
  result->asics=malloc(nb_chips*sizeof(int *));
  for (i=0;i<nb_chips;i++) {
    result->asics[i]=malloc(nb_chans*sizeof(int));
    for (j=0;j<nb_chans;j++)
      result->asics[i][j]=0;
  }
  result->first_hit=malloc(nb_chips*sizeof(int *));
  for (i=0;i<nb_chips;i++) {
    result->first_hit[i]=malloc(nb_chans*sizeof(int));
    for (j=0;j<nb_chans;j++)
      result->first_hit[i][j]=0;
  }
  init_CLUT(&(result->colors));
  backcolor(result->screen,255,255,255);
  for(i=0;i<nb_chips;i++)
    for(j=0;j<nb_chans;j++)
      print_rgb_pixel(result->screen,result->mapping[i][j].x,result->mapping[i][j].y,0,0,0);
  return result;
}

void fill_from_file(struct dif_stream * ds,char *filename) {

  FILE *f;
  int pixel_value;
  int dif,chip,chan,val;
  int res;

  f=fopen(filename,"r");
  if (f==NULL) {
    perror("fopen");
    exit(1);
  }

  while(1) {
    res=fscanf(f,"%d %d %d %d",&dif,&chip,&chan,&val);
    if (res!=4) {
      perror("fscanf");
      break;
    }
    //printf("read for dif %d\n",dif);
    if (ds->difid==dif) {
      pixel_value=max(0,min(255,(val-ds->minval)*255/(float)(ds->maxval-ds->minval)));
      printf("filling (%d,%d) with %d\n",chip,chan,pixel_value); 
      ds->asics[chip][chan]=pixel_value;
      ds->first_hit[chip][chan]=1;
    }
  }
}




void save_current_pic(struct dif_stream * ds) {
  char name[2048];
    sprintf(name,"values_dif%d.bmp",ds->difid);
    SDL_SaveBMP(ds->screen,name);
}


int main(int argc,char **argv) {
  struct dif_stream *ds;

  //check arg number
  if (argc<4) {
    printf("usage : %s difid filename minval maxval\n",argv[0]);
    exit(1);
  }
  
  //init graphics
  SDL_Init(SDL_INIT_VIDEO);
  SDL_WM_SetCaption("Calicoes Hitcam", NULL);
  TTF_Init();

  //init data structure
  ds=new_dif_stream(atoi(argv[1]),NB_CHIPS,NB_CHANS,atoi(argv[3]),atoi(argv[4]));
  
  fill_from_file(ds,argv[2]);
  print_map(ds);
  SDL_Flip(ds->screen);
  save_current_pic(ds);
  while(1) {
    sdlgetevent();
  }


  //never reached
  TTF_Quit();
  SDL_Quit();
  return 1;
  
}//main
