/* ***********************************************************************
 * Online monitor program for wagasci 
 * Program : om_wagasci.C
 * Name: Naruhiro Chikuma
 * Date: 2017-05-08 23:33:49
 * ********************************************************************** */

// #define __HttpServer__
// For Http server, ROOT needs to be compiled with HTTP mode.

//#define DEBUG


#include <stdio.h>
#include <stdlib.h>
#include <time.h>
#include <fstream>
#include <TStyle.h>
#include <TH1F.h>
#include <TH2F.h>
#include <TF1.h>
#include <TGraph.h>
#include <TGraph2D.h>
#include <TThread.h>
#include <TCanvas.h>
#include <TString.h>
#include <TApplication.h>
#include <TFrame.h>
#include <TGaxis.h>
#include <TText.h>
#include <TLine.h>
#include <TBox.h>
#include <TLegend.h>
#include <pyrame.h>

using namespace std;

#ifdef __HttpServer__
#include <THttpServer.h>
#endif

#define NumHist 10

#define NumSca 16
#define NumChip 20
#define NumChipCh 32
#define NumPln 8
#define NumCh 80
#define NumChAll NumPln*NumCh

#define OffsetTimeEven 4000.
#define OffsetTimeOdd 500.
#define RampTimeEven -1.
#define RampTimeOdd 1.
#define NormBCID 580 //ns
#define ExpectedBCIDstart 24
#define ExpectedHitTiming 100 //ns from beginning of bcid
#define ExpectedTimingWidth 100 //ns

#define NbUpdateBlock 100
#define NbSubUpdate 10
#define SpillNbUpdate 100
#define EvtDispUpdate -1
#define NbHit_Threshold 0

#define STATUS_SPILL_GOOD 10
#define STATUS_SPILL_NOTBAD 100

#define SideView 0
#define TopVeiw 1

#define WATARTANK_X 466  //mm
#define WATARTANK_Y 1256 //mm
#define SCINTI_W 25 //mm width
#define SCINTI_T 3  //mm thickness

struct map_cell 
{
  Int_t pln;
  Int_t ch;
  Float_t x;
  Float_t y;
  Float_t z;
} mapcell;

// ==================================================================

struct or_ws 
{
  // --- block&event parameters ---
  bool first_time;
  Int_t nevt;
  Int_t nblock;

  struct map_cell **mapping;
  struct evloop *loop;

  Int_t ini_spill;
  Int_t last_spill;
  Int_t lastblock_evtdisp;

  Int_t mode;
  Int_t dif_id;

  // --- main histograms ---
  TCanvas *canvas;
  TPad *pad[NumHist];
  TH1F *h_charge;
  TH1F *h_spillnb;
  TH1F *h_nrate;
  TH1F *h_gain;
  TH1F *h_bcid;
  TH1F *h_timing;
  TH2F *h_nrate_st;
  TH2F *h_gain_st;
  TH2F *h_status;
  TGraph   *g_evtdisp2D;

  // --- OM status ---
  TText *txt_time;
  TText *txt_spill;
  Int_t nb_out_nr[NumChip];
  Int_t nb_out_gain[NumChAll];
  Int_t nb_bcid;
  Int_t nb_out_bcid;
  Int_t nb_out_timing;
  Int_t mean_charge;
  Int_t nb_spillgap;
  Int_t nb_nonhit_ch;
  // --- for gain fitting ---
  TH1F *h_gain_ped[NumChAll];
  TH1F *h_gain_hit[NumChAll];
  // --- for noise rate calc. ---
  Int_t nrate_chip[NumChip];
  Int_t nrate[NumChAll];
  // --- for event disp. ---
  Int_t nb_hits[NumSca];
  Int_t hit_pln[NumSca][NumPln];
  Int_t hit_ch[NumSca][NumCh];

  TThread *updater_t;
  pthread_mutex_t * updater_mutex;

#ifdef __HttpServer__
  THttpServer *serv;
#endif
};

//================================updater==============================
void *updater(void *workspace) {
  struct or_ws *ws=(struct or_ws *)workspace;
  while(1) {
    pthread_mutex_lock(ws->updater_mutex);
    for (int i=0;i<1;i++) {
      ws->canvas->cd(i)->Modified();
    }
    ws->canvas->Update();
    pthread_mutex_unlock(ws->updater_mutex);
    usleep(100);
  }
  return NULL;
}

// ==================================================================

struct map_cell **load_mapping(Int_t dif_id   = 0, 
    char *filename = "wagasci_mapping_table.txt",
    Int_t nb_chips = NumChip,
    Int_t nb_chans = NumChipCh
    ) 
{

  struct map_cell **mapping;
  FILE *f;
  Int_t i;
  Int_t dif,chip,chip_ch;
  Int_t pln,ch;
  Float_t x,y,z;
  int res;

  mapping=(struct map_cell**)malloc(sizeof(struct map_cell *)*nb_chips);
  for (i=0;i<nb_chips;i++)
    mapping[i]=(struct map_cell*)malloc(sizeof(struct map_cell)*nb_chans);

  //open mapping file
  f=fopen(filename,"r");
  if (f==NULL) {
    printf("cant open file %s\n",filename);
    return NULL;
  }

  //reading mapping
  while(!feof(f)) {
    res=fscanf(f,"%d %d %d %f %f %f %d %d",&dif,&chip,&chip_ch,&x,&y,&z,&pln,&ch);
    if ((res!=8)&&(res!=-1)) {
      printf("syntax error in file %s obtained %d items instead of 6\n",filename,res);
    }
    if(dif==dif_id-1){
      mapping[chip][chip_ch].x   = x;
      mapping[chip][chip_ch].y   = y;
      mapping[chip][chip_ch].z   = z;
      mapping[chip][chip_ch].pln = pln;
      mapping[chip][chip_ch].ch  = ch;
    }
  } 
  fclose(f);
  return mapping;
} //load_mapping

// ==================================================================

bool isExpectedBcid(Int_t bcid)
{
  if(bcid>=ExpectedBCIDstart&&bcid<ExpectedBCIDstart+8) return true;
  else return false;
} //isExpectedBcid

// ==================================================================

bool isExpectedTiming(Int_t bcid, Float_t hittiming)
{
  Float_t offset_time = bcid%2==0 ? OffsetTimeEven : OffsetTimeOdd;
  Float_t ramp_time   = bcid%2==0 ? RampTimeEven   : RampTimeOdd; 
  Float_t exp_timing  = bcid*NormBCID + (ExpectedHitTiming-offset_time)*ramp_time;
  if(fabs(hittiming - exp_timing)*2<ExpectedTimingWidth ) return true;
  else return false;
} //isExpectedTiming

// ==================================================================

void newblock(void *workspace,struct block *b) 
{
  Int_t spillnb   = (Int_t)atoi(get_block_field(b,"spill"));
  struct or_ws *ws=(struct or_ws *)workspace;

  printf("new block\n");
  pthread_mutex_lock(ws->updater_mutex);

  if (ws->first_time) {
    printf("waiting for app to start\n");
    sleep(1);
    ws->nblock = 0;
    ws->ini_spill = spillnb;
    ws->last_spill = spillnb-1;
    ws->lastblock_evtdisp = 0;

    ws->first_time=false;
  }
  ws->nevt = 0;

  // ------- hist info for event disp ----------------  
  for(int isca=0;isca<NumSca;isca++){
    ws->nb_hits[isca] = 0; 
    memset(ws->hit_pln[isca],0,NumPln);
    memset(ws->hit_ch [isca],0,NumCh );
  }

  if(ws->nblock==0){

    // ------- for noise rate calc. ----------------  
    memset(ws->nrate_chip,0,NumChip);
    memset(ws->nrate,0,NumChAll);

    // ------- new histograms for fit ----------------  
    for(int ich=0;ich<NumChAll;ich++){
      string tmp;
      tmp=Form("gain_ped_%05d",ich);
      ws->h_gain_ped[ich] = new TH1F(tmp.c_str(),tmp.c_str(),400,400.,800.);
      tmp=Form("gain_hit_%05d",ich);
      ws->h_gain_hit[ich] = new TH1F(tmp.c_str(),tmp.c_str(),400,400.,800.);
    }
  }

  // ------ for status pad -------
  struct tm tm;
  char *dayofweek[] = {"Sun", "Mon", "Tue", "Wed", "Thu", "Fri", "Sat"};
  time_t t = time(NULL);
  localtime_r(&t, &tm);
  string SpillText = Form("SPILL#: %05d", spillnb);
  string TimeText = Form("Updated: %04d/%02d/%02d %s %02d:%02d:%02d\n",
      tm.tm_year + 1900, tm.tm_mon + 1, tm.tm_mday,
      dayofweek[tm.tm_wday], tm.tm_hour, tm.tm_min, tm.tm_sec);
  ws->txt_spill->SetText(0.82,0.95,SpillText.c_str());
  ws->txt_time ->SetText(0.82,0.91,TimeText.c_str());

  printf("new spill : %d\n",spillnb);
  pthread_mutex_unlock(ws->updater_mutex);

} //newblock

// ==================================================================

void endblock(void *workspace,struct block *b) 
{
  struct or_ws *ws=(struct or_ws *)workspace;
  Int_t spillnb = (Int_t)atoi(get_block_field(b,"spill"));
  Int_t spillflag = (Int_t)atoi(get_block_field(b,"spill_flag"));
  Int_t time_endblock = (Int_t)time(NULL);

  pthread_mutex_lock(ws->updater_mutex);


  // ------- Filling spill number ----------------  
  if ( ((ws->mode==0)&&(spillflag==1))||
      ((ws->mode==1)&&(ws->nblock%NbSubUpdate==0)) )
  {
    ws->h_spillnb->Fill((spillnb-ws->ini_spill)%SpillNbUpdate);
  }


  // ------- Drawing event display ----------------  
  int sca_with_max_nb_hits = -1;
  int max_nbhits=NbHit_Threshold;
  for(int isca=0;isca<NumSca;isca++){
    if(ws->nb_hits[isca]>max_nbhits){
      sca_with_max_nb_hits = isca;
    }
  }
  if(sca_with_max_nb_hits>-1){
    for(int chip=0;chip<NumChip;chip++){
      for(int chip_ch=0;chip_ch<NumChipCh;chip_ch++){
        Int_t pln = ws->mapping[chip][chip_ch].pln;
        Int_t ch  = ws->mapping[chip][chip_ch].ch;
        Int_t ip = pln*NumCh + ch;
        if((ws->hit_pln[sca_with_max_nb_hits][pln]==1) &&
            (ws->hit_ch[sca_with_max_nb_hits][ch]==1))
        {
          Float_t x   = ws->mapping[chip][chip_ch].x;
          Float_t y   = ws->mapping[chip][chip_ch].y;
          Float_t z   = ws->mapping[chip][chip_ch].z;

          Float_t XX, YY;
          XX = z;
          if     (ws->dif_id==1){ YY = y; }
          else if(ws->dif_id==2){ YY = x; }
          ws->g_evtdisp2D->SetPoint(ip,XX,YY);
        }
        else{
          ws->g_evtdisp2D->SetPoint(ip,-WATARTANK_X,-WATARTANK_Y);
        }
      }
    }
    ws->lastblock_evtdisp = ws->nblock;
  }


  if(ws->nblock<NbUpdateBlock-1){ ws->nblock++; }
  else{
    ws->nblock=0; 

    // ---------- noise rate per chip calculation -------------
    for(int ichip=0;ichip<NumChip;ichip++){
      Float_t noise_rate_chip; //kHz
      Float_t noise_rate_ch;   //kHz
      if(ws->nrate_chip[ichip]!=0){
        noise_rate_chip = 1e+6/NormBCID/(1.+NumSca*NbUpdateBlock/ws->nrate_chip[ichip]);
      }
      else{ noise_rate_chip = 1e-1; }
      ws->h_nrate->SetBinContent(ichip+1,noise_rate_chip);
      for(int ichipch=0;ichipch<NumChipCh;ichipch++){
        if(ws->nrate[ichip*NumChipCh+ichipch]==0){ noise_rate_ch = 1e-1; }
        else{ 
          noise_rate_ch = 1e+6/NormBCID/(1.+NumSca*NbUpdateBlock/ws->nrate[ichip*NumChipCh+ichipch]); 
        }
        ws->h_nrate_st->Fill(time_endblock,noise_rate_ch);
      }
    }

    // ---------- noise rate & gain calculation -------------
    for(int ich=0;ich<NumChAll;ich++){
      Int_t pedestal;
      Int_t hit_pos ;
      Float_t dist;

      if(ws->h_gain_hit[ich]->GetEntries()==0||ws->h_gain_ped[ich]->GetEntries()==0){ 
        dist = 1e-1;
      }
      else{

        TF1* fit_ped = new TF1("fit_ped","gaus",400.,800.);
        TF1* fit_hit = new TF1("fit_hit","gaus",400.,800.);
        Float_t sigma = 10.;
        Float_t lim_sigma = 2.;
        Float_t lim_pos   = 10.;
        pedestal=500; //TODO extract from pedestal file
        hit_pos =525; //TODO extract from dont know where
        fit_ped->SetParameters(1.,pedestal,sigma);
        fit_hit->SetParameters(1.,hit_pos ,sigma);
        fit_ped->FixParameter(0,1.);
        fit_hit->FixParameter(0,1.);
        fit_ped->SetParLimits(1,pedestal-lim_pos,pedestal+lim_pos);
        fit_hit->SetParLimits(1,hit_pos -lim_pos,hit_pos +lim_pos);
        fit_ped->SetParLimits(2,sigma-lim_sigma,sigma+lim_sigma);
        fit_hit->SetParLimits(2,sigma-lim_sigma,sigma+lim_sigma);
        ws->h_gain_ped[ich]->Scale(1./ws->h_gain_ped[ich]->GetMaximum());
        ws->h_gain_hit[ich]->Scale(1./ws->h_gain_hit[ich]->GetMaximum());
        //ws->h_gain_ped[ich]->Fit("fit_ped","Q","0",400.,800.);
        //ws->h_gain_hit[ich]->Fit("fit_hit","Q","0",400.,800.);

        dist = fit_hit->GetParameter(1)-fit_ped->GetParameter(1);

        delete fit_ped;
        delete fit_hit;
      }

      ws->h_gain->SetBinContent(ich+1,dist);
      ws->h_gain_st ->Fill(time_endblock,dist);

      delete ws->h_gain_ped[ich];
      delete ws->h_gain_hit[ich];
    }
  }

  // ------- for spill gap ----------------  
  if( (spillnb-ws->last_spill>1) ||
      ((ws->last_spill==65535)&&(spillnb!=0))) 
  {
    ws->nb_spillgap++;
  }
  ws->last_spill = spillnb;
  Float_t status_spill;
  if     (ws->nb_spillgap<STATUS_SPILL_GOOD  ) status_spill = 1.;
  else if(ws->nb_spillgap<STATUS_SPILL_NOTBAD) status_spill = 2.;
  else                                         status_spill = 3.;

  
  // ------- Fill the OM status ----------------  
  ws->h_status->SetBinContent(1,2,2.); //gain
  ws->h_status->SetBinContent(1,1,1.); //noise rate
  ws->h_status->SetBinContent(2,2,1.); //hit timing 
  ws->h_status->SetBinContent(2,1,3.); //bcid 
  ws->h_status->SetBinContent(3,2,1.); //ADC value
  ws->h_status->SetBinContent(3,1,status_spill); //spill gap
  ws->h_status->SetBinContent(4,2,1.); //hit channel
  ws->h_status->SetBinContent(4,1,0.); //nothing


#ifdef DEBUG
  printf("time:%d \n",(Int_t)time(NULL));
  printf("  number of events: %d \n",ws->nevt);
#endif

  pthread_mutex_unlock(ws->updater_mutex); 


} // endblock

// ==================================================================

void reinit(void *workspace) 
{
} //reinit

// ==================================================================

void newevt(void *workspace,struct event *e) 
{
  struct or_ws *ws=(struct or_ws *)workspace;
  //Int_t spillnb  =(Int_t)atoi(get_event_field(e,"spill"     )); 
  Int_t spillflag=(Int_t)atoi(get_event_field(e,"spill_flag")); 
  Int_t chip     =(Int_t)atoi(get_event_field(e,"roc"       )); 
  Int_t chip_ch  =(Int_t)atoi(get_event_field(e,"rocchan"   )); 
  Int_t bcid     =(Int_t)atoi(get_event_field(e,"bcid"      )); 
  Int_t sca      =(Int_t)atoi(get_event_field(e,"sca"       ));
  Int_t pln      =(Int_t)atoi(get_event_field(e,"plane"     )); 
  Int_t ch       =(Int_t)atoi(get_event_field(e,"channel"   )); 
  Int_t energy   =(Int_t)atoi(get_event_field(e,"en"        )); 
  Int_t data_time=(Int_t)atoi(get_event_field(e,"time"      )); 
  Int_t hit      =(Int_t)atoi(get_event_field(e,"hit"       )); 
  Int_t gain     =(Int_t)atoi(get_event_field(e,"gain"      ));
  //Float_t x =(Float_t)atoi(get_event_field(e,"x"));
  //Float_t y =(Float_t)atoi(get_event_field(e,"y"));
  //Float_t z =(Float_t)atoi(get_event_field(e,"z"));

  pthread_mutex_lock(ws->updater_mutex);


#ifdef DEBUG
  printf("New event %d == spillnb:%d, spillflag:%d, chip:%d, chip_ch:%d, bcid:%d, sca:%d, pln:%d, ch:%d, energy:%d, data_time:%d, hit:%d, gain:%d\n",
      ws->nevt,spillnb,spillflag,chip,chip_ch,bcid,sca,pln,ch,energy,data_time,hit,gain);
#endif

  // ------- Fill energy histogram ----------------
  if(hit==1){
    if ( ((ws->mode==0)&&(spillflag==1)) || (ws->mode==1) )
    {
      ws->h_charge->Fill(energy);
    }
  }

  // ------- Counting number of hits for noise rate calcu. ----------------
  if(spillflag==0){ //internal trigger
    if(hit==0){   ws->h_gain_ped[chip*NumChipCh+chip_ch]->Fill(energy);}
    else{
      if(bcid<NumSca){
        ws->nrate_chip[chip]++;
        ws->nrate[chip*NumChipCh+chip_ch]++;
      }
      if(gain==1){ws->h_gain_hit[chip*NumChipCh+chip_ch]->Fill(energy);}
    }
  }

  // ------- Filling hit timing & bcid  ----------------
  if ( ((ws->mode==0)&&(spillflag==1))||
      ((ws->mode==1)&&(ws->nblock%NbSubUpdate==0)) )
  { //NU beam trigger or Every 10 events for interanal trigger
    Float_t offset_time = bcid%2==0 ? OffsetTimeEven : OffsetTimeOdd;
    Float_t ramp_time   = bcid%2==0 ? RampTimeEven   : RampTimeOdd;
    Float_t hittiming = bcid*NormBCID + (data_time-offset_time)*ramp_time;

    ws->h_bcid   -> Fill(bcid);
    ws->h_timing -> Fill(hittiming);
  }

  // ------- Hits info for event display ----------------
  if( (ws->nblock - ws->lastblock_evtdisp > EvtDispUpdate ) ||
      ((ws->nblock - ws->lastblock_evtdisp < 0 ) && (ws->nblock>EvtDispUpdate) ))
  {
    if( ((ws->mode==0)&&(spillflag==1))||
        ((ws->mode==1)&&(ws->nblock%NbSubUpdate==0)) ) 
    {  
      if(hit==1){
        ws->nb_hits[sca]++;
        ws->hit_pln[sca][pln]=1;
        ws->hit_ch [sca][ ch]=1;
      }
    }
  }


  ws->nevt++;

  pthread_mutex_unlock(ws->updater_mutex);

} //newevt

// ==================================================================

bool DrawScintillator(Float_t x, Float_t y, bool grid)
{
  Float_t sci_x, sci_y;
  if(grid){ sci_x=SCINTI_W; sci_y=SCINTI_T; }
  else    { sci_x=SCINTI_T; sci_y=SCINTI_W; }

  TBox *scinti = new TBox(x-sci_x/2.,y-sci_y/2.,x+sci_x/2.,y+sci_y/2.);
  scinti->SetLineColor(kSpring-1);
  scinti->SetLineWidth(1);
  scinti->Draw("l same");
  return true;
} //DrawScintillator

// ==================================================================

bool DrawModule(Int_t dif_id, struct map_cell **mapping)
{

  Double_t FrameMargin = 10.;
  TH1F *frame = gPad->DrawFrame(
      -WATARTANK_X/2.-FrameMargin, 
      -WATARTANK_Y/2.-FrameMargin,
      WATARTANK_X/2. +FrameMargin,
      WATARTANK_Y/2. +FrameMargin);
  frame->GetXaxis()->SetTickLength(0.);
  frame->GetXaxis()->SetLabelSize (0.);
  frame->GetYaxis()->SetTickLength(0.);
  frame->GetYaxis()->SetLabelSize (0.);

  TBox *watertank = new TBox(
      -WATARTANK_X/2.,
      -WATARTANK_Y/2.,
      WATARTANK_X/2.,
      WATARTANK_Y/2.);
  watertank->SetFillColor(kCyan-10);
  watertank->Draw("same");

  for(int ichip=0;ichip<NumChip;ichip++){
    for(int ichipch=0;ichipch<NumChipCh;ichipch++){
      Int_t pln = mapping[ichip][ichipch].pln;
      Int_t ch  = mapping[ichip][ichipch].ch;
      if((pln<0)||(pln>7)||(ch<0)||(ch>79)){
        printf("wrong alignment of pln/ch, pln:%d ch:%d\n",pln,ch);
        return false;
      }
      bool isGrid;
      if(ch<40){isGrid=false;}else{isGrid=true;}
      Float_t XX,YY;
      XX = mapping[ichip][ichipch].z;
      if     ((dif_id-1)==SideView){ YY = mapping[ichip][ichipch].y; }
      else if((dif_id-1)==TopVeiw ){ YY = mapping[ichip][ichipch].x; }
      else{
        printf("wrong dif id\n");
        return false;
      }
#ifdef DEBUG
      printf("XX:%f, YY:%f\n",XX,YY);
#endif
      if(!DrawScintillator(XX,YY,isGrid)){ return false; }
    }
  }
  return true;
}//DrawModule

// ==================================================================

int main(int argc, char **argv) 
{

  if (argc!=3) {
    printf("usage : %s <dif_id 1 or 2 > <mode 0:beam, 1:pedestal>\n",argv[0]);
    exit(1);
  }
  Int_t dif_id = (Int_t)atoi(argv[1]);

  //initialize
  struct or_ws *ws=(struct or_ws *)malloc(sizeof(struct or_ws));
  TApplication ecalApp("App",&argc,argv);
  Int_t ini_time = (Int_t) time(NULL);
  Int_t bin_time = 1000;
  Int_t fin_time = ini_time + bin_time*20; //every 20sec
  //Int_t fin_time = ini_time + bin_time; //every 1sec

  ws->updater_mutex=(pthread_mutex_t*)malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(ws->updater_mutex,NULL);

  ws->dif_id = dif_id;
  ws->mapping=load_mapping(dif_id);
  if(!ws->mapping){
    printf("Cannot align wagasci mapping\n");
    return 1;
  }
  ws->nblock = 0;
  ws->nevt   = 0;
  //for(int ibunch=0;ibunch<8;ibunch++){
  //  Int_t   bcid = ExpectedBCIDstart + ibunch;
  //  Float_t offset_time = bcid%2==0 ? OffsetTimeEven : OffsetTimeOdd;
  //  Float_t ramp_time   = bcid%2==0 ? RampTimeEven   : RampTimeOdd;
  //  ws->timing_window[ibunch] 
  //    = bcid*NormBCID + (ExpectedHitTiming-offset_time)*ramp_time;
  //}

#ifdef __HttpServer__
  ws->serv=new THttpServer("http:8080");
#endif

  gStyle->SetOptStat(0);
  gStyle->SetPalette(1);
  Float_t margin_t = 0.00;
  Float_t margin_b = 0.10;
  Float_t margin_l = 0.05;
  Float_t margin_r = 0.02;
  Float_t title_size = 0.05;
  Float_t label_size = 0.04;
  gStyle->SetPadTopMargin   (margin_t);
  gStyle->SetPadBottomMargin(margin_b);
  gStyle->SetPadLeftMargin  (margin_l);
  gStyle->SetPadRightMargin (margin_r);

  //initialise canvas
  ws->canvas=new TCanvas("ecal","Ecal Dynamic Histogram",1920*0.6,1080*0.4);
  Float_t pad_div[NumHist][4] = {
    {0.01,0.51,0.19,0.99}, // 0.Noise rate
    {0.21,0.51,0.39,0.99}, // 1.Gain
    {0.01,0.26,0.39,0.49}, // 2.Noise rate stability
    {0.01,0.01,0.39,0.24}, // 3.Gain stability
    {0.41,0.51,0.59,0.99}, // 4.BCID for nu beam
    {0.61,0.51,0.79,0.99}, // 5.Hit timing for nu beam
    {0.41,0.01,0.59,0.49}, // 6.Spill number for nu beam
    {0.61,0.01,0.79,0.49}, // 7.Charge for hit channel
    {0.81,0.01,0.99,0.80}, // 8.Event display
    {0.81,0.81,0.99,0.90}  // 9.Status display
  };
  for(int ipad=0;ipad<NumHist;ipad++){
    string name = Form("pad%d",ipad);
    ws->pad[ipad]=new TPad(name.c_str(),name.c_str(),
        pad_div[ipad][0],
        pad_div[ipad][1],
        pad_div[ipad][2],
        pad_div[ipad][3]);
    ws->canvas->cd(0);
    ws->pad[ipad]->Draw();
  }

  // define plots
  Double_t max_gain  =  50;
  Double_t min_gain  =   0;
  Double_t max_nrate = 100;
  Double_t min_nrate =   0;
  Int_t bin_gain  = (Int_t) max_gain  - min_gain;
  Int_t bin_nrate = (Int_t) max_nrate - min_nrate;
  
  ws->pad[0]->cd();
  ws->pad[0]->SetTopMargin(0.02);
  ws->pad[0]->SetLeftMargin(0.07);
  ws->h_nrate = new TH1F("NoiseRate","",NumChip,.0,(Double_t)NumChip);
  ws->h_nrate ->SetMarkerStyle(23);
  ws->h_nrate ->SetMaximum(max_nrate);
  ws->h_nrate ->SetMinimum(min_nrate);
  for(int i=0;i<NumChip;i++){
    ws->h_nrate->SetBinContent(i+1,1);
  }
  ws->h_nrate->GetXaxis()->SetTitle("chip id");
  ws->h_nrate->GetXaxis()->CenterTitle();
  ws->h_nrate->GetXaxis()->SetTitleOffset(0.8);
  ws->h_nrate->GetXaxis()->SetTitleSize(title_size);
  ws->h_nrate->GetXaxis()->SetNdivisions(4);
  ws->h_nrate->GetXaxis()->SetLabelSize(label_size);
  ws->h_nrate->GetYaxis()->SetLabelSize(label_size);
  ws->h_nrate ->Draw("HIST p");
  TLegend *leg0 = new TLegend(margin_l,0.9,1.-margin_r,0.98);
  leg0->SetFillStyle(0);
  leg0->SetBorderSize(0);
  //leg0->SetHeader("Noise rate per chip [kHz]","c");
  leg0->Draw();

  ws->pad[1]->cd();
  ws->pad[1]->SetTopMargin(0.02);
  ws->pad[1]->SetGridx(1);
  ws->h_gain = new TH1F("Gain","",NumChAll,0.0,(Double_t)NumChAll);
  //ws->h_gain ->SetMarkerStyle(29);
  //ws->h_gain ->SetMarkerSize(1);
  ws->h_gain ->SetLineColor(kBlue);
  ws->h_gain ->SetMaximum(max_gain);
  ws->h_gain ->SetMinimum(min_gain);
  ws->h_gain ->GetXaxis()->SetTitle("chip_id");
  ws->h_gain ->GetXaxis()->CenterTitle();
  ws->h_gain ->GetXaxis()->SetTitleOffset(0.8);
  ws->h_gain ->GetXaxis()->SetTitleSize(title_size);
  for(int ichip=0;ichip<NumChip;ichip++){
    for(int ichipch=0;ichipch<NumChipCh;ichipch++){
      Int_t ibin = ichip*NumChipCh + ichipch + 1;
      if(ichipch==16&&ichip%5==0)
        ws->h_gain ->GetXaxis()->SetBinLabel(ibin,Form("%d",ichip));
      else
        ws->h_gain ->GetXaxis()->SetBinLabel(ibin,"");
    }
  }
  ws->h_gain ->GetXaxis()->SetLabelSize(label_size*1.5);
  ws->h_gain ->GetYaxis()->SetLabelSize(label_size);
  ws->h_gain ->GetXaxis()->SetTickLength(0);
  ws->h_gain ->GetXaxis()->SetNdivisions(NumChip);
  //ws->h_gain ->Draw("HIST p");
  ws->h_gain ->Draw("");
  TLegend *leg1 = new TLegend(margin_l+0.02,0.89,1.-margin_r,0.97);
  leg1->SetBorderSize(0);
  //leg1->SetHeader("Gain for each chip/ch ","c");
  leg1->Draw();


  ws->pad[2]->cd();
  ws->pad[2]->SetGridy(1);
  ws->pad[2]->SetTopMargin(0.03);
  ws->h_nrate_st = new TH2F("NoiseStability","",
      bin_time,ini_time,fin_time,bin_nrate,min_nrate,max_nrate);
  ws->h_nrate_st ->GetXaxis()->SetTimeDisplay(1);
  //ws->h_nrate_st ->GetXaxis()->SetTimeFormat("#splitline{%M:%M}{%d/%b}");
  ws->h_nrate_st ->GetXaxis()->SetTimeFormat("%d/%b");
  ws->h_nrate_st ->GetXaxis()->SetNdivisions(510);
  ws->h_nrate_st ->GetYaxis()->SetNdivisions(205);
  ws->h_nrate_st ->GetXaxis()->SetLabelOffset(0.01);
  ws->h_nrate_st ->GetXaxis()->SetLabelSize(label_size*2);
  ws->h_nrate_st ->GetYaxis()->SetLabelSize(label_size*2);
  ws->h_nrate_st ->Draw("colz");
  TLegend *leg2 = new TLegend(margin_l,0.79,1.-margin_r,0.97);
  leg2->SetFillStyle(0);
  leg2->SetBorderSize(0);
  //leg2->SetHeader("Noise rate stability","c");
  leg2->Draw();

  ws->pad[3]->cd();
  ws->pad[3]->SetGridy(1);
  ws->pad[3]->SetTopMargin(0.03);
  ws->h_gain_st = new TH2F("GainStability","",
      bin_time,ini_time,fin_time,bin_gain,min_gain,max_gain);
  ws->h_gain_st ->GetXaxis()->SetTimeDisplay(1);
  ws->h_gain_st ->GetXaxis()->SetTimeFormat("%d/%b");
  ws->h_gain_st ->GetXaxis()->SetNdivisions(510);
  ws->h_gain_st ->GetYaxis()->SetNdivisions(205);
  ws->h_gain_st ->GetXaxis()->SetLabelOffset(0.01);
  ws->h_gain_st ->GetXaxis()->SetLabelSize(label_size*2);
  ws->h_gain_st ->GetYaxis()->SetLabelSize(label_size*2);
  ws->h_gain_st ->Draw("colz");
  TLegend *leg3 = new TLegend(margin_l,0.79,1.-margin_r,0.97);
  leg3->SetFillStyle(0);
  leg3->SetBorderSize(0);
  //leg3->SetHeader("Gain stability","c");
  leg3->Draw();


  ws->pad[4]->cd();
  ws->pad[4]->SetTopMargin(0.1);
  //ws->pad[4]->SetLogy(1);
  ws->h_bcid = new TH1F("Bcid","",300,0.0,100.0);
  ws->h_bcid ->SetFillColor(kBlue);
  ws->h_bcid ->SetLineColor(kBlue);
  ws->h_bcid ->Draw();
  TLine *line_bcid0
    = new TLine(ExpectedBCIDstart-0.33,0.,ExpectedBCIDstart-0.33,1e+8);
  TLine *line_bcid1
    = new TLine(ExpectedBCIDstart+8-0.33,0.,ExpectedBCIDstart+8-0.33,1e+8);
  line_bcid0->SetLineColor(kRed);
  line_bcid1->SetLineColor(kRed);
  line_bcid0->SetLineStyle(2);
  line_bcid1->SetLineStyle(2);
  line_bcid0->Draw();
  line_bcid1->Draw();
  TLegend *leg4 = new TLegend(margin_l+0.01,0.91,1.-margin_r,1.-margin_t);
  //leg4->SetFillStyle(0);
  leg4->SetBorderSize(0);
  //leg4->SetHeader("Coarse trigger timing (BCID)","c");
  leg4->Draw();

  ws->pad[5]->cd();
  ws->pad[5]->SetTopMargin(0.1);
  ws->pad[5]->SetLogy(1);
  ws->h_timing = new TH1F("Hit Timing","",15000,0.0,15000.);
  ws->h_timing ->SetFillColor(kBlue);
  ws->h_timing ->SetLineColor(kBlue);
  ws->h_timing ->Draw();
  TLegend *leg5 = new TLegend(margin_l+0.01,0.91,1.-margin_r,1.-margin_t);
  //leg5->SetFillStyle(0);
  leg5->SetBorderSize(0);
  //leg5->SetHeader("Hit timing","c");
  leg5->Draw();

  ws->pad[6]->cd();
  ws->h_spillnb = new TH1F("SpillNb","",SpillNbUpdate,0.0,(Double_t)SpillNbUpdate);
  ws->h_spillnb ->SetFillColor(kBlue);
  ws->h_spillnb ->Draw();
  TLegend *leg6 = new TLegend(margin_l+0.01,0.89,1.-margin_r,0.97);
  //leg6->SetFillStyle(0);
  leg6->SetBorderSize(0);
  //leg6->SetHeader("Spill number recorded","c");
  leg6->Draw();

  ws->pad[7]->cd();
  ws->h_charge = new TH1F("HitCharge","",500,0.0,2000.);
  ws->h_charge ->Draw();
  TLegend *leg7 = new TLegend(margin_l+0.01,0.89,1.-margin_r,0.97);
  //leg7->SetFillStyle(0);
  leg7->SetBorderSize(0);
  //leg7->SetHeader("ADC value for hits","c");
  leg7->Draw();

  ws->pad[8]->cd();
  ws->pad[8]->SetTopMargin   (0.01);
  ws->pad[8]->SetBottomMargin(0.01);
  ws->pad[8]->SetLeftMargin  (0.01);
  ws->pad[8]->SetRightMargin (0.01);
  ws->g_evtdisp2D = new TGraph(NumChAll);
  ws->g_evtdisp2D ->GetXaxis()->SetLabelSize(0);
  ws->g_evtdisp2D ->GetYaxis()->SetLabelSize(0);
  ws->g_evtdisp2D ->GetXaxis()->SetTickLength(0);
  ws->g_evtdisp2D ->GetYaxis()->SetTickLength(0);
  if(!DrawModule(dif_id,ws->mapping)){
    printf("cannot draw modules\n");
    return 1;
  }
  for(int chip=0;chip<NumChip;chip++){
    for(int chip_ch=0;chip_ch<NumChipCh;chip_ch++){
      int ip = chip*NumChipCh + chip_ch;
      ws->g_evtdisp2D->SetPoint(ip,-WATARTANK_X,-WATARTANK_Y);
    }
  }
  ws->g_evtdisp2D ->SetMarkerStyle(20); //20:circle, 29:star
  ws->g_evtdisp2D ->SetMarkerSize(1.3);
  ws->g_evtdisp2D ->SetMarkerColor(kRed);
  ws->g_evtdisp2D ->Draw("P same");
  TLegend *leg8 = new TLegend(margin_l+0.01,0.9,1.-margin_r,1.);
  leg8->SetFillStyle(0);
  leg8->SetBorderSize(0);
  //leg8->SetHeader("Event display","c");
  leg8->Draw();

  ws->pad[9]->cd();
  ws->pad[9]->SetTopMargin   (0.01);
  ws->pad[9]->SetBottomMargin(0.02);
  ws->pad[9]->SetLeftMargin  (0.01);
  ws->pad[9]->SetRightMargin (0.01);
  ws->pad[9]->SetGridx(1);
  ws->pad[9]->SetGridy(1);
  ws->h_status = new TH2F("OM Status","",4,0.,4.,2,0.,2.);
  ws->h_status->GetXaxis()->SetLabelSize(0);
  ws->h_status->GetYaxis()->SetLabelSize(0);
  ws->h_status->GetXaxis()->SetTickLength(0);
  ws->h_status->GetYaxis()->SetTickLength(0);
  ws->h_status->GetXaxis()->SetNdivisions(4);
  ws->h_status->GetYaxis()->SetNdivisions(2);
  ws->h_status->GetZaxis()->SetRange(-1.,2.);
  ws->h_status ->Draw("colz");
  TText *txt9_1 = new TText(0.10,0.5,"Noise R");
  TText *txt9_2 = new TText(0.10,1.5,"Gain");
  TText *txt9_3 = new TText(1.10,0.5,"BCID");
  TText *txt9_4 = new TText(1.10,1.5,"Hit time");
  TText *txt9_5 = new TText(2.10,0.5,"spill#");
  TText *txt9_6 = new TText(2.10,1.5,"ADC");
  TText *txt9_7 = new TText(3.10,1.5,"Hit ch.");
  txt9_1->SetTextSize(0.3);
  txt9_2->SetTextSize(0.3);
  txt9_3->SetTextSize(0.3);
  txt9_4->SetTextSize(0.3);
  txt9_5->SetTextSize(0.3);
  txt9_6->SetTextSize(0.3);
  txt9_7->SetTextSize(0.3);
  txt9_1->Draw();
  txt9_2->Draw();
  txt9_3->Draw();
  txt9_4->Draw();
  txt9_5->Draw();
  txt9_6->Draw();
  txt9_7->Draw();
  memset(ws->nb_out_nr  ,0,NumChip);
  memset(ws->nb_out_gain,0,NumChAll);
  ws->nb_bcid       = 0;
  ws->nb_out_bcid   = 0;
  ws->nb_out_timing = 0;
  ws->mean_charge   = 0;
  ws->nb_spillgap   = 0;
  ws->nb_nonhit_ch  = NumChAll;

  ws->canvas->cd(0);
  ws->txt_time  = new TText();
  ws->txt_spill = new TText();
  ws->txt_time  ->SetTextSize(0.03);
  ws->txt_spill ->SetTextSize(0.03);
  ws->txt_time  ->Draw();
  ws->txt_spill ->Draw();


#ifdef __HttpServer__
  ws->serv->Register("energy",ws->histo);
#endif

  ws->canvas->cd(0);

  ws->mode = (Int_t)atoi(argv[2]);
  char* data_source_name = Form("converter_dif_1_1_%d",dif_id);

  ws->loop=new_event_loop(data_source_name,ws,newevt,newblock,endblock,reinit,'f');
  ws->first_time=true;
  run_loop(ws->loop);

  ws->updater_t=new TThread("Updater thread",updater,(void *)ws);
  ws->updater_t->Run();

  //run the TApp
  ecalApp.Run();

  //never reached
  return 0;
} //main


// ==================================================================

