/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Calicoes.
*/

STOPPED="stopped";
ENDED="ended";
run_script_container_visible=false;

/* Get script list */
function update_script_list(callback) {
  pyrame.sendcmd(return_get_file_list,["script_files",4,true,callback],manager_port,"get_scripts_rc");
}
document.getElementById("get_script_list").onclick=update_loaded_script;


/* Load script */
document.getElementById("reload_script").onclick=load_script;
document.getElementById("script_files").onchange=load_script;

function load_script() {
  var selected_script=document.getElementById("script_files").value;
  if (selected_script=="None") {
    selected_script="/opt/pyrame/rc_scripts/standard/normal_run.py";
  }
  document.getElementById("run_script_container").style.display="none";
  document.getElementById("progress_bar").innerHTML="";
  document.getElementById("progress_bar").style.width="0";
  document.getElementById("progress_bar_msg").innerHTML="not running";
  pyrame.sendcmd(update_loaded_script,"force",manager_port,"load_script_rc",selected_script);
}


/* update loaded script */
function update_loaded_script(retcode,res,force) {
  if (retcode=="0") {
    add_to_error_log(res,1);
    return;
  }
  // Do nothing if we aren't on the script_run_tab
  if (!document.getElementById("script_run_tab").classList.contains("active")) return;
  // First update script list, then get_loaded_script, then show parameters
  update_script_list(function(){
    pyrame.sendcmd(return_get_loaded_script,null,manager_port,"get_loaded_script_rc");
  });
  function return_get_loaded_script(retcode,res) {
    if (retcode=="0") {
      add_to_error_log(res,1);
      return;
    }
    if (res!="None") {
      update_progress_bar();
    }
    if (res=="None") {
      load_script();
      return;
    }
    var sel=document.getElementById("script_files");
    var opts=sel.options;
    for (var opt,j=0;opt=opts[j];j++) {
      if (opt.value==res) {
        sel.selectedIndex=j;
        break;
      }
    }
    // At this point we are sure that a script is loaded (res!="")
    // If the run_script_container is not shown, force load of params
    if (!run_script_container_visible) {
      force="force";
    }
    // If there's nothing to change, do nothing
    if (force!="force" && document.getElementById("current_script").innerHTML==res) return;
    document.getElementById("current_script").innerHTML=res;
    document.getElementById("script_files").value=res;
    // If result is not a file, hide run_script_container and do not continue
    if (res[0]!="/") {
      document.getElementById("run_script_container").style.display="none";
      return;
    }
    function return_get_script_params(retcode,res) {
      if (retcode=="0") {
        add_to_error_log(res,1);
        return;
      }
      script_params=[];
      var params_form="";
      if (res!="") {;
        var p=res.split(",");
        for (var i=0;i<p.length;i++) {
          var params=p[i].split(":").map(atob);
          var param_name=params[0];
          var param_desc=params[1];
          var param_default=params[2];
          script_params.push(param_name);
          params_form+="<p>"+param_desc+": <input type=\"text\" id=\"script_param_"+param_name+"\" value=\""+param_default+"\"></p>";
        }
      }
      document.getElementById("script_params").innerHTML=params_form;
      if (script_params.length!=0) {
        document.getElementById("script_param_"+script_params[0]).focus();
      }
      document.getElementById("run_script_container").style.display="block";
      run_script_container_visible=true;
      update_progress_bar(true);
    }
    pyrame.sendcmd(return_get_script_params,null,manager_port,"get_script_params_rc");
  }
}

/* Update progress bar */
function update_progress_bar(force) {
  // Do nothing if we aren't on the rc tab
  if (!document.getElementById("script_run_tab").classList.contains("active")) return;
  // Do nothing if run_script_container is not visible
  if (!run_script_container_visible) return;
  // Do nothing if it's not running
  if (!force) {
    if (document.getElementById("progress_bar_msg").innerHTML==STOPPED ||
        document.getElementById("progress_bar_msg").innerHTML=="not running" ||
        document.getElementById("progress_bar_msg").innerHTML==ENDED) return;
  }
  function return_get_progress(retcode,res) {
    if (retcode=="0") {
      add_to_error_log(res,1);
      return;
    }
    if (res=="end") {
      document.getElementById("progress_bar").innerHTML="";
      document.getElementById("progress_bar").style.width="0";
      document.getElementById("progress_bar_msg").innerHTML=ENDED;
      document.getElementById("start_script").disabled=false;
      document.getElementById("stop_script").disabled=true;
    } else if (res=="running") {
      document.getElementById("progress_bar").innerHTML="";
      document.getElementById("progress_bar").style.width="100%";
      document.getElementById("progress_bar_msg").innerHTML=res;
      document.getElementById("start_script").disabled=true;
      document.getElementById("stop_script").disabled=false;
    } else if (res=="not running") {
      document.getElementById("progress_bar").innerHTML="";
      document.getElementById("progress_bar").style.width="0";
      document.getElementById("progress_bar_msg").innerHTML=res;
      document.getElementById("start_script").disabled=false;
      document.getElementById("stop_script").disabled=true;
    } else if (res=="stop") {
      document.getElementById("progress_bar").innerHTML="";
      document.getElementById("progress_bar_msg").innerHTML=STOPPED;
      document.getElementById("start_script").disabled=false;
      document.getElementById("stop_script").disabled=true;
    } else if (res=="0") {
      document.getElementById("progress_bar").innerHTML="1%";
      document.getElementById("progress_bar").style.width="1%";
      document.getElementById("progress_bar_msg").innerHTML="";
      document.getElementById("start_script").disabled=true;
      document.getElementById("stop_script").disabled=false;
    } else {
      document.getElementById("progress_bar").innerHTML=res+"%";
      document.getElementById("progress_bar").style.width=res+"%";
      document.getElementById("progress_bar_msg").innerHTML="";
      document.getElementById("start_script").disabled=true;
      document.getElementById("stop_script").disabled=false;
    }
  }
  pyrame.sendcmd(return_get_progress,null,manager_port,"get_script_progress_rc");
  function return_get_last_error(retcode,res) {
    if (retcode=="0" || res!="") {
      add_to_error_log(res,1);
      return;
    }
  }
  var date=new Date();
  pyrame.sendcmd(return_get_last_error,null,manager_port,"get_last_script_error_rc");
}


/* Run script */
document.getElementById("start_script").onclick=start_script;

function start_script() {
  var returns_set_script=0;
  function return_set_script_param(retcode,res,i) {
    if (retcode=="0") {
      add_to_error_log(res,1);
      return;
    }
    returns_set_script+=1;
    if (returns_set_script==script_params.length) {
      document.getElementById("progress_bar").style.width="1%";
      document.getElementById("progress_bar").innerHTML="1%";
      document.getElementById("progress_bar_msg").innerHTML="";
      pyrame.sendcmd(generic_return,null,manager_port,"start_script_rc");
    }
  }
  if (script_params.length!=0) {
    for (var i=0;i<script_params.length;i++) {
      pyrame.sendcmd(return_set_script_param,null,manager_port,"set_script_param_rc",script_params[i],document.getElementById("script_param_"+script_params[i]).value);
    }
  } else {
    document.getElementById("progress_bar").style.width="1%";
    document.getElementById("progress_bar").innerHTML="1%";
    document.getElementById("progress_bar_msg").innerHTML="";
    pyrame.sendcmd(generic_return,null,manager_port,"start_script_rc");
  }
}


/* Stop script */
document.getElementById("stop_script").onclick=function() {
  pyrame.sendcmd(generic_return,null,manager_port,"stop_script_rc","5");
  document.getElementById("stop_script").disabled=true;
}

/* Load and start */
function load_and_start_script(script,param1,value1) {
  // accepting just one param for the moment
  function return_ls_load_script(retcode,res) {
    if (retcode=="0") {
      add_to_error_log(res,1);
      return;
    }
    function return_ls_set_script_param(retcode,res) {
      if (retcode=="0") {
        add_to_error_log(res,1);
        return;
      }
      pyrame.sendcmd(generic_return,null,manager_port,"start_script_rc");
    }
    pyrame.sendcmd(return_ls_set_script_param,null,manager_port,"set_script_param_rc",param1,value1);
  }
  pyrame.sendcmd(return_ls_load_script,null,manager_port,"load_script_rc",script);
}
