/*
Copyright 2012-2017 Frédéric Magniette, Miguel Rubio-Roy
This file is part of Calicoes.
*/

var default_params={"dest_datadir":"/home/data/nodb"};

/* Update state machine */
function update_state_machine() {
  // Do nothing if we aren't on the operation tab
  if (!document.getElementById("sm_tab").classList.contains("active") &&
      !document.getElementById("stats_tab").classList.contains("active")) return;
  if (document.getElementById("dialog_bg").style.display != "none") return;
  // Disable everything before updating
  var active_states=document.querySelectorAll("div.state.active");
  for (var i=0;i<active_states.length;i++)
    active_states[i].classList.remove("active");
  for (var i=0;i<transition_buttons.length;i++)
    transition_buttons[i].disabled=true;
  document.getElementById("acq_id").innerHTML=acq_state_string;
  // Do nothing if state machine not ready (config set)
  if (!sm_ready) return;
  function return_get_state(retcode,res) {
    if (retcode=="0") {
      add_to_error_log(res,1);
      return;
    }
    if (res.toLowerCase()=="reconfigured") {
      res="CONFIGURED"; // for the GUI, RECONFIGURED OR CONFIGURED is the same
    }
    document.getElementById(res.toLowerCase()).classList.add("active");
    switch(res) {
      case "UNDEFINED":
        document.getElementById("init").disabled=false;
        stats_ready=false;
        break;
      case "READY":
        document.getElementById("config").disabled=false;
        document.getElementById("deinit").disabled=false;
        stats_ready=false;
        break;
      case "CONFIGURED":
        document.getElementById("start_acq").disabled=false;
        document.getElementById("inval").disabled=false;
        stats_ready=false;
        break;
      case "ACQUIRING":
        document.getElementById("stop_acq").disabled=false;
        stats_ready=true;
        pyrame.sendcmd(return_acq_id,null,manager_port,"getvar_varmod","varmod","ecal","current_acq_id");
        break;
    }
    function return_acq_id(retcode,res) {
      if (retcode=="0") {
        add_to_error_log(res,1);
        return;
      }
      document.getElementById("acq_id").innerHTML=acq_state_string+"<br>"+res;
    }
  }
  pyrame.sendcmd(return_get_state,null,manager_port,"get_state_ecal","ecal");
}


/* State machine transition handler */
function transition(e) {
  stats_ready=false;
  var transition_name=e.currentTarget.id;
  var transition_fallback=e.currentTarget.getAttribute("fallback");
  function return_transition(retcode,res) {
    if (retcode=="0") add_to_error_log(res,1);
    update_state_machine();
  }
  for (var i=0;i<transition_buttons.length;i++)
    transition_buttons[i].disabled=true;
  var api=[];
  // We present the start_acq api like the startacq.sh script
  if (transition_name=="start_acq") {
    api=["acq_name"];
  }
  var argument_list=[return_transition,null,manager_port,"transition_cmod","ecal","false",transition_name,transition_fallback];
  if (api.length!=0) {
    f_ok = function() {
      var params={};
      for (var i=0;i<api.length;i++) {
        params[api[i]]=document.getElementById("dialog_prompt_"+api[i]).value;
      }
      if (transition_name=="start_acq") {
        load_and_start_script("/opt/pyrame/rc_scripts/standard/start_acq.py",api[0],params[api[0]]);
        return;
      }
      var ext_params=JSON.stringify(params);
      argument_list.push(ext_params);//.replace(/"/g,"'"));
      pyrame.sendcmd.apply(pyrame,argument_list);
    }
    f_cancel=function() {
      update_state_machine();
    }
    dialog("Introduce the following parameters for the transition "+transition_name+":",api,f_ok,argument_list,f_cancel,[]);
  } else {
    pyrame.sendcmd.apply(pyrame,argument_list);
  }
}

