/// \file
/// \ingroup tutorial_tree
/// \notebook -nodraw
///  Read data from an ascii file and create a root file with an histogram and an ntuple.
///  See a variant of this macro in basic2.C.
///
/// \macro_code
///
/// \author Rene Brun

#include "Riostream.h"
#include "TString.h"
#include "TGraph.h"
#include "TDatime.h"
#include "TCanvas.h"

using namespace std;

void hvgraph() {
// read file $ROOTSYS/tutorials/tree/basic.dat
// this file has 3 columns of float data
   TString dir = "/home/data/";
   ifstream in;
   in.open(Form("%shv.log",dir.Data()));

   TString day, hour, units;
   Float_t current;

   Int_t nlines = 0;

   while (1 && nlines<1) {
     in >> day >> hour >> current >> units;
     if (!in.good()) break;
     nlines++;
   }

   TString tok;
   TString filename="hv";
   Ssiz_t from = 0;
   while(day.Tokenize(tok, from, "[/]")) {
     filename+="-"+tok;
   }
   from = 0;
   while(hour.Tokenize(tok, from, "[:]")) {
     filename+="_"+tok;
   }
   std::cout<<"HV Power Supply current log: "<< filename << std::endl;

   auto f = TFile::Open("/home/data/"+filename+".root","RECREATE");

   auto *graph= new TGraph;

   nlines=0;
   while (1) {
     in >> day >> hour >> current >> units;
      if (!in.good()) break;

      //      if (nlines < 5) std::cout<<day<<" "<<hour<<" "<<current<<" "<<units<<std::endl;

      TString tok;
      TString y,m,d,h,min,sec;
      Ssiz_t from = 0;
      while(day.Tokenize(tok, from, "[/]")) {
	if(from==5) y=tok;
        if(from==8) m=tok;
        if(from==-1) d=tok;
      }
      from = 0;
      while(hour.Tokenize(tok, from, "[:]")) {
	if(from==3) h=tok;
        if(from==6) min=tok;
        if(from==9) sec=tok;
      }

      TDatime da1(y.Atoi(),m.Atoi(),d.Atoi(),h.Atoi(),min.Atoi(),sec.Atoi());
      
      Int_t np=graph->GetN();
      graph->SetPoint(np,da1.Convert(),1000000*current);
	
      nlines++;
   }

   auto *c1 = new TCanvas(filename,filename,0,0,1800,600);
   
   c1->cd();

   graph->SetMarkerStyle(20);

   graph->Draw("al");
   graph->GetXaxis()->SetTimeDisplay(1);
   graph->GetXaxis()->SetNdivisions(514);
   graph->GetXaxis()->SetTimeFormat("#splitline{%d/%m}{%H:%M}");
   graph->GetYaxis()->SetTitle("uA");
   graph->GetXaxis()->SetTimeOffset(0,"local");
   graph->GetXaxis()->SetLabelOffset(0.022);

   printf(" found %d points\n",nlines);

   in.close();
   f->cd();
   c1->Write();
   graph->SetName("hv_graph");
   graph->Write();
   f->Write();
}
