#define plotMips_commissioning_cxx
#include "plotMips_commissioning.h"
#include <TPaveStats.h>
#include <TH2.h>
#include <TStyle.h>
#include <TCanvas.h>
#include <TString.h>
#include <iostream>
#include <TFitResult.h>

using namespace std;

void plotMips_commissioning::Loop()
{

  bool global = true;
  TString slab = "17";
  int maxnhit=5;

  if (fChain == 0) return;

  Long64_t nentries = fChain->GetEntriesFast();

  std::vector<std::vector<std::vector<float> > > ped_mean;
  std::vector<std::vector<std::vector<float> > > ped_rms;

  std::vector<std::vector<float> > signal_mean;
  std::vector<std::vector<float> > signal_rms;

  for(int ichip=0; ichip<16; ichip++) {
    ped_mean.push_back(std::vector<std::vector<float> >() );
    ped_rms.push_back(std::vector<std::vector<float> >() );
    signal_mean.push_back(std::vector<float> () );
    signal_mean.push_back(std::vector<float> () );

    for(int ichn=0; ichn<64; ichn++) {
      ped_mean.at(ichip).push_back(std::vector<float> () );
      ped_rms.at(ichip).push_back(std::vector<float> () );
    }
  }
  //

  // --------------------
  // all sca
  //  std::vector<TCanvas*> chip;
  std::vector<std::vector<TH1F*> > signal;
  std::vector<std::vector<TH1F*> > ped ;
  std::vector<std::vector<std::vector<TH1F*> > > ped_sca ;

  TH1F *hits_sca0 = new TH1F("hits_sca0","hits_sca0",16,-0.5,15.5);
  TH1F *hits = new TH1F("hits","hits",16,-0.5,15.5);

  std::vector<TH1F*> mip;
  std::vector<TH1F*> noise;

  TH1F* mip_all = new TH1F("mip_all","mip_all",4096,-499.5,3596.5);

  for(int ichip=0; ichip<16; ichip++) {

    TH1F *temp = new TH1F("mip histo","mip histo",4096,-499.5,3596.5);
    TH1F *temp2 = new TH1F("noise histo","noise histo",4096,-499.5,3596.5);

    if( (ichip % 4) == 0) mip.push_back(temp);
    if( (ichip % 4) == 0) noise.push_back(temp2);

    std::vector<TH1F*> signaltemp;
    std::vector<TH1F*> pedtemp;
    std::vector<std::vector<TH1F*> >pedtemp_sca;

    for(int ichn=0; ichn<64; ichn++) {
      TH1F *temp = new TH1F("temp","temp",4096,-499.5,3596.5);
      TH1F *temp2 = new TH1F("temp2","temp2",4096,-499.5,3596.5);
      signaltemp.push_back(temp);
      pedtemp.push_back(temp2);
      std::vector<TH1F*> pedtemp_sca2;
      for(int isca=0; isca<15; isca++) {
	TH1F *temp = new TH1F("temp","temp",4096,-499.5,3596.5);
	pedtemp_sca2.push_back(temp);
      }
      pedtemp_sca.push_back(pedtemp_sca2);
    }
    signal.push_back(signaltemp);
    ped.push_back(pedtemp);
    ped_sca.push_back(pedtemp_sca);
     
   }
  
  
  // --------------------
  //  sca 0
  //  std::vector<TCanvas*> chip_0;
  std::vector<std::vector<TH1F*> > signal_0;
  std::vector<std::vector<TH1F*> > ped_0;
   
  for(int ichip=0; ichip<16; ichip++) {
    std::vector<TH1F*> signaltemp;
    std::vector<TH1F*> pedtemp;
    for(int ichn=0; ichn<64; ichn++) {
      TH1F *temp = new TH1F("temp","temp",4096,-499.5,3596.5);
      TH1F *temp2 = new TH1F("temp2","temp2",4096,-499.5,3596.5);
      signaltemp.push_back(temp);
      pedtemp.push_back(temp2);
    }
    signal_0.push_back(signaltemp);
    ped_0.push_back(pedtemp);
     
  }


  // -----------------------------------------------------------------------------------------------------   
  // SCA analysis
  Long64_t nbytes = 0, nb = 0;
  for (Long64_t jentry=0; jentry<nentries;jentry++) {
    Long64_t ientry = LoadTree(jentry);
    if (ientry < 0) break;
    nb = fChain->GetEntry(jentry);   nbytes += nb;

    std::vector<bool> gooddata;
     
    for(int isca=0; isca<15; isca++) gooddata.push_back(true);

    if(global == true) {

      for(int isca=0; isca<15; isca++) {
	int ntaggedasbad = 0;
	for(int ichip=0; ichip<16; ichip++) {
	  for(int ichn=0; ichn<64; ichn++) {
	    if(charge_hiGain[ichip][isca][ichn]==4) {
	      ntaggedasbad++;
	    }
	  }//ichn           
	}//ichip
	if(ntaggedasbad>0) gooddata.at(isca) = false;
      }//isca

    }

    for(int ichip=0; ichip<16; ichip++) {
      for(int ichn=0; ichn<64; ichn++) {
	for(int isca=0; isca<15; isca++) {
	  if(charge_hiGain[ichip][isca][ichn]>10 && gain_hit_high[ichip][isca][ichn]==0 && badbcid[ichip][isca]==0 && nhits[ichip][isca]<maxnhit+1 && gooddata.at(isca)==true)
	    ped_sca.at(ichip).at(ichn).at(isca)->Fill(charge_hiGain[ichip][isca][ichn]);
	}//isca           
      }//ichn
    }//ichip 
   
  }
  for(int ichip=0; ichip<16; ichip++) {
    for(int ichn=0; ichn<64; ichn++) {
      for(int isca=0; isca<15; isca++) {
	ped_mean.at(ichip).at(ichn).push_back(ped_sca.at(ichip).at(ichn).at(isca)->GetMean());
	ped_rms.at(ichip).at(ichn).push_back(ped_sca.at(ichip).at(ichn).at(isca)->GetRMS());
      }
    }
  }
  // -----------------------------------------------------------------------------------------------------
  // Final analysis
   
  nbytes = 0, nb = 0;
  for (Long64_t jentry=0; jentry<nentries;jentry++) {
    Long64_t ientry = LoadTree(jentry);
    if (ientry < 0) break;
    nb = fChain->GetEntry(jentry);   nbytes += nb;
     
    std::vector<bool> gooddata;
     
    for(int isca=0; isca<15; isca++) gooddata.push_back(true);

   if(global == true) {

      for(int isca=0; isca<15; isca++) {
	int ntaggedasbad = 0;
	for(int ichip=0; ichip<16; ichip++) {
	  for(int ichn=0; ichn<64; ichn++) {
	    if(charge_hiGain[ichip][isca][ichn]==4) {
	      ntaggedasbad++;
	    }
	  }//ichn           
	}//ichip
	if(ntaggedasbad>0) gooddata.at(isca) = false;
      }//isca

    }

    for(int ichip=0; ichip<16; ichip++) {
      for(int ichn=0; ichn<64; ichn++) {
	for(int isca=0; isca<15; isca++) {
	  if( gooddata.at(isca)==true ){
	    if(charge_hiGain[ichip][isca][ichn]>10 && badbcid[ichip][isca]==0 && nhits[ichip][isca]<maxnhit+1 ) {
	      if(gain_hit_high[ichip][isca][ichn]==0) {
		ped.at(ichip).at(ichn)->Fill(charge_hiGain[ichip][isca][ichn]-ped_mean.at(ichip).at(ichn).at(isca));
		
		if(ichip == 0 || ichip == 1 || ichip ==2 || ichip ==3) noise.at(0)->Fill(charge_hiGain[ichip][isca][ichn]-ped_mean.at(ichip).at(ichn).at(isca));
		if(ichip == 4 || ichip == 5 || ichip ==6 || ichip ==7) noise.at(1)->Fill(charge_hiGain[ichip][isca][ichn]-ped_mean.at(ichip).at(ichn).at(isca));
		if(ichip == 8 || ichip == 9 || ichip ==10 || ichip ==11) noise.at(2)->Fill(charge_hiGain[ichip][isca][ichn]-ped_mean.at(ichip).at(ichn).at(isca));
		if(ichip == 12 || ichip == 13 || ichip ==14 || ichip ==15) noise.at(3)->Fill(charge_hiGain[ichip][isca][ichn]-ped_mean.at(ichip).at(ichn).at(isca));
	      }
	      if(gain_hit_high[ichip][isca][ichn]==1) { 
		signal.at(ichip).at(ichn)->Fill(charge_hiGain[ichip][isca][ichn]-ped_mean.at(ichip).at(ichn).at(isca));
		hits->Fill(ichip);
		if(ichip == 0 || ichip == 1 || ichip ==2 || ichip ==3) mip.at(0)->Fill(charge_hiGain[ichip][isca][ichn]-ped_mean.at(ichip).at(ichn).at(isca));
		if(ichip == 4 || ichip == 5 || ichip ==6 || ichip ==7) mip.at(1)->Fill(charge_hiGain[ichip][isca][ichn]-ped_mean.at(ichip).at(ichn).at(isca));
		if(ichip == 8 || ichip == 9 || ichip ==10 || ichip ==11) mip.at(2)->Fill(charge_hiGain[ichip][isca][ichn]-ped_mean.at(ichip).at(ichn).at(isca));
		if(ichip == 12 || ichip == 13 || ichip ==14 || ichip ==15) mip.at(3)->Fill(charge_hiGain[ichip][isca][ichn]-ped_mean.at(ichip).at(ichn).at(isca));

	        mip_all->Fill(charge_hiGain[ichip][isca][ichn]-ped_mean.at(ichip).at(ichn).at(isca));

	      }
	    }
	  }
	}//isca
	 
	if( gooddata.at(0)==true ) {

	  if(charge_hiGain[ichip][0][ichn]>10 && badbcid[ichip][0]==0 && nhits[ichip][0]<maxnhit+1 ) {
	    if(gain_hit_high[ichip][0][ichn]==0) 
	      ped_0.at(ichip).at(ichn)->Fill(charge_hiGain[ichip][0][ichn]);
	    if(gain_hit_high[ichip][0][ichn]==1) {
	      signal_0.at(ichip).at(ichn)->Fill(charge_hiGain[ichip][0][ichn]);
	      hits_sca0->Fill(ichip);
	    }
	  }
	}

      }//ichn
    }//ichip
  }

  TCanvas *canvas_mip_all = new TCanvas("mip_all","mip_all",800,800);
  canvas_mip_all->cd(1);
  // gPad->SetLogy();
  mip_all->GetXaxis()->SetRangeUser(-100,500);
  mip_all->GetXaxis()->SetTitle("ADC");
  mip_all->GetYaxis()->SetTitle("#");
  mip_all->SetTitle("MIP (all chips/channel/SCA)");
  mip_all->Draw("");

  TCanvas *canvas_hits = new TCanvas("canvas_hits","canvas_hits",1200,400);
  canvas_hits->Divide(2,1);
  canvas_hits->cd(1);
  hits_sca0->GetXaxis()->SetTitle("chip");
  hits_sca0->GetYaxis()->SetTitle("# hits sca=0");
  hits_sca0->Draw("");
  canvas_hits->cd(2);
  hits->GetXaxis()->SetTitle("chip");
  hits->GetYaxis()->SetTitle("# hits all sca");
  hits->Draw("");

  TCanvas *canvas_mip = new TCanvas("canvas_mip","canvas_mip",1200,1200);
  canvas_mip->Divide(2,2);

  canvas_hits->Print("Nhits_slab"+slab+".eps");


  // for(int ichip=0; ichip<4; ichip++) {

    canvas_mip->cd(1);
    gPad->SetLogy();
    int ichip=0;
    noise.at(ichip)->GetXaxis()->SetRangeUser(-100,500);
    noise.at(ichip)->SetTitle(TString::Format("MIP, wafer-%i",ichip+1));
    noise.at(ichip)->GetXaxis()->SetTitle("ADC");
    noise.at(ichip)->GetYaxis()->SetTitle("#");
    noise.at(ichip)->SetLineColor(2);
    noise.at(ichip)->Draw("ls");
    mip.at(ichip)->GetXaxis()->SetRangeUser(-100,500);
    mip.at(ichip)->SetTitle(TString::Format("MIP, wafer-%i",ichip+1));
    mip.at(ichip)->GetXaxis()->SetTitle("ADC");
    mip.at(ichip)->GetYaxis()->SetTitle("#");
    mip.at(ichip)->Draw("sames");

    TFitResultPtr fit_result_wafer1 = LandauFit(*mip.at(ichip));
    TF1 *fit_wafer1 = mip.at(ichip)->GetFunction("landau_gauss_function");
    cout<<TString::Format("MIP, wafer-%i",ichip+1)<<endl;
    cout<<"max ="<<fit_wafer1->GetMaximumX()<< " MPV=" <<fit_wafer1->GetParameter(1)<<" +- " <<fit_wafer1->GetParError(1)<< ", Chi2/NDF="<<  fit_wafer1->GetChisquare()/fit_wafer1->GetNDF()<<endl;


    canvas_mip->cd(3);
    gPad->SetLogy();
    ichip=1;
    noise.at(ichip)->GetXaxis()->SetRangeUser(-100,500);
    noise.at(ichip)->SetTitle(TString::Format("MIP, wafer-%i",ichip+1));
    noise.at(ichip)->GetXaxis()->SetTitle("ADC");
    noise.at(ichip)->GetYaxis()->SetTitle("#");
    noise.at(ichip)->SetLineColor(2);
    noise.at(ichip)->Draw("ls");

    mip.at(ichip)->GetXaxis()->SetRangeUser(-100,500);
    mip.at(ichip)->SetTitle(TString::Format("MIP, wafer-%i",ichip+1));
    mip.at(ichip)->GetXaxis()->SetTitle("ADC");
    mip.at(ichip)->GetYaxis()->SetTitle("#");
    mip.at(ichip)->Draw("sames");

    TFitResultPtr fit_result_wafer2 = LandauFit(*mip.at(ichip));
    TF1 *fit_wafer2 = mip.at(ichip)->GetFunction("landau_gauss_function");
    cout<<TString::Format("MIP, wafer-%i",ichip+1)<<endl;
    cout<<"max ="<<fit_wafer2->GetMaximumX()<< " MPV=" <<fit_wafer2->GetParameter(1)<<" +- " <<fit_wafer2->GetParError(1)<< ", Chi2/NDF="<<  fit_wafer2->GetChisquare()/fit_wafer2->GetNDF()<<endl;


    canvas_mip->cd(2);
    gPad->SetLogy();
    ichip=2;
    noise.at(ichip)->GetXaxis()->SetRangeUser(-100,500);
    noise.at(ichip)->SetTitle(TString::Format("MIP, wafer-%i",ichip+1));
    noise.at(ichip)->GetXaxis()->SetTitle("ADC");
    noise.at(ichip)->GetYaxis()->SetTitle("#");
    noise.at(ichip)->SetLineColor(2);
    noise.at(ichip)->Draw("ls");

    mip.at(ichip)->GetXaxis()->SetRangeUser(-100,500);
    mip.at(ichip)->SetTitle(TString::Format("MIP, wafer-%i",ichip+1));
    mip.at(ichip)->GetXaxis()->SetTitle("ADC");
    mip.at(ichip)->GetYaxis()->SetTitle("#");
    mip.at(ichip)->Draw("sames");

    TFitResultPtr fit_result_wafer3 = LandauFit(*mip.at(ichip));
    TF1 *fit_wafer3 = mip.at(ichip)->GetFunction("landau_gauss_function");
    cout<<TString::Format("MIP, wafer-%i",ichip+1)<<endl;
    cout<<"max ="<<fit_wafer3->GetMaximumX()<< " MPV=" <<fit_wafer3->GetParameter(1)<<" +- " <<fit_wafer3->GetParError(1)<< ", Chi2/NDF="<<  fit_wafer3->GetChisquare()/fit_wafer3->GetNDF()<<endl;


    canvas_mip->cd(4);
    gPad->SetLogy();
    ichip=3;
    noise.at(ichip)->GetXaxis()->SetRangeUser(-100,500);
    noise.at(ichip)->SetTitle(TString::Format("MIP, wafer-%i",ichip+1));
    noise.at(ichip)->GetXaxis()->SetTitle("ADC");
    noise.at(ichip)->GetYaxis()->SetTitle("#");
    noise.at(ichip)->SetLineColor(2);
    noise.at(ichip)->Draw("ls");

    mip.at(ichip)->GetXaxis()->SetRangeUser(-100,500);
    mip.at(ichip)->SetTitle(TString::Format("MIP, wafer-%i",ichip+1));
    mip.at(ichip)->GetXaxis()->SetTitle("ADC");
    mip.at(ichip)->GetYaxis()->SetTitle("#");
    mip.at(ichip)->Draw("sames");

    TFitResultPtr fit_result_wafer4 = LandauFit(*mip.at(ichip));
    TF1 *fit_wafer4 = mip.at(ichip)->GetFunction("landau_gauss_function");
    cout<<TString::Format("MIP, wafer-%i",ichip+1)<<endl;
    cout<<"max ="<<fit_wafer4->GetMaximumX()<< " MPV=" <<fit_wafer4->GetParameter(1)<<" +- " <<fit_wafer4->GetParError(1)<< ", Chi2/NDF="<<  fit_wafer4->GetChisquare()/fit_wafer4->GetNDF()<<endl;


    canvas_mip->Print("MIP_slab"+slab+".eps");
  

}



static double landau_gauss(double *x, double *par) {
  //Fit parameters:
  //par[0]=Width (scale) parameter of Landau density
  //par[1]=Most Probable (MP, location) parameter of Landau density
  //par[2]=Total area (integral -inf to inf, normalization constant)
  //par[3]=Width (sigma) of convoluted Gaussian function
  //
  //In the Landau distribution (represented by the CERNLIB approximation),
  //the maximum is located at x=-0.22278298 with the location parameter=0.
  //This shift is corrected within this function, so that the actual
  //maximum is identical to the MP parameter.
    
  // Numeric constants
  Double_t invsq2pi = 0.3989422804014;   // (2 pi)^(-1/2)
  Double_t mpshift = -0.22278298;       // Landau maximum location
    
  // Control constants
  Double_t np = 200.0;      // number of convolution steps
  Double_t sc = 5.0;      // convolution extends to +-sc Gaussian sigmas
    
  // Variables
  Double_t xx;
  Double_t mpc;
  Double_t fland;
  Double_t sum = 0.0;
  Double_t xlow, xupp;
  Double_t step;
  Double_t i;
    
  // MP shift correction
  mpc = par[1] - mpshift * par[0];
    
  // Range of convolution integral
  xlow = x[0] - sc * par[3];
  xupp = x[0] + sc * par[3];
    
  step = (xupp - xlow) / np;
    
  // Convolution integral of Landau and Gaussian by sum
  for (i = 1.0; i <= np / 2; i++) {
    xx = xlow + (i - .5) * step;
    fland = TMath::Landau(xx, mpc, par[0]) / par[0];
    sum += fland * TMath::Gaus(x[0], xx, par[3]);
        
    xx = xupp - (i - .5) * step;
    fland = TMath::Landau(xx, mpc, par[0]) / par[0];
    sum += fland * TMath::Gaus(x[0], xx, par[3]);
  }
    
  return (par[2] * step * sum * invsq2pi / par[3]);
}

TFitResultPtr plotMips_commissioning::LandauFit(TH1F &hist)
{
  //int Module = Chip2Module(Chip); //iLayer from 1 to 5
  double max = hist.GetMaximum();

  //cout << "Max : " << max << " Mean " << hist.GetMean() << " X max " << hist.GetXaxis()->GetXmax() << endl;
 
  double lower_bound = hist.GetXaxis()->GetBinLowEdge(TMath::Max(15,hist.FindFirstBinAbove(max/4)));
  double upper_bound = hist.GetXaxis()->GetBinUpEdge(TMath::Max(hist.FindLastBinAbove (max/8),200));
  double parameters[4];
    
  //par[0]=Width (scale) parameter of Landau density
  //par[1]=Most Probable (MP, location) parameter of Landau density
  //par[2]=Total area (integral -inf to inf, normalization constant)
  //par[3]=Width (sigma) of convoluted Gaussian function

  // parameters[0] = 50;
   parameters[0] = 15; //I modified it!
   //if(N==0) parameters[1] = FillMIPValue(Module);
    parameters[1] = hist.GetMean();
   
  //parameters[2] = hist.Integral("width")*1.5;
  parameters[2] = hist.Integral("width"); //I modified it
  //parameters[3] = 50;
  parameters[3] = hist.GetRMS();// I modified it
    
  //cout << "Lower bound : " << lower_bound << " Upper bound : " << upper_bound << endl;

  TF1 *landau_gauss_function  = new TF1("landau_gauss_function", landau_gauss, 15,200,4);//lower_bound, upper_bound, 4);
  //TF1 *landau_gauss_nobound = new TF1("landau_gauss_function", landau_gauss, 0, 4000, 4);
  TF1 *landau_gauss_nobound = new TF1("landau_gauss_nobound", landau_gauss, 0.5, 6, 4); //I modified it
  TF1 *landau_gauss_iterations[3];
    
  // Set style for convenience-function plots
  landau_gauss_nobound  ->SetLineStyle(5);
  landau_gauss_function ->SetNpx(300);
  landau_gauss_nobound  ->SetNpx(300);
  landau_gauss_function ->SetParameters(parameters);
  landau_gauss_function ->SetParNames("landau width", "mpv", "area", "gauss width");
  // Logging
  landau_gauss_iterations[0] = (TF1*)landau_gauss_function->Clone("0th_iteration");
  landau_gauss_iterations[0]->SetLineColor(46);
    
  // 1st fit fitting only area
  // landau_gauss_function->FixParameter(0, parameters[0]);
  //landau_gauss_function->FixParameter(1, parameters[1]);
  //landau_gauss_function->FixParameter(3, parameters[3]);
  //hist.Fit(landau_gauss_function, "RQB");
  // Logging
  landau_gauss_iterations[1] = (TF1*)landau_gauss_function->Clone("1st_iteration");
  landau_gauss_iterations[1]->SetLineColor(43);
 
    
  // 2nd fit releasing landau width, mpv and gauss width
  //landau_gauss_function->ReleaseParameter(0);
  //landau_gauss_function->ReleaseParameter(1);
  //landau_gauss_function->ReleaseParameter(3);
    
  //if(landau_gauss_function->GetParameter(0) < 15)   landau_gauss_function->SetParameter(0, 15);
  //if(landau_gauss_function->GetParameter(0) > 1000) landau_gauss_function->SetParameter(0, 1000);
  //if(landau_gauss_function->GetParameter(3) < 20)   landau_gauss_function->SetParameter(3, 20);
  //if(landau_gauss_function->GetParameter(3) > 100)  landau_gauss_function->SetParameter(3, 100);
  //landau_gauss_function->SetParLimits(0, 15, 1000);
  //landau_gauss_function->SetParLimits(3, 20, 500);
    
  //hist.Fit(landau_gauss_function, "RQB");

  // Logging
  landau_gauss_iterations[2] = (TF1*)landau_gauss_function->Clone("2nd_iteration");
  landau_gauss_iterations[2]->SetLineColor(41);
    
  // 3rd fit with corrected range
  //TH1F tmp("tmp", "tmp", 2501, -.5, 2500.5);
  // TH1F tmp("tmp", "tmp", 200, 0, 11); //I modified it
  //max = landau_gauss_function->GetMaximum();
  //tmp.Eval(landau_gauss_function);
  //lower_bound = tmp.GetBinCenter(tmp.FindFirstBinAbove(.30*max));
  //upper_bound = tmp.GetBinCenter(tmp.FindLastBinAbove (.30*max));
  //lower_bound = tmp.GetBinCenter(tmp.FindFirstBinAbove(0.5*max));
  //upper_bound = tmp.GetBinCenter(tmp.FindLastBinAbove (0.5*max));
  //landau_gauss_function->SetRange(lower_bound, upper_bound);

  // correct parameters landau and gauss width
  landau_gauss_function->GetParameters(parameters);
  //if(parameters[3] >= 90. || parameters[3] <= 35.) parameters[3] = 50;
  //if(parameters[0] < 30.) parameters[3] = 30;
  //  if(parameters[3] >= 0.5) //I modified it
  //  parameters[3] = 0.1; //I modified it

  landau_gauss_function->SetParameters(parameters);
  // landau_gauss_function->SetParLimits(0, 5, 1000);
  //landau_gauss_function->SetParLimits(3, 15, 500);
  TFitResultPtr result = hist.Fit(landau_gauss_function, "RQBS");
    
  landau_gauss_function->GetParameters(parameters);
  landau_gauss_nobound ->SetParameters(parameters);
    
  // add functions to histograms to be saved in root file
  //hist.GetListOfFunctions()->Add(landau_gauss_iterations[0]);
  //hist.GetListOfFunctions()->Add(landau_gauss_iterations[1]);
  //hist.GetListOfFunctions()->Add(landau_gauss_iterations[2]);
  hist.GetListOfFunctions()->Add(landau_gauss_nobound);
  //hist.GetListOfFunctions()->Add(landau_gauss_function, "R");
    


  return result;
}
