#!/usr/bin/python2
# -*- coding: utf-8 -*-

def tdc_charge_init():
    declare_param("acq_time","Acquisition time (min)","4")
    declare_param("gtrigger","Global trigger threshold","110")
    declare_param("trig_on","Comma-separated list of channels with trigger ON","8")
    declare_param("preamp_on","Comma-separated list of channels with preamp ON","8")
    declare_param("inputDAC","Comma-separated list value of DAC input","3")
    declare_param("high_gain_preamp","Comma-separated list value of high gain preamp","56")
    declare_param("launch_root","Launch analysis in ROOT (ON/OFF)","OFF")

def tdc_charge_run(acq_time,gtrigger,trig_on,preamp_on,inputDAC,high_gain_preamp,launch_root):
    CHIP = "spiroc2d"
    inpDac_on = "0-35" #channel #overwrite
    #inputDAC  = "3" #DAC
    date = time.strftime("%Y%m/%d_%H%M")
    print("Date: %s"%(date))
    run_name = "mppc_2D/"+date+"_trig%s"%(gtrigger)
    acq_name = "trig%s"%(gtrigger)

    configure()
    for roc in list_dev(CHIP,"root"):
        reconfigure(roc,"disallow_trig_chans_{0}".format(CHIP),"0-35")
        reconfigure(roc,"allow_trig_chans_{0}".format(CHIP),trig_on)
        reconfigure(roc,"disable_preamp_chans_{0}".format(CHIP),"0-35")
        reconfigure(roc,"enable_preamp_chans_{0}".format(CHIP),preamp_on)
        reconfigure(roc,"set_hg_preamp_chans_{0}".format(CHIP),preamp_on,high_gain_preamp)
        reconfigure(roc,"disable_input_dac_chans_{0}".format(CHIP),"0-35")
        reconfigure(roc,"enable_input_dac_chans_{0}".format(CHIP),inpDac_on)
        reconfigure(roc,"set_input_dac_chans_{0}".format(CHIP),"0-35",inputDAC)
        reconfigure(roc,"set_gtrigger_{0}".format(CHIP),gtrigger)

    #start the run
    run=new_run(run_name)

    print("File name = %s"%(run_name))
    print("Run path = %s"%(run["path"]))
    print("Acquisition time = %d sec"%(float(acq_time)*60))
    print("Trigger enabled: ch %s"%(trig_on))
    print("Run with trigger = %s"%(gtrigger))
    print("PreAmp enabled: ch %s"%(preamp_on))
    print("InpurDAC enabled: ch %s"%(inpDac_on))
    print("InpurDAC value = %s"%(inputDAC))

    set_param_run(run,"acq_time",acq_time)
    set_param_run(run,"gtrigger",gtrigger)
    set_param_run(run,"trig_on",trig_on)
    set_param_run(run,"preamp_on",preamp_on)
    set_param_run(run,"launch_root",launch_root)

    timed_acq(acq_name,run,float(acq_time)*60)
    if launch_root.lower()=="on":
        wait_run_finished(run)
        rawfilename = "%s/%s_by_dif0.raw"%(run["path"],acq_name)
        rc_exec("/opt/root_scripts/forFG/electronics %s"%(rawfilename))

def tdc_charge_get_progress():
    # progress_acq is updated by the function timed_acq
    return progress_acq

execfile("/opt/calicoes/pycaldaq_rcs.py")

