#!/usr/bin/python2
# -*- coding: utf-8 -*-

def tdc_charge_response_init():
    declare_param("acq_time","Acquisition time (min)","4")
    declare_param("gtrigger","Global trigger threshold","165")
    declare_param("trig_on","Comma-separated list of channels with trigger ON","14")
    declare_param("preamp_on","Comma-separated list of channels with preamp ON","14")
    declare_param("high_gain","High gain preamp value","56")
    declare_param("launch_root","Launch analysis in ROOT (ON/OFF)","OFF")

def tdc_charge_response_run(acq_time,gtrigger,trig_on,preamp_on,high_gain,launch_root):
    CHIP = "spiroc2d"
    inpDac_on = "0-35" #channel #overwrite
    inputDAC  = "1" #DAC
    date = time.strftime("%Y%m/%d_%H%M")
    print("Date: %s"%(date))
    run_name = "mppc_2D/"+date+"_trig%s"%(gtrigger)
    acq_name = "trig%s"%(gtrigger)

    for roc in list_dev(CHIP,"root"):
	reconfigure(roc,"set_hg_preamp_chans_{0}".format(CHIP),preamp_on,high_gain)
	##reconfigure(roc,"set_lg_preamp_chans_{0}".format(CHIP),chans,value)
	reconfigure(roc,"disallow_trig_chans_{0}".format(CHIP),"0-35")
	reconfigure(roc,"allow_trig_chans_{0}".format(CHIP),trig_on)

	reconfigure(roc,"disable_preamp_chans_{0}".format(CHIP),"0-35")
	reconfigure(roc,"enable_preamp_chans_{0}".format(CHIP),preamp_on)

	reconfigure(roc,"disable_input_dac_chans_{0}".format(CHIP),"0-35")
	reconfigure(roc,"enable_input_dac_chans_{0}".format(CHIP),inpDac_on)
	reconfigure(roc,"set_input_dac_chans_{0}".format(CHIP),"0-35",inputDAC)

    for roc in list_dev(CHIP,"root"):
	reconfigure(roc,"set_gtrigger_{0}".format(CHIP),gtrigger)

    print("File name = %s"%(run_name))
    print("Acquisition time = %d sec"%(float(acq_time)*60))
    print("Trigger enabled: ch %s"%(trig_on))
    print("Run with trigger = %s"%(gtrigger))
    print("PreAmp enabled: ch %s"%(preamp_on))
    print("InpurDAC enabled: ch %s"%(inpDac_on))
    print("InpurDAC value = %s"%(inputDAC))
    print("High Gain value = %s"%(high_gain))

    #start the run
    run=new_run(run_name)

    set_param_run(run,"acq_time",acq_time)
    set_param_run(run,"gtrigger",gtrigger)
    set_param_run(run,"trig_on",trig_on)
    set_param_run(run,"preamp_on",preamp_on)
    set_param_run(run,"high_gain",high_gain)
    set_param_run(run,"launch_root",launch_root)

    acq=timed_acq(acq_name,run,float(acq_time)*60)
    wait_run_finished(run)
    if launch_root=="ON":
        rawfilename = "%s/%s_dif_1_1_1.raw"%(acq["path"],acq_name)
        rc_exec(". /opt/root/bin/thisroot.sh ; /opt/root_scripts/forFG/electronics %s"%(rawfilename))

def tdc_charge_response_get_progress():
    # progress_acq is updated by the function timed_acq
    return progress_acq

execfile("/opt/calicoes/pycaldaq_rcs.py")

