#include "cmd_sdhf.hpp"

struct sdhf_workspace *gws;


void newblock(void *workspace,struct block *b) {
  struct el_workspace *ws;
  int new_spill;
  int i;
  if ((b==NULL) || (workspace==NULL))
    return;
  ws=(struct el_workspace *)workspace;
  ws->mute=0;
  //printf("new block\n");

  //handle lost spills
  new_spill=atoi(get_block_field(b,"spill"));

  if (ws->index>0) {
    printf("warning : sdhf : block not sent : ws->index=%d\n",ws->index);
  }
  ws->index=0;
  
 
  //removing obviously bad spills
  if (new_spill>ws->last_spill+100) {  
    printf("warning : sdhf : crap spill %d on dif %d: rejecting\n",new_spill,ws->dif_id);
    ws->mute=1;
    return;
  }

  //removing already sent spill
  if (new_spill<ws->last_spill+1) {  
    printf("warning : sdhf : spill %d already sent on dif %d: rejecting\n",new_spill,ws->dif_id);
    ws->mute=1;
    return;
  }

  //send empty spills to complete the sequence
  //printf("waiting for %d getting %d on dif %d\n",ws->last_spill+1,new_spill,ws->dif_id);
  if (new_spill>ws->last_spill+1) {  
    printf("warning : sdhf : compensating from %d to %d\n",ws->last_spill+1,new_spill);
    for(i=ws->last_spill+1;i<new_spill;i++) {
      printf("publish spill %d on dif %d\n",i,ws->dif_id);
      ws->ds->publish(i,0,0);
    }
  }

  ws->last_spill=new_spill;
}

void endblock(void *workspace,struct block *b) {
  struct el_workspace *ws=(struct el_workspace *)workspace;
  //printf("endblock with ws->index=%d\n",ws->index);
   if (!ws->mute)  {
     printf("publish spill %d on dif %d\n",ws->last_spill,ws->dif_id);
     ws->ds->publish(ws->last_spill,0,ws->index);
   }
   ws->index=0;
}


void newevt(void *workspace,struct event *e) {
  struct el_workspace *ws=(struct el_workspace *)workspace;
  struct CaloTransHit hit;

  //printf("new event\n");
  if (ws->mute)  {
    //printf("muted event\n");
    return;
  }

  //fill the hit
  hit.dif_id=ws->dif_id;;
  hit.asic_id=atoi(get_event_field(e,"roc"));
  hit.chan_id=atoi(get_event_field(e,"chan"));
  hit.map_x=atoi(get_event_field(e,"l"));
  hit.map_y=atoi(get_event_field(e,"c"));
  hit.map_z=(int)atof(get_event_field(e,"p"));;
  hit.x=atof(get_event_field(e,"x"));
  hit.y=atof(get_event_field(e,"y"));
  hit.z=atof(get_event_field(e,"z"));
  hit.asic_bcid=atoi(get_event_field(e,"bcid"));
  hit.spill=atoi(get_event_field(e,"spill"));
  hit.time=atof(get_event_field(e,"time"));
  hit.adc_energy=atoi(get_event_field(e,"en"));;
  hit.energy=0.0;
  hit.sca=0;//atoi(get_event_field(e,"sca"));
  memcpy((void *)((char *)ws->ds->payload()+ws->index),(void *)&hit,sizeof(struct CaloTransHit));
  ws->index+=sizeof(struct CaloTransHit);
  //printf("event stocked, index now at %d\n",ws->index);
}

void reinit(void *workspace) {
  struct el_workspace *ws=(struct el_workspace *)workspace;

  ws->index=0;  
  ws->last_spill=0;
}

extern "C" void *init();
void *init() {
  //common workspace declarations
  struct sdhf_workspace *ws=(struct sdhf_workspace *)malloc(sizeof(struct sdhf_workspace));
  memset(ws,0,sizeof(struct sdhf_workspace));
  return ws;
}

void print_env() {
  struct el_workspace *ews=gws->chain;
  printf("nb ecal dif = %d\n",gws->nb_ecaldif);
  while(ews!=NULL) {
    printf("ews difid=%d\n",ews->dif_id);
    ews=ews->next;
  }
}

extern "C" struct cmd_result *init_sdhf(struct cmd *,void *);
//pyrapi:init_sdhf:dev_name,parent_device,source_name,dim_dns
struct cmd_result *init_sdhf(struct cmd *command,void *workspace) {
  struct sdhf_workspace *ws=(struct sdhf_workspace *)workspace;
  struct cmd_result *cres;
  int id;
  struct el_workspace *ews;
  char *tmp;
  struct cmd_result *pres;
  
  if (command->nb_params!=4) {
    return build_result(0,"init_sdhf, %d arguments given, 4 needed : dev_name parent_device source_name dim_dns",command->nb_params);
  }

  gws=ws;

  //register in cmod
  cres=submod_execcmd("new_device_cmod","sdhf",command->params[0],command->params[1],"end");
  if (cres->retcode==0) {
    pres=build_result(0,"sdhf: cant register in cmod <- %s",cres->str);
    free_cmd_result(cres);
    return pres;
  }
  ws->cmod_id=strdup(cres->str);
  free_cmd_result(cres);
  submod_execcmd("set_param@cmod",ws->cmod_id,"sdhf_dim_dns",command->params[3],"end");
  submod_execcmd("set_param@cmod",ws->cmod_id,"sdhf_source_name",command->params[2],"end");


  //set the dim dns
  printf("setting dns name = %s\n",command->params[3]);
  setenv("DIMDNS",command->params[3],1);
  dim_set_dns_node(command->params[3]);
  

  //init levbdim service
  DimServer::start("sdhf");
  
  //get the data source ids
  cres=submod_execcmd("getid_byname_acq",command->params[2],"end");
  if (cres->retcode==0) {
    printf("cant get data sources : %s\n",cres->str);
    submod_exit(1);
  } else {
    printf("sources = %s\n",cres->str);
  }
  //parse ids list
  tmp=cres->str;
  ws->nb_ecaldif=0;
  while(sscanf(tmp,"%d",&id)==1) {
    printf("registering dif id = %d\n",id);

    //allocate new workspace
    ews=(struct el_workspace *)malloc(sizeof(struct el_workspace));
    memset(ews,0,sizeof(struct el_workspace));

    //init last_spill
    ews->last_spill=0;

    //set the dif id
    ews->dif_id=id+1;

    //create the datasource
    ews->ds=new levbdim::datasource(1100,id+1,BUF_SIZE);

    //init index
    ews->index=0;

    //init event loop
    ews->el=new_event_loop("localhost",id,ews,newevt,newblock,endblock,reinit,FIRST);

    //chain the workspace
    ews->next=ws->chain;
    ws->chain=ews;
    
    while(*tmp!=0 && *tmp!=',')
      tmp++;
    if (*tmp!=0)
      tmp++;
    ws->nb_ecaldif++;
  }
  free_cmd_result(cres);

  //printf("sdhf env after init\n");
  //print_env();
  return build_result(1,ws->cmod_id);
}

extern "C" struct cmd_result *deinit_sdhf(struct cmd *,void *);
//pyrapi:deinit_sdhf:
struct cmd_result *deinit_sdhf(struct cmd *command,void *workspace) {
  struct sdhf_workspace *ws=(struct sdhf_workspace *)workspace;
  struct el_workspace *ews=ws->chain;
  struct el_workspace *todel;
  printf("sdhf env before deinit\n");
  print_env();
  while(ews!=NULL) {
    delete ews->ds;
    stop_loop(ews->el);
    free_event_loop(ews->el);
    todel=ews;
    ews=ews->next;
    free(todel);
  }
  ws->chain=NULL;
  free(ws->cmod_id);
  return build_result(1,"ok");
}

extern "C" struct cmd_result *start_acq_fin_sdhf(struct cmd *,void *);
//pyrapi:startacq_sdhf: 
struct cmd_result *start_acq_fin_sdhf(struct cmd *command,void *workspace) {
  struct sdhf_workspace *ws=(struct sdhf_workspace *)workspace;
  struct el_workspace *ews=ws->chain;
  //waiting for the chain to be properly initialized
  sleep(2);
  while(ews!=NULL) {
    run_loop(ews->el);
    ews=ews->next;
  }
  return build_result(1,"ok");
}

extern "C" struct cmd_result *stop_acq_sdhf(struct cmd *,void *);
//pyrapi:stopacq_sdhf: 
struct cmd_result *stop_acq_sdhf(struct cmd *command,void *workspace) {
  struct sdhf_workspace *ws=(struct sdhf_workspace *)workspace;
  struct el_workspace *ews=ws->chain;
  while(ews!=NULL) {
    stop_loop(ews->el);
    ews=ews->next;
  }
  return build_result(1,"ok");
}
