#include "cmd_sdhr.h"

struct sdhr_workspace *gws;

void print_hit(struct CaloTransHit *thishit) {
  printf("hit on dif %d asic %d chan %d\n",thishit->dif_id,thishit->asic_id,thishit->chan_id);
  printf("spill=%d asic_bcid=%ld time=%f\n",thishit->spill,(long int)thishit->asic_bcid,thishit->time);
  printf("relative coordinates (%d,%d,%d)\n",thishit->map_x,thishit->map_y,thishit->map_z);
  printf("coordinates (%f,%f,%f)\n",thishit->x,thishit->y,thishit->z);
  printf("sca=%d en_adc=%d energy=%f\n",thishit->sca,thishit->adc_energy,thishit->energy);
  
}

void cons_serv_received(long *tag,char *buffer,int *size) {
  int nb_hits;
  int i;
  struct CaloTransHit *thishit;
  int payload_size;
  int hitsize=56;//(int)sizeof(struct CaloTransHit);
  int headersize=20;//(int)sizeof(struct CaloTransHeader);
  //int difid;
  float x,y,z;
  float newx,newy,newz;
  struct incr_string *dout;

  //printf("new block of size=%d on tag %ld\n",*size,*tag);

  if (*size==0) {
    return;
  }

  if (!gws->acq_active) {
    printf("acquition is not active...throwing away data\n");
    return;
  }

  if (*size<headersize) {
    printf("error : buffer size is lower than header size\n");
    return;
  }
  payload_size=*size-headersize;
  //printf("sizeof(hit)=%d\n",hitsize);
  if (payload_size%hitsize!=0) {
    printf("warning : sdhr : size is not adapted : reste=%d\n",payload_size%hitsize);
    return;
  }
  nb_hits=(*size-headersize)/hitsize;
 
  pthread_mutex_lock(gws->mutex);
  for(i=0;i<nb_hits;i++) {
    
    thishit=(struct CaloTransHit *)(buffer+headersize+i*hitsize);
    print_hit(thishit);
    //difid=thishit->dif_id;
    
    if (gws->acq_socket==-1) {
      printf("warning : sdhr : socket is not properly opened\n");
      pthread_mutex_unlock(gws->mutex);
      break;
    }
    
    //permute axis
    x=-1*thishit->y+gws->offset_x;
    y=-1*thishit->x+gws->offset_y;
    z=-1*thishit->z+gws->offset_z;

    //rotate around y axis
    newx=x*cos(gws->angle_y)+z*sin(gws->angle_y);
    newz=z*cos(gws->angle_y)-x*sin(gws->angle_y);
    x=newx;
    z=newz;

    //rotate around x axis (should be 0)
    newy=y*cos(gws->angle_x)-z*sin(gws->angle_x);
    newz=y*sin(gws->angle_x)+z*cos(gws->angle_x);
    y=newy;
    z=newz;

    //rotate around z axis (must be 0)
    newx=x*cos(gws->angle_z)-y*sin(gws->angle_z);
    newy=x*sin(gws->angle_z)+y*cos(gws->angle_z);
    x=newx;
    y=newy;

    //add one to spill to begin at pos 1
    thishit->spill++;

    dout=new_is();
    
    if (i==0) { //send the block props
      fsuffix_is(dout,"%d,%d,%d,%f",thishit->spill,thishit->dif_id,thishit->map_z,z);
    }
    
    //send the hit to acq
    //format="!bcid,time|dif,roc,chan,c,l,p,x,y|en";
    fsuffix_is(dout,"!%d,%f|%d,%d,%d,%d,%f,%f|%d",(int)thishit->asic_bcid,thishit->time,thishit->asic_id,thishit->chan_id,thishit->map_x,thishit->map_y,x,y,thishit->adc_energy);
    if (i==nb_hits-1) {
      fsuffix_is(dout,"\n");
    }
    write(gws->acq_socket,dout->str,dout->size);
    //printf(dout->str);
    free_is(dout);
  }
  pthread_mutex_unlock(gws->mutex);
}

void *init() {
  //common workspace declarations
  struct sdhr_workspace *ws=malloc(sizeof(struct sdhr_workspace));
  memset(ws,0,sizeof(struct sdhr_workspace));
  ws->dif_buffer=malloc(32000*sizeof(int));
  ws->acq_active=0;
  gws=ws;
  return ws;
}

//pyrapi:init_sdhr:dev_name,parent_device,dim_dns,offset_x,offset_y,offset_z,offset_bcid
struct cmd_result *init_sdhr(struct cmd *command,void *workspace) {
  struct sdhr_workspace *ws=(struct sdhr_workspace *)workspace;
  struct cmd_result *cres;
  char tmp[2048];
  char name[1024];
  int i;
  struct cmd_result *pres;

  //check parameters
  if (command->nb_params!=10) {
    return build_result(0,"init_sdhr, %d arguments given, 10 needed : dev_name parent_device dim_dns offset_x offset_y offset_z offset_bcid angle_x angle_y angle_z",command->nb_params);
  }

  //register in cmod
  cres=submod_execcmd("new_device_cmod","sdhr",command->params[0],command->params[1],"end");
  if (cres->retcode==0) {
    pres=build_result(0,"sdhr: cant register in cmod <- %s",cres->str);
    free_cmd_result(cres);
    return pres;
  }
  ws->cmod_id=strdup(cres->str);
  free_cmd_result(cres);
  submod_execcmd("set_param_cmod",ws->cmod_id,"sdhr_dim_dns",command->params[2],"end");
  submod_execcmd("set_param_cmod",ws->cmod_id,"sdhr_offset_x",command->params[3],"end");
  submod_execcmd("set_param_cmod",ws->cmod_id,"sdhr_offset_y",command->params[4],"end");
  submod_execcmd("set_param_cmod",ws->cmod_id,"sdhr_offset_z",command->params[5],"end");
  submod_execcmd("set_param_cmod",ws->cmod_id,"sdhr_offset_bcid",command->params[6],"end");
   submod_execcmd("set_param_cmod",ws->cmod_id,"sdhr_angle_x",command->params[7],"end");
  submod_execcmd("set_param_cmod",ws->cmod_id,"sdhr_angle_y",command->params[8],"end");
  submod_execcmd("set_param_cmod",ws->cmod_id,"sdhr_angle_z",command->params[9],"end");

  //set the dim dns node
  ws->server=strdup(command->params[2]);
  setenv("DIMDNS",ws->server,1);
  dim_set_dns_node(ws->server);

  //set the coordinates offset
  ws->offset_x=atof(command->params[3]);
  ws->offset_y=atof(command->params[4]);
  ws->offset_z=atof(command->params[5]);
  ws->offset_bcid=atoi(command->params[6]);

   //set the coordinate reverse (boolean)
   ws->angle_x=atof(command->params[7]);
   ws->angle_y=atof(command->params[8]);
   ws->angle_z=atof(command->params[9]);

  //setup a mutex for avoiding data mix
  ws->mutex=malloc(sizeof(pthread_mutex_t));
  pthread_mutex_init(ws->mutex,NULL);

   //create the acqunit
  //sprintf(tmp,"%d",50000+i); //port
  sprintf(tmp,"%d",50000); //port
  //sprintf(name,"shdcal_dif%d_",i);
  sprintf(name,"shdcal_difs_");
  cres=submod_execcmd("newunit_acq","1","/opt/pyrame/tcps_acq.so","/opt/pyrame/uncap_dummy.so","sdhcal_dif",tmp,"null","null",name,"100000","0","end");
  if (cres->retcode==0) {
    pres=build_result(0,"unable to open new acq unit : %s",cres->str);
    free_cmd_result(cres);
    return pres;
  } else {
    ws->auid=strdup(cres->str);
    free_cmd_result(cres);
  }
  
  /*//get the stream id 
  cres=submod_execcmd("get_fid_acq",ws->auid[i],"end");
  if (cres->retcode==0) {
    submod_setres(0,"cant get fid : %s",cres->str);
    free_cmd_result(cres);
    return;
  }
  ws->fid[i]=strdup(cres->str);
  free_cmd_result(cres);
  
  //start the converter
    cres=submod_execcmd("start_converter_acq",ws->fid[i],"/opt/pyrame/hle_decoder.so","spill,dif,p,z!bcid,time|roc,chan,c,l,x,y|en","spill","end");
    if (cres->retcode==0) {
    submod_setres(0,"cant launch converter : %s",cres->str);
    free_cmd_result(cres);
    return;
    }
    free_cmd_result(cres);*/
  
  for (i=0;i<NB_DIFS;i++) {

    
    sprintf(tmp,"/LEVBDIM/DS-101-%d/DATA",sdhcal_difs[i]);
    //sprintf(tmp,"/LEVBDIM/DS-1100-%d/DATA",i+1);
    ws->sid[i]=dic_info_service(tmp,MONITORED,0,ws->dif_buffer,32000,cons_serv_received,i,NULL,0);
    printf("sid=%d\n",ws->sid[i]);
    
  }
  
  return build_result(1,ws->cmod_id);
}

//pyrapi:deinit_sdhr:
struct cmd_result *deinit_sdhr(struct cmd *command,void *workspace) {
  struct sdhr_workspace *ws=(struct sdhr_workspace *)workspace;
  int i;
  for (i=0;i<NB_DIFS;i++) {
    dic_release_service(ws->sid[i]); 
  }
  pthread_mutex_lock(gws->mutex);

  if (ws->acq_socket!=-1) {
    close(ws->acq_socket);
  }
  free(ws->mutex);
  return build_result(1,"ok");
}

//pyrapi:start_acq_sdhr: 
struct cmd_result *start_acq_sdhr(struct cmd *command,void *workspace) {
  struct sdhr_workspace *ws=(struct sdhr_workspace *)workspace;
  //int i;
  ws->acq_socket=open_socket("localhost",50000);
  if (ws->acq_socket==-1) {
    return build_result(0,"cant open socket with acq_chain");
  } 
  gws->acq_active=1;
  return build_result(1,"ok");
}

//pyrapi:stop_acq_sdhr: 
struct cmd_result *stop_acq_sdhr(struct cmd *command,void *workspace) {
  //struct sdhr_workspace *ws=(struct sdhr_workspace *)workspace;
  usleep(100000);
  gws->acq_active=0;
  return build_result(1,"ok");
}




