#include "fsmweb.hh"
#include <iostream>
#include <sstream>
#include <pyrame.h>

using namespace std;

class ecalWeb {
public:
  ecalWeb(string name,uint32_t port);
  void configure(levbdim::fsmmessage* m);
  void start(levbdim::fsmmessage* m);
  void stop(levbdim::fsmmessage* m);
  void destroy(Mongoose::Request &request, Mongoose::JsonResponse &response);
  void apply_sequence(vector<string> trans);
  bool isDone(){return done;}
  void currentspill(Mongoose::Request &request, Mongoose::JsonResponse &response);
private:
  fsmweb* _fsm;
  int rc_port;
  int varmod_port;
  int elog_port;
  string elog_id;
  string config_file;
  unsigned int run_nb;
  bool done;
  int retcode;
  string retstr;
  int current_spill;
  string run_name;
};



ecalWeb::ecalWeb(string name,uint32_t port) {

  struct cmd_result *cres;

  _fsm=new fsmweb(name);
  _fsm->addState("CREATED");
  _fsm->addState("CONFIGURED");
  _fsm->addState("RUNNING");
  _fsm->addTransition("CONFIGURE","CREATED","CONFIGURED",boost::bind(&ecalWeb::configure, this,_1));
  _fsm->addTransition("CONFIGURE","CONFIGURED","CONFIGURED",boost::bind(&ecalWeb::configure, this,_1));
  _fsm->addTransition("START","CONFIGURED","RUNNING",boost::bind(&ecalWeb::start, this,_1));
  _fsm->addTransition("STOP","RUNNING","CONFIGURED",boost::bind(&ecalWeb::stop, this,_1));

  _fsm->addCommand("CURRENTSPILL",boost::bind(&ecalWeb::currentspill,this,_1,_2));
  _fsm->addCommand("DESTROY",boost::bind(&ecalWeb::destroy,this,_1,_2));
  
  //initialize parameters
  run_nb=0;
  done=false;
  rc_port=get_port("RC_PORT");
  varmod_port=get_port("VARMOD_PORT");

  //init the elog system
  elog_id="none";
  elog_port=get_port("ELOG_PORT");
  cres=sendcmd("localhost",elog_port,"init_elog","llrelog.in2p3.fr","8080","calice","end");
  if (cres->retcode==0) {
    cout<<"no elog available"<<endl;
  } else {
    elog_id=cres->str;;
  }

  //Start server
  stringstream s0;
  s0.str(string());
  s0<<"ecalWeb-"<<name;
  _fsm->start(port);

  system("curl http://lyosdhcal9:45000/WDAQ/CMD?name=ECALPAUSE");
  system("/root/pyrame/rc/set_use_rundb.sh 0");

  cout<<"ecalWeb launched"<<endl;

}


void ecalWeb::apply_sequence(vector<string> trans) {
  struct cmd_result *cres;
  for (vector<string>::iterator it=trans.begin();it!=trans.end();++it) {
    cout<<"firing transition "<<it->c_str()<<endl;
    cres=sendcmd("localhost",rc_port,"transition_rc",it->c_str(),"undef","end");
    if (cres->retcode==0) {
      retcode=0;
      retstr=cres->str;
      return;
    } else {
      free_cmd_result(cres);
      cres=NULL;
    }
  }
  retcode=1;
  retstr="ok";
} 

void ecalWeb::configure(levbdim::fsmmessage* m) {

  Json::Value rep;
  struct cmd_result *cres;
  vector<string> trans;
  
  cout<<"Received "<<m->command()<<endl;
  cout<<"Received "<<m->value()<<endl;

  //getting parameters values
  Json::Value jc=m->content();
  if (!jc.isMember("configfile")) {
    rep["ECALSTATUS"]="No config file available for ecalWeb";
    m->setAnswer(rep);
    return;
  } else {
    config_file=jc["configfile"].asString();
  }
   cout<<"config file =  "<<config_file<<endl;


   //killing old pyrame
   system("/usr/local/bin/kill_pyrame.sh");
   ::sleep(1);

   //launching pyrame
   system("/usr/local/bin/autolauncher.py -n 2>&1 & > /dev/null");
   ::sleep(5);

  //getting the state machine to the right state
  string state="unknown";
  retcode=1;
  cres=sendcmd("localhost",rc_port,"get_state_rc","end");
  if (cres->retcode!=0) {
    state=cres->str;
    if (state=="ACQUIRING") {
      trans.clear();
      trans.push_back("stopacq");
      trans.push_back("inval");
      trans.push_back("deinit");
      apply_sequence(trans);
    }
    if (state=="CONFIGURED" || state=="RECONFIGURED") {
      trans.clear();
      trans.push_back("inval");
      trans.push_back("deinit");
      apply_sequence(trans);
    }
    if (state=="READY") {
      trans.clear();
      trans.push_back("deinit");
      apply_sequence(trans);
    }
    if (retcode==0) {
      rep["ECALSTATUS"]=retstr;
      m->setAnswer(rep);
      return;
    }
  }
  free_cmd_result(cres);
  cres=NULL;

  //load config file
  cres=sendcmd("localhost",rc_port,"load_config_file_rc",config_file.c_str(),"end");
  if (cres->retcode==0) {
    rep["ECALSTATUS"]=cres->str;
    m->setAnswer(rep);
    free_cmd_result(cres);
    return;
  } else {
    free_cmd_result(cres);
    cres=NULL;
  }
  
  //apply configuration
  trans.clear();
  trans.push_back("init");
  trans.push_back("config");
  apply_sequence(trans);
  if (retcode==0) {
    rep["ECALSTATUS"]=retstr;
    m->setAnswer(rep);
    return;
  }

  //set internal variables
  current_spill=0;

  //set varmod variable
  cres=sendcmd("localhost",varmod_port,"setvar_varmod","default","cur_spill","0","end"); 

  //set status ok for ecal
  rep["ECALSTATUS"]="OK";
  m->setAnswer(rep);
  return;
}
void ecalWeb::start(levbdim::fsmmessage* m) {

  Json::Value rep;
  struct cmd_result *cres;
  stringstream run_ss;
  stringstream title;
  FILE *f;

  cout<<"Received "<<m->command()<<endl;

  //get the parameters
  Json::Value jc=m->content();
  run_nb=jc["run"].asUInt();

  //name of the run
  run_ss<<"Combined_SDHCAL/run_"<<run_nb;
  run_name=run_ss.str();

  //starting the acquisition
  cres=sendcmd("localhost",rc_port,"transition_rc","start_acq","undef",run_name.c_str(),"end");
  if (cres->retcode==0) {
    rep["ECALSTATUS"]=cres->str;
    free_cmd_result(cres);
    m->setAnswer(rep);
    return;
  } else {
    free_cmd_result(cres);
  }

  system("curl http://lyosdhcal9:45000/WDAQ/CMD?name=ECALRESUME");


  //print the start run in the elog
  f=fopen("/tmp/elog_msg.txt","w");
  fprintf(f,"run %s started\n",run_name.c_str());
  fprintf(f,"Data available at /home/data/%s",run_name.c_str());
  fclose(f);
  title<<"run "<<run_name<<" started";
  cres=sendcmd("localhost",elog_port,"add_msg_file_elog",elog_id.c_str(),"Run",title.str().c_str(),"/tmp/elog_msg.txt","end");
  if (cres->retcode==0) {
    printf("error during elog connexion\n");
  }
  

  //free and send status
  rep["ECALSTATUS"]="OK";
  m->setAnswer(rep);
  return;
}


void ecalWeb::stop(levbdim::fsmmessage* m) {
  
  Json::Value rep;
  struct cmd_result *cres;
  stringstream title;
  FILE *f;
  
  cout<<"Received "<<m->command()<<endl;

  //get the parameters
  Json::Value jc=m->content();
 
  system("curl http://lyosdhcal9:45000/WDAQ/CMD?name=ECALPAUSE");

  //stopping the acquisition
  cres=sendcmd("localhost",rc_port,"transition_rc","stop_acq","undef","end");
  if (cres->retcode==0) {
    rep["ECALSTATUS"]=cres->str;
    free_cmd_result(cres);
    return;
  } else {
    free_cmd_result(cres);
  }

  //print the stop run in the elog
  f=fopen("/tmp/elog_msg.txt","w");
  fprintf(f,"run %s stopped\n",run_name.c_str());
  fprintf(f,"Data available at /home/data/%s",run_name.c_str());
  fclose(f);
  title<<"run "<<run_name<<" stopped";
  cres=sendcmd("localhost",elog_port,"add_msg_file_elog",elog_id.c_str(),"Run",title.str().c_str(),"/tmp/elog_msg.txt","end");
  if (cres->retcode==0) {
    printf("error during elog connexion\n");
  }

  //send status
  rep["ECALSTATUS"]="OK";
  m->setAnswer(rep);
  return;
}


void ecalWeb::destroy(Mongoose::Request &request, Mongoose::JsonResponse &response) {
  system("/usr/local/bin/kill_pyrame.sh");
  ::sleep(1);
  done=true;
  response["STATUS"]="DONE";
}


void ecalWeb::currentspill(Mongoose::Request &request, Mongoose::JsonResponse &response) {
  struct cmd_result *cres;
  //example of param grabing : 
  //uint32_t nlines=atol(request.get("lines","100").c_str());
  cres=sendcmd("localhost",varmod_port,"getvar_varmod","default","cur_spill","end");
  if (cres->retcode==1)
    current_spill=atoi(cres->str);
  else
    current_spill=0;
  free_cmd_result(cres);
  response["STATUS"]="DONE";
  response["CURRENTSPILL"]=current_spill;
}

int main() {
  //45000 is the port
  ecalWeb s("ecalWeb",45000);
  while (1) {
    ::sleep(1);
    if (s.isDone())
      break;
  }
  return 0;
}




